from abaqusGui import *
from abaqusConstants import *
import meshToolsIcons

# Cut mesh form and dialog
#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
class CutMeshForm(AFXForm):
    def __init__(self, owner):
        AFXForm.__init__(self, owner)

        self.fromModelTgt = AFXStringTarget(initialValue="")
        self.fromPartTgt = AFXStringTarget(initialValue="")
        self.toModelTgt = AFXStringTarget(initialValue="")
        self.toPartTgt = AFXStringTarget(initialValue="")
        
        self.point1Kw = AFXTupleKeyword(command=None, name='point1',
                                        minLength=3, maxLength=3,isRequired=TRUE)
        self.point2Kw = AFXTupleKeyword(command=None, name='point2',
                                        minLength=3, maxLength=3,isRequired=TRUE)
        self.point3Kw = AFXTupleKeyword(command=None, name='point3',
                                        minLength=3, maxLength=3,isRequired=TRUE)
                
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        return CutMeshDB(self)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):
        """check the data before sending it"""

        stdSubmitErrors = 'Error(s) in data:\n'
        submitErrors = stdSubmitErrors

        for tgt, err in ((self.fromModelTgt, 'From Model'),
                         (self.fromPartTgt, 'From Part'),
                         (self.toModelTgt, 'To Model'),
                         (self.toPartTgt, 'To Part')):
            if tgt.getValue() == '':
                submitErrors += '    %s must be defined.\n' % err

        from kernelAccess import mdb, session
        toModel = self.toModelTgt.getValue()
        if toModel != '':
            toModelParts = mdb.models[toModel].parts.keys()
            toPart = self.toPartTgt.getValue()
            if toPart in toModelParts:
                submitErrors += '    %s already exists in model %s. ' \
                                'A unique part name must be chosen.\n' % (toPart, toModel)
                
        for kw, err in ((self.point1Kw, 'Point 1'),
                        (self.point2Kw, 'Point 2'),
                        (self.point3Kw, 'Point 3')):

            try:
                for i in range(3):
                    val = float(kw.getValue(index=i))
            except:
                submitErrors += '    %s must be a 3-point tuple.\n' % err
                

        # Check if we have errors and post them
        #
        if submitErrors != stdSubmitErrors:
            db = self.getCurrentDialog()
            showAFXErrorDialog(db, submitErrors)
            return FALSE
        else:
            return TRUE

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getCommandString(self):
        """Derive the command string and return it"""
        
        cmd = "p = mdb.models['%s'].PartFromSection3DMeshByPlane(" % self.toModelTgt.getValue()
        cmd += "name='%s', " % self.toPartTgt.getValue()
        cmd += "part=mdb.models['%s'].parts['%s'], " % (self.fromModelTgt.getValue(),
                                                        self.fromPartTgt.getValue())
        cmd += "point1=%s, " % self.point1Kw.getValueAsString()
        cmd += "point2=%s, " % self.point2Kw.getValueAsString()
        cmd += "point3=%s)" % self.point3Kw.getValueAsString()

        # Display part
        #
        cmd += "\nsession.viewports['%s'].setValues(displayedObject=p)" % session.currentViewportName
                                                                            
        return cmd
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def handleException(self, exc):
        """Overload the exception handling method"""

        errorType = exc[0]
        errorValue = exc[1]
        db = self.getCurrentDialog()

        # Then post error dialog
        #
        showAFXErrorDialog(db, str(errorValue))
        
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
class CutMeshDB(AFXDataDialog):
    def __init__(self, form):
        self.form = form
        AFXDataDialog.__init__(self, form, 'Cut Mesh Tool', \
            self.OK|self.APPLY|self.CANCEL, DIALOG_ACTIONS_SEPARATOR)
        hf = FXHorizontalFrame(self, pl=0, pr=0, pt=0, pb=0, hs=0, vs=0)
        vf = FXVerticalFrame(p=hf)
        
        gb = FXGroupBox(p=vf, text='New Part', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        va = AFXVerticalAligner(p=gb, pr=0, pt=0, pb=0, hs=0, vs=0)
        self.toModelCb = AFXComboBox(va, 15, 1, 'Model:', form.toModelTgt, 0)
        AFXTextField(p=va, ncols=15, labelText='Part:', tgt=form.toPartTgt, sel=0)
        
        gb = FXGroupBox(p=vf, text='Existing Part', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        va = AFXVerticalAligner(p=gb, pr=0, pt=0, pb=0, hs=0, vs=0)        
        self.fromModelCb = AFXComboBox(va, 15, 1, 'Model:', form.fromModelTgt, 0)
        self.fromPartCb = AFXComboBox(va, 15, 1, 'Part:', form.fromPartTgt, 0)
        
        gb = FXGroupBox(p=vf, text='Cutting plane',
                        opts=FRAME_GROOVE|LAYOUT_FILL_X)
        va = AFXVerticalAligner(p=gb, pr=0, pt=0, pb=0, hs=0, vs=0)        
        AFXTextField(p=va, ncols=15, labelText='Point 1',
                     tgt=form.point1Kw, sel=0)
        AFXTextField(p=va, ncols=15, labelText='Point 2',
                     tgt=form.point2Kw, sel=0)
        AFXTextField(p=va, ncols=15, labelText='Point 3',
                     tgt=form.point3Kw, sel=0)

        vf = FXVerticalFrame(p=hf)
        cutIcon = FXXPMIcon(getAFXApp(), meshToolsIcons.cutIconData)
        FXLabel(p=vf, text='', ic=cutIcon)
        
        # reset available models
        self.availableModels = []
        self.availableFromParts = []
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def processUpdates(self):
        """Method to do some checks."""
        
        from kernelAccess import mdb, session

        availableModels = mdb.models.keys()
        availableModels.sort()
        # Only process updates if we have to
        #
        if availableModels != self.availableModels:
            currentFromModel = self.form.fromModelTgt.getValue()
            if currentFromModel not in availableModels:
                self.form.fromModelTgt.setValue('')
            currentToModel = self.form.toModelTgt.getValue()
            if currentToModel not in availableModels:
                self.form.toModelTgt.setValue('')
            # Clear and re-populate model combo boxes
            #
            self.fromModelCb.clearItems()
            self.toModelCb.clearItems()
            for modelName in availableModels:
                self.fromModelCb.appendItem(text=modelName)
                self.toModelCb.appendItem(text=modelName)
            self.fromModelCb.setNumVisible(nvis=max(1,len(availableModels)))
            self.toModelCb.setNumVisible(nvis=max(1,len(availableModels)))
            self.availableModels = availableModels[:]

        fromModelName = self.form.fromModelTgt.getValue()
        if fromModelName in self.availableModels:
            availableFromParts = mdb.models[fromModelName].parts.keys()
            availableFromParts.sort()
            if availableFromParts != self.availableFromParts:
                currentFromPart = self.form.fromPartTgt.getValue()
                if currentFromPart not in availableFromParts:
                    self.form.fromPartTgt.setValue('')
                # clear and re-populate part combo box
                self.fromPartCb.clearItems()
                for partName in availableFromParts:
                    self.fromPartCb.appendItem(text=partName)
                self.fromPartCb.setNumVisible(nvis=max(1,len(availableFromParts)))
                self.availableFromParts = availableFromParts[:]
                
# Extract perimeter form and dialog
#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
class ExtractPerimeterForm(AFXForm):
    def __init__(self, owner):
        AFXForm.__init__(self, owner)

        self.fromModelTgt = AFXStringTarget(initialValue="")
        self.fromPartTgt = AFXStringTarget(initialValue="")
        self.toModelTgt = AFXStringTarget(initialValue="")
        self.toPartTgt = AFXStringTarget(initialValue="")
        self.featureAngleTgt = AFXFloatTarget(initialValue=20.)
        self.splineCurvatureLimitTgt = AFXFloatTarget(initialValue=90.)
        
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        return ExtractPerimeterDB(self)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):
        """check the data before sending it"""

        stdSubmitErrors = 'Error(s) in data:\n'
        submitErrors = stdSubmitErrors

        for tgt, err in ((self.fromModelTgt, 'From Model'),
                         (self.fromPartTgt, 'From Part'),
                         (self.toModelTgt, 'To Model'),
                         (self.toPartTgt, 'To Part')):
            if tgt.getValue() == '':
                submitErrors += '    %s must be defined.\n' % err

        from kernelAccess import mdb, session
        toModel = self.toModelTgt.getValue()
        if toModel != '':
            toModelParts = mdb.models[toModel].parts.keys()
            toPart = self.toPartTgt.getValue()
            if toPart in toModelParts:
                submitErrors += '    %s already exists in model %s. ' \
                                'A unique part name must be chosen.\n' % (toPart, toModel)
                
        for tgt, err in ((self.featureAngleTgt, 'Feature Angle'),
                        (self.splineCurvatureLimitTgt, 'Spline Curvature Limit'),):

            try:
                val = float(tgt.getValue())
                if val <= 0.:
                    submitErrors += '    %s must be greater than 0.\n' % err
                if tgt.getValue() >= 1e38:
                    submitErrors += '    %s is not correctly defined.\n' % err
            except:
                submitErrors += '    %s could not be converted to a float.\n' % err
                

        # Check if we have errors and post them
        #
        if submitErrors != stdSubmitErrors:
            db = self.getCurrentDialog()
            showAFXErrorDialog(db, submitErrors)
            return FALSE
        else:
            return TRUE

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getCommandString(self):
        """Derive the command string and return it"""
        
        cmd = "p = mdb.models['%s'].Part2DGeomFrom2DMesh(" % self.toModelTgt.getValue()
        cmd += "name='%s', " % self.toPartTgt.getValue()
        cmd += "part=mdb.models['%s'].parts['%s'], " % (self.fromModelTgt.getValue(),
                                                        self.fromPartTgt.getValue())
        if self.splineCurvatureLimitTgt.getValue() != 90.:
            cmd += "featureAngle=%s, " % self.featureAngleTgt.getValue()
            cmd += "splineCurvatureLimit=%s)"  % self.splineCurvatureLimitTgt.getValue()
        else:
            cmd += "featureAngle=%s)" % self.featureAngleTgt.getValue()

        # Display part
        #
        cmd += "\nsession.viewports['%s'].setValues(displayedObject=p)" % session.currentViewportName
                                                                            
        return cmd
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def handleException(self, exc):
        """Overload the exception handling method"""

        errorType = exc[0]
        errorValue = exc[1]
        db = self.getCurrentDialog()

        # Then post error dialog
        #
        showAFXErrorDialog(db, str(errorValue))
        
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
class ExtractPerimeterDB(AFXDataDialog):
    def __init__(self, form):
        self.form = form
        AFXDataDialog.__init__(self, form, 'Perimeter Extraction Tool', \
            self.OK|self.APPLY|self.CANCEL, DIALOG_ACTIONS_SEPARATOR)
        hf = FXHorizontalFrame(self, pl=0, pr=0, pt=0, pb=0, hs=0, vs=0)
        vf = FXVerticalFrame(p=hf)
        
        gb = FXGroupBox(p=vf, text='New Part', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        va = AFXVerticalAligner(p=gb, pr=0, pt=0, pb=0, hs=0, vs=0)        
        self.toModelCb = AFXComboBox(va, 15, 1, 'Model:', form.toModelTgt, 0)
        AFXTextField(p=va, ncols=15, labelText='Part:', tgt=form.toPartTgt, sel=0)
        
        gb = FXGroupBox(p=vf, text='Existing Part', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        va = AFXVerticalAligner(p=gb, pr=0, pt=0, pb=0, hs=0, vs=0)        
        self.fromModelCb = AFXComboBox(va, 15, 1, 'Model:', form.fromModelTgt, 0)
        self.fromPartCb = AFXComboBox(va, 15, 1, 'Part:', form.fromPartTgt, 0)
        
        gb = FXGroupBox(p=vf, text='Parameters',
                        opts=FRAME_GROOVE|LAYOUT_FILL_X)
        va = AFXVerticalAligner(p=gb, pr=0, pt=0, pb=0, hs=0, vs=0)        
        AFXTextField(p=va, ncols=15, labelText='Feature Angle',
                     tgt=form.featureAngleTgt, sel=0, opts=AFXTEXTFIELD_FLOAT)
        AFXTextField(p=va, ncols=15, labelText='Spline Curvature Limit',
                     tgt=form.splineCurvatureLimitTgt, sel=0, opts=AFXTEXTFIELD_FLOAT)

        # Reset available models
        #
        self.availableModels = []
        self.availableFromParts = []
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def processUpdates(self):
        """Method to do some checks."""
        
        from kernelAccess import mdb, session

        availableModels = mdb.models.keys()
        availableModels.sort()
        # Only process updates if we have to
        #
        if availableModels != self.availableModels:
            currentFromModel = self.form.fromModelTgt.getValue()
            if currentFromModel not in availableModels:
                self.form.fromModelTgt.setValue('')
            currentToModel = self.form.toModelTgt.getValue()
            if currentToModel not in availableModels:
                self.form.toModelTgt.setValue('')
            # Clear and re-populate model combo boxes
            #
            self.fromModelCb.clearItems()
            self.toModelCb.clearItems()
            for modelName in availableModels:
                self.fromModelCb.appendItem(text=modelName)
                self.toModelCb.appendItem(text=modelName)
            self.fromModelCb.setNumVisible(nvis=max(1,len(availableModels)))
            self.toModelCb.setNumVisible(nvis=max(1,len(availableModels)))
            self.availableModels = availableModels[:]

        fromModelName = self.form.fromModelTgt.getValue()
        if fromModelName in self.availableModels:
            availableFromParts = mdb.models[fromModelName].parts.keys()
            availableFromParts.sort()
            if availableFromParts != self.availableFromParts:
                currentFromPart = self.form.fromPartTgt.getValue()
                if currentFromPart not in availableFromParts:
                    self.form.fromPartTgt.setValue('')
                # Clear and re-populate part combo box
                #
                self.fromPartCb.clearItems()
                for partName in availableFromParts:
                    self.fromPartCb.appendItem(text=partName)
                self.fromPartCb.setNumVisible(nvis=max(1,len(availableFromParts)))
                self.availableFromParts = availableFromParts[:]

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

import os, uti

majorNumber, minorNumber, updateNumber = getAFXApp().getBaseVersionNumbers()
verAbaqus = majorNumber * 1.0 + minorNumber * 0.1
if verAbaqus > 6.6:
    import _abqPluginUtils

# Register commands
#
version = '1.1-1'
if verAbaqus < 6.6:
    status = "Plug-in " + str(version) + " version was not registered because "\
             "it is not supported in Abaqus version " + str(uti.getVersion())
elif verAbaqus == 6.6:
    status = 'OK'
else:
    status = _abqPluginUtils.checkCompatibility('Extracting Two-Dimensional Part', version)
if status == 'OK':
    toolset=getAFXApp().getAFXMainWindow().getPluginToolset()
    toolset.registerGuiMenuButton(
        object=CutMeshForm(toolset),
        buttonText='Tools|Cut Mesh...',
        messageId=AFXMode.ID_ACTIVATE,
        icon=None,
        kernelInitString='',
        applicableModules=['Part', 'Property', 'Assembly', 'Step', 'Interaction', 'Load', 'Mesh', 'Job', 'Sketch',],
        version=version,
        author='Dassault Systemes Simulia Corp.',
        description='Tool to extract the perimeter of a 2D mesh.\n' \
            'Makes use of the PartFromSection3DMeshByPlane kernel command.',
        helpUrl='http://abaqus.custhelp.com/cgi-bin/abaqus.cfg'
                '/php/enduser/std_adp.php?p_faqid=2414')
    toolset.registerGuiMenuButton(
        object=ExtractPerimeterForm(toolset),
        buttonText='Tools|Extract Perimeter...',
        messageId=AFXMode.ID_ACTIVATE,
        icon=None,
        kernelInitString='',
        applicableModules=['Part', 'Property', 'Assembly', 'Step', 'Interaction', 'Load', 'Mesh', 'Job', 'Sketch',],
        version=version,
        author='Dassault Systemes Simulia Corp.',
        description='Tool to create 2D mesh by slicing existing 3D mesh.\n' \
            'Makes use of the PartFromSection3DMeshByPlane kernel command.',
        helpUrl='http://abaqus.custhelp.com/cgi-bin/abaqus.cfg'
                '/php/enduser/std_adp.php?p_faqid=2414')
else:
    getAFXApp().getAFXMainWindow().writeToMessageArea(status)
    getAFXApp().beep()