"""
Module that implements utility functions for the main modules.
"""
from odbAccess import OdbType, OdbStepType
from computeContributionsConstants import *

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def getComplexFrequencyStep(odb):
    # Method that returns the complex frequency step from the odb
    # Returns None if no complex frequency step is found.
    #
    if type(odb) != OdbType:
        raise TypeError, 'odb must be of type OdbType; got %s.' % type(odb)

    for step in odb.steps.values():
        if step.procedure == '*COMPLEX FREQUENCY':
            return step

    return None

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
#
def getUnstableModes(cfStep):

    # Method that returns a list of unstable modes from the complex
    # frequency step cfStep.
    #
    if type(cfStep) != OdbStepType:
        raise TypeError, 'cfStep must be of type OdbStepType; got %s.' % type(cfStep)

    if cfStep.procedure != '*COMPLEX FREQUENCY':
        raise ValueError, 'cfStep is not a *COMPLEX FREQUENCY step.' 
    
    unstableModes = []

    index = 0
    for frame in cfStep.frames:
        if index ==0:
            index += 1
            continue
        desc = frame.description
        realPart, frequency = _parseDescription(desc)            

        if realPart >= TOLERANCE:
            unstableModes.append(index)

        index += 1

    return unstableModes

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def _parseDescription(description):

    # Utility method to parse the frame description.
    #

    # description: Frame description
    # example:
    # Returns: realPart complex mode, frequency in cycles/time
    #
    temp = description.split('=')
    realPart = float(temp[-1])
    frequency = float(temp[1].split('(cycles/time)')[0])

    return realPart, frequency

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def getComponentsDict(odb):

    # Method to return a dictionay of componentNames as keys and 
    # corresponding objects as values
    #
    if type(odb) != OdbType:
        raise TypeError, 'odb must be of type OdbType; got %s.' % type(odb)
   
    componentsDict = {} 
    
    instances = odb.rootAssembly.instances.values()

    # Search for component node sets
    #
    if len(instances) < 2: 
        
        componentSpecification = 'NODE_SET_BASED'

        instance = instances[0] # the only instance available
        for nodeSet in instance.nodeSets.values():
            if nodeSet.name.startswith(COMP_NSET_START_STRING):
                componentsDict[nodeSet.name] = nodeSet                      

    else:
        componentSpecification = 'INSTANCE_BASED'
        for instance in instances:
            if instance.name != 'ASSEMBLY':
                componentsDict[instance.name] = instance

    return componentsDict

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def getComponentSpecification(odb):

    # Method to return the component specification.
    # Returns: 'NODE_SET_BASED' if the odb has node set based components.
    # else returns 'INSTANCE_BASED'.
    #
    if type(odb) != OdbType:
        raise TypeError, 'odb must be of type OdbType; got %s.' % type(odb)
   
    componentsDict = {} 
    
    instances = odb.rootAssembly.instances.values()

    # Search for component node sets
    #
    if len(instances) < 2:
        
        componentSpecification = 'NODE_SET_BASED'
    else:
        componentSpecification = 'INSTANCE_BASED'
    
    return componentSpecification