from abaqusGui import *
from abaqusConstants import *
from abq_BrakeNoise.computeContributionsConstants import *
from abq_BrakeNoise.computeContributionsDB import ComputeContributionsDB
import os

###########################################################################
# Class definition
###########################################################################

class ComputeContributionsForm(AFXForm):
    [
        ID_OVERWRITE,
        ID_LAST
    ] = range(AFXForm.ID_LAST, AFXForm.ID_LAST+2)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):

        # Construct the base class
        #
        AFXForm.__init__(self, owner)
        
        self.firstDb = None
        self.secondDb = None
        
        FXMAPFUNC(self, SEL_COMMAND, self.ID_OVERWRITE, ComputeContributionsForm.onCmdOverwrite)
                  
        # Command to openOdb
        #            
        self.cmd1 = AFXGuiCommand(self, 'openOdb', 'cfOdb = abq_BrakeNoise.computeContributionsModule')
        self.odbPathKw = AFXStringKeyword(self.cmd1, 'odbPath', TRUE)
        
        # Command to compute CCF
        #        
        self.cmd2 = AFXGuiCommand(self, 'computeCcf', 'abq_BrakeNoise.computeContributionsModule')
        self.ccfOdbKw = AFXObjectKeyword(self.cmd2, 'odb', TRUE, 'cfOdb')
        self.ccfUnstableModesKw = AFXTupleKeyword(self.cmd2, 'unstableModes', TRUE)
        self.componentNamesKw = AFXTupleKeyword(self.cmd2, 'componentNames', TRUE)
        self.ccfOutputFileNameKw = AFXStringKeyword(self.cmd2, 'outputFileName', TRUE)
        
        # Command to compute CMCF
        #
        self.cmd3 = AFXGuiCommand(self, 'computeCmcf', 'abq_BrakeNoise.computeContributionsModule')
        self.cmcfOdbKw = AFXObjectKeyword(self.cmd3, 'odb', TRUE, 'cfOdb')
        self.cmcfUnstableModesKw = AFXTupleKeyword(self.cmd3, 'unstableModes', TRUE)
        self.componentOdbsDictKw = AFXObjectKeyword(self.cmd3, 'componentOdbsDict', TRUE, 'componentOdbsDict')
        self.cmcfOutputFileNameKw = AFXStringKeyword(self.cmd3, 'outputFileName', TRUE)
        
        # Temporary targets
        #
        self.computeTgt = AFXIntTarget(CCF.getId())    
        self.umSelKw = AFXTupleKeyword(None,'selected',FALSE)
        self.outputFileNameTgt = AFXStringTarget()
        self.componentOdbPathTgt = AFXStringTarget() 
                
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        # Show the openOdb dialog
        #
        from abq_BrakeNoise.openCfOdbDB import OpenOdbDB
        self.firstDb = OpenOdbDB(self)
        return self.firstDb
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyCurrentKeywordValues(self):

        # Method to check if the odbName is valid
        # 
        currentDb = self.getCurrentDialog()
        
        if currentDb == self.firstDb:
            if self.odbPathKw.getValue().strip() == '':
                showAFXErrorDialog(currentDb,'ODB file name must be specified.')
                return FALSE
            
        return TRUE
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):

        # Method that issues the commands between the dialogs
        #
        currentDb = self.getCurrentDialog()
        
        if currentDb == self.firstDb:        
            # Issue the openOdb command before opening the dialog
            #
            self.cmd1.activate()
            self.cmd2.deactivate()
            self.cmd3.deactivate()
            status = self.issueCommands()
            if not status:
                return FALSE
        elif currentDb == self.secondDb:
            # Check if the output file already exists
            #            
            fileName = self.outputFileNameTgt.getValue()

            # Get file name with extension
            #
            fileNameWithExt = fileName

            if not fileName[-4:] == '.ccf':
                fileNameWithExt = '%s.ccf' % fileName
          
            if os.path.isfile(fileNameWithExt):
                errMsg = 'A results file with name "%s" already exists.\n\n' % fileNameWithExt        
                errMsg += 'OK to overwrite?'        
                showAFXWarningDialog(currentDb, errMsg,AFXDialog.YES|AFXDialog.NO, 
                                     self, self.ID_OVERWRITE)
                return FALSE
        
        return TRUE
                
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getNextDialog(self,previousDb):
        # Show the compute contributions dialog
        #
        if previousDb == self.firstDb:
            # Now open the compute contributions dialog
            #
            self.secondDb = ComputeContributionsDB(self)
            return self.secondDb
        else:
            return None

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        currentDb = self.getCurrentDialog()

        outputFileName = self.outputFileNameTgt.getValue()
        
        if not outputFileName.strip():
            showAFXErrorDialog(currentDb, 'Output file name must be specified.')
            return FALSE

        db = self.secondDb
        table = db.table
        rows = table.getNumRows()
        cols = table.getNumColumns()

        if self.computeTgt.getValue() == CCF.getId():
            # Set ccf as the command and deactivate the other command
            #
            self.cmd1.deactivate()
            self.cmd2.activate()
            self.cmd3.deactivate()
            self.ccfOutputFileNameKw.setValue(outputFileName)
            self.componentNamesKw.setValues('')
            self.ccfUnstableModesKw.setValues('')

        else:
            # Set cmcf as the command and deactivate the other command
            #
            self.cmd1.deactivate()
            self.cmd2.deactivate()
            self.cmd3.activate()
            self.cmcfOutputFileNameKw.setValue(outputFileName)
            self.cmcfUnstableModesKw.setValues('')
        
        componentsDict = {}
        
        for row in range(1,rows):
            isSelected = table.getItemBoolValue(row, 0)

            if isSelected:
                cName = table.getItemText(row,1)
                odbName = table.getItemText(row,2)
                
                componentsDict[cName] = odbName

        if not componentsDict:
            errMsg = 'At least one component must be selected.'
            showAFXErrorDialog(currentDb, errMsg)
            return FALSE

        if self.computeTgt.getValue() == CCF.getId():
            index = 0
            for cName in componentsDict.keys():
                self.componentNamesKw.setValue(index,cName)
                self.componentNamesKw.setElementType(index,AFXTUPLE_TYPE_STRING)
                index += 1

        else:
            # Form the dictionary
            #
            cmds = 'componentOdbsDict = {\n'

            for cName, odbName in componentsDict.items():
                cmds += '"%s":"%s",\n' % (cName, odbName)

            cmds += '}\n'

            self.componentOdbsDictKw.setSetupCommands(cmds)

        # Collect the unstable modes
        #
        numItems = db.selectionList.getNumItems()

        if not numItems:
            errMsg = 'At least one unstable mode must be selected.'
            showAFXErrorDialog(currentDb, errMsg)
            return FALSE

        for i in range(numItems):
            mode = db.selectionList.getItemText(i)
            if self.computeTgt.getValue() == CCF.getId():
                self.ccfUnstableModesKw.setValue(i,mode)
                self.ccfUnstableModesKw.setElementType(i,AFXTUPLE_TYPE_INT)
            else:
                self.cmcfUnstableModesKw.setValue(i,mode)
                self.cmcfUnstableModesKw.setElementType(i,AFXTUPLE_TYPE_INT)

        return TRUE

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdOverwrite(self, sender, sel, ptr):
        if sender.getPressedButtonId() == AFXDialog.ID_CLICKED_YES:
                AFXForm.issueCommands(self)
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomTasks(self):

        # Write Success message
        #
        currentDb = self.getCurrentDialog()

        if currentDb == self.firstDb: 
            return

        fileName = self.outputFileNameTgt.getValue()

        # Get file name with extension
        #
        fileNameWithExt = fileName

        if not fileName[-4:] == '.ccf':
            fileNameWithExt = '%s.ccf' % fileName   

        msg = 'The contributions have been written to "%s".' % fileNameWithExt
        getAFXApp().getAFXMainWindow().writeToMessageArea(msg)