"""
Module to compute the component contribution factors (CCF) and
component mode contribution factors (CMCF).This module can be used only 
from within visualization module of ABAQUS/CAE.
"""

from abaqus import *
from abaqusConstants import *
import abq_BrakeNoise.ccfutils as ccfutils
import customKernel
import os, types
from cPickle import Pickler
from odbAccess import OdbType
from abq_BrakeNoise.computeContributionsConstants import *

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def computeCcf(odb, unstableModes, componentNames, outputFileName):
    """   
    Method to compute the component contribution factors.
    
    Required arguments:
    
    odb: ODB object
    unstableModes: A sequence of unstable modes.
    componentNames: A sequence of component names.
    outputFileName: The file name to which results are written to.    
    
    WARNING: The file outputFileName will be overwritten if it already exists.
    
    Return None.
    """
    
    # Check args
    #
    cfStep = _checkOdb(odb)
    unstableModes = _checkUnstableModes(unstableModes, odb, cfStep)
    components = _checkComponentNames(componentNames, odb)
    outputFileName = _checkOutputFileName(outputFileName)
    
    # Open the file for pickling
    #   
    outFile = open(outputFileName, 'w')
    pfile = Pickler(outFile) 
        
    # The data structures for storing the information
    #

    # This is used to store the data and this object will be dumped 
    # to the pfile
    #
    pickleDict = {}

    # This stores the mode numbers as keys 
    #  and the corresponding frequency as values
    #
    unstableModesDict = {}

    # This stores the componentNames as a list
    #
    componentNamesForCCF = componentNames

    # This stores the actual contributions with unstable modes as keys
    #
    ccfDict = {}
    
    # Loop through the unstable modes
    #
    for mode in unstableModes:
        frame = cfStep.frames[mode]
        realPart, frequency = ccfutils._parseDescription(frame.description)
                
        # Compute the total phi
        #
        ufo = frame.fieldOutputs['U']
        #sum = _getPhi(ufo) # phi squared -> sum of components

        # Compute weighting factors sum for all components
        #
        weightedComponentsDict = _getSumWeightedComponents(odb, ufo)

        # Now grab CCF for each component selected
        #
        componentCCFResultsDict = {}
        for component in components:
            msg = 'Processing CCF for Mode: %d, Component: %s...' % (mode, component.name)
            milestone(msg)
            componentCCFResultsDict[component.name] = weightedComponentsDict[component.name]


        unstableModesDict[mode] = frequency
        ccfDict[mode] = componentCCFResultsDict


    pickleDict['complexFrequencyODBName'] = odb.path
    pickleDict['unstableModesDict'] = unstableModesDict
    pickleDict['componentNamesForCCF'] = componentNamesForCCF
    pickleDict['ccfDict'] = ccfDict
    pickleDict['componentNamesForCMCF'] = []
    pickleDict['cmcfDict'] = {}

    pfile.dump(pickleDict)
    outFile.close()

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def computeCmcf(odb, unstableModes, componentOdbsDict, outputFileName):
    """
    Method to compute the component mode contribution factors.
    
    Required arguments:
    
    odb: ODB object
    unstableModes: A sequence of unstable modes.
    componentOdbsDict: A dictionary of component names as keys and 
                       corresponding *FREQUENCY ODB names as values.
    outputFileName: The file name to which results are written to.    
    
    WARNING: The file outputFileName will be overwritten if it already exists.
    
    Return None.
    """
     
    # Check args
    #
    cfStep = _checkOdb(odb)
    unstableModes = _checkUnstableModes(unstableModes, odb, cfStep)
    components, componentOdbsObjDict = _checkComponentOdbsDict(componentOdbsDict, odb)
    outputFileName = _checkOutputFileName(outputFileName)
        
    # Open the file for pickling
    #
    outFile = open(outputFileName, 'w')
    pfile = Pickler(outFile) 
        
    # The data structures for storing the information
    #

    # This is used to store the data and this object will be dumped 
    # to the pfile
    #
    pickleDict = {} 

    # This stores the mode numbers as keys 
    # and the corresponding frequency as values
    #
    unstableModesDict = {}

    # This stores the componentNames as a list for the CCF
    #
    componentNamesForCCF = componentOdbsDict.keys()

    # This stores the componentNames as a list for the CMCF
    #
    componentNamesForCMCF = []

    # This stores the actual ccf contributions with unstable modes as keys
    #
    ccfDict = {}

    # This stores the actual cmcf contributions with unstable modes as keys
    #
    cmcfDict = {}
        
    atleastOneCMCF = FALSE
    # Loop through the unstable modes
    #
    for mode in unstableModes:
        frame = cfStep.frames[mode]
        realPart, frequency = ccfutils._parseDescription(frame.description)
                
        # Compute the total phi
        #
        ufo = frame.fieldOutputs['U']
        sum = _getPhi(ufo)
                        
        componentCCFResultsDict = {}
        componentCMCFResultsDict = {}
        
        # Now compute the CCF for each component
        #
        for component in components:
            msg = 'Processing CCF for Mode: %d, Component: %s...' % (mode, component.name)
            milestone(msg)
            
            compUfo = ufo.getSubset(region=component)
            compOdb, freqStepName, realCompObj = componentOdbsObjDict[component.name]
            
            if not compOdb:     
                compSum = _getPhi(compUfo)
            else:
                compSum, nodeLabels, realData, imagData = _getPhiAndMag2Data(compUfo)
            
            contribution = compSum*100.0/sum
            componentCCFResultsDict[component.name] = contribution
            
            # Compute CMCF
            #
            if not compOdb:
                continue
            
            numValsInComponent = len(realData)
            frequencyCMCFDict = {}
            freqIndex = 0
            freqFrames = compOdb.steps[freqStepName].frames
            
            for freqFrame in freqFrames:
                msg = 'Processing CMCF for Mode: %d, Component: %s, Frequency Frame: %d...' % (mode, component.name, freqIndex)
                milestone(msg)
                u = freqFrame.fieldOutputs['U'].getSubset(region=realCompObj)
                
                freqValues = u.values   
                
                if len(freqValues) != numValsInComponent:
                    msg = 'The number of result values between the frequency ODB \n"%s"\n' % compOdb.name
                    msg += 'and component in the complex frequency ODB \n"%s"\n' % odb.name
                    msg += 'do not match.'
                    raise ValueError, msg
                
                nodeIndex = 0
                cmcf = 0
                rsum = 0
                isum = 0
                realFreqMag = 0
                for valIndex in range(numValsInComponent):
                    freqVal = freqValues[nodeIndex] 

                    # Check for the equality of node number
                    # 
                    if freqVal.nodeLabel != nodeLabels[nodeIndex]:
                        msg = 'Node labels in the complex frequency ODB: "%s"\n' % odb.name
                        msg += 'and real frequency ODB: "%s"\n' % compOdb.name
                        msg += 'do not match.' 
                        raise ValueError, msg
                    
                    
                    if freqVal.precision == SINGLE_PRECISION:
                        data = freqVal.data
                    else:
                        data = freqVal.dataDouble
                    
                    tempR = realData[valIndex]
                    tempI = imagData[valIndex]

                    # Now take the dot product
                    # 
                    for i in range(len(tempR)): 
                        rsum += tempR[i]*data[i]
                        isum += tempI[i]*data[i]
                        realFreqMag += data[i]*data[i]
                        
                    nodeIndex += 1
                
                # Store the cmcf in the dictionary
                #
                numerator = (rsum*rsum) + (isum*isum)
                
                denominator = (compSum)*(realFreqMag)
                
                if denominator:
                    frequencyCMCFDict[freqIndex] = numerator/denominator
                else:
                    frequencyCMCFDict[freqIndex] = 0
                freqIndex += 1
            
            # End of processing frequencies
            #
            if frequencyCMCFDict:
                atleastOneCMCF = TRUE
                componentCMCFResultsDict[component.name] = frequencyCMCFDict
        
        # End of processing components
        #            
        unstableModesDict[mode] = frequency
        ccfDict[mode] = componentCCFResultsDict
        cmcfDict[mode] = componentCMCFResultsDict
        componentNamesForCMCF = componentCMCFResultsDict.keys()
    
    # End of processing modes
    #
    pickleDict['complexFrequencyODBName'] = odb.path
    pickleDict['unstableModesDict'] = unstableModesDict
    pickleDict['componentNamesForCCF'] = componentNamesForCCF
    pickleDict['componentNamesForCMCF'] = componentNamesForCMCF
    pickleDict['ccfDict'] = ccfDict
    pickleDict['cmcfDict'] = cmcfDict

    pfile.dump(pickleDict)
    outFile.close()

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def openOdb(odbPath):
    """
    Method to open a *COMPLEX frequency ODB and store component names and 
    unstable modes in the session.customData object.
    Required arguments:
    odbPath: The path to the *COMPLEX FREQUENCY ODB
    
    Returns the odb object.
    """
    
    # Check the arg type
    #
    if type(odbPath) != types.StringType:
        raise TypeError, 'odbPath must be a string; got %s.' % type(odbPath)
    
    if odbPath.strip() == '':
        raise ValueError, 'odbPath cannot be an empty string.'

    vp = session.viewports[session.currentViewportName]
    odb = session.openOdb(odbPath)
    cfStep = ccfutils.getComplexFrequencyStep(odb)
    
    if not cfStep:
        raise ValueError, 'The ODB "%s" does not contain a *COMPLEX FREQUENCY step.' % odbPath
    
    session.customData.unstableModes = ccfutils.getUnstableModes(cfStep)
    
    if len(session.customData.unstableModes) == 0:
        raise ValueError, 'The ODB "%s" does not contain any unstable modes.' % odbPath
    
    componentsDict = ccfutils.getComponentsDict(odb)
    
    session.customData.componentNames = componentsDict.keys()
    
    if len(session.customData.componentNames) == 0:
        raise ValueError, 'The ODB "%s" doe not contain any components.' % odbPath
    
    return odb

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def _checkComponentNames(componentNames, odb):

    # Private method to check the validity of component names
    # Returns a sorted list of component objects
    #
    if type(componentNames) != types.ListType and type(componentNames) != types.TupleType:
        raise TypeError, 'componentNames must be a list or a tuple; got %s/' % type(componentNames)
    
    if not componentNames:
        raise ValueError, 'componentNames cannot be an empty sequence.'
    
    for cName in componentNames:
        if type(cName) != types.StringType:
            raise TypeError, 'Items in componentNames must be strings; got %s.' % type(cName)

    allComponentsDict = ccfutils.getComponentsDict(odb)    
    allComponentNames = allComponentsDict.keys()
    invalidCN = []
    for cn in componentNames:
        if cn not in allComponentNames:
            invalidCN.append(cn)
    
    if invalidCN:
        raise ValueError, 'The following components are invalid. \n%s' % invalidCN
    
    componentNames = list(componentNames) # Make sure it is of type list before sorting
    componentNames.sort() # I do not know if this is required or not. For now sorting it.
    
    components = []
    
    for cn in componentNames:
        components.append(allComponentsDict[cn])
    
    return components

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def _checkComponentOdbsDict(componentOdbsDict, odb):
    """
    Private method to check the validity of the componentOdbsDict in 
    the odb. 
    Returns a dictionary of componentNames as keys and corresponding ODB object, 
    frequency step name, realComponent region object as values. If it is an empty string then the ODB objects are None.
    """
    if type(componentOdbsDict) != types.DictType:
        raise TypeError, 'componentOdbsDict must be a dictionary; got %s.' % type(componentOdbsDict)
    
    if not componentOdbsDict:
        raise ValueError, 'componentOdbsDict cannot be an empty dictionary.'
    
    for key, value in componentOdbsDict.items():
        if type(key) != types.StringType:
            raise TypeError, 'Keys in componentOdbsDict must be strings; got %s.' % type(key)
        if type(value) != types.StringType:
            raise TypeError, 'Values in componentOdbsDict must be strings; got %s.' % type(value)
        
    componentNames = componentOdbsDict.keys()    
    components = _checkComponentNames(componentNames, odb)
    componentSpecification = ccfutils.getComponentSpecification(odb)
    
    componentOdbsObjDict = {}
    # Check if the ODBs exist and if they have *frequency steps
    #
    for cName in componentNames:
        fodbName = componentOdbsDict[cName]
        if fodbName == '':
            componentOdbsObjDict[cName] = (None, None, None)
            continue
        # This will throw the appropriate error message
        #
        fodb = session.openOdb(fodbName)
        for step in fodb.steps.values():
            if step.procedure == '*FREQUENCY':
                break
        else:
            raise ValueError, 'The component ODB "%s" does not have a frequency step.' % fodbName
        
        # Get the component objects from the * frequency ODBS
        #
        if componentSpecification == 'NODE_SET_BASED':
            instance = fodb.rootAssembly.instances.values()[0]
            if not instance.nodeSets.has_key(cName):
                raise ValueError, 'There is no node set named "%s" in ODB "%s".' % (cName, fodbName)
            
            realComponent = instance.nodeSets[cName]
            
        else:
            if not fodb.rootAssembly.instances.has_key(cName):
                raise ValueError, 'There is no instance named "%s" in ODB "%s".' % (cName, fodbName)
            
            realComponent = fodb.rootAssembly.instances[cName]

        componentOdbsObjDict[cName] = (fodb, step.name, realComponent)
    
    return components, componentOdbsObjDict

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def _checkOdb(odb):

    # Private method to check the validity of odb
    # Returns complex frequency step
    #
    if type(odb) != OdbType:
        raise TypeError, 'odb must be of type OdbType; got %s.' % type(odb)
        
    cfStep = ccfutils.getComplexFrequencyStep(odb)
    
    if not cfStep:
        raise ValueError, 'The ODB "%s" does not contain a *COMPLEX FREQUENCY step.' % odb.name
   
    return cfStep

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def _checkOutputFileName(outputFileName):

    # Private method to check the validity of outputFileName.
    #
    if type(outputFileName) != types.StringType:
        raise TypeError, 'outputFileName must be a string; got %s.' % type(outputFileName)

    if not outputFileName.strip():
        raise ValueError, 'outputFileName cannot be an empty string.'

    # Check the extension
    #
    if not outputFileName.endswith(FILE_EXTENSION):
        outputFileName += FILE_EXTENSION

    return outputFileName

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def _checkUnstableModes(unstableModes, odb, cfStep):

    # Private method to check the validity of unstable modes
    # Returns a sorted list of unstableModes.
    #
    if type(unstableModes) != types.ListType and type(unstableModes) != types.TupleType:
        raise TypeError, 'unstableModes must be a list or a tuple; got %s/' % type(unstableModes)
    if not unstableModes:
        raise ValueError, 'unstableModes cannot be an empty sequence.'
    
    for mode in unstableModes:
        if type(mode) != types.IntType:
            raise TypeError, 'Items in unstableModes must be integers; got %s.' % type(mode)
    
    allUnstableModes = ccfutils.getUnstableModes(cfStep)
    
    if len(allUnstableModes) == 0:
        raise ValueError, 'The ODB "%s" does not contain any unstable modes.' % odb.name    
    invalidUM = []
    
    for mode in unstableModes:
        if mode not in allUnstableModes:
            invalidUM.append(mode)
    if invalidUM:
        raise ValueError, 'The following modes specified are not unstable.\n%s.' % invalidUM
    
    unstableModes = list(unstableModes) # Make sure it is of type list before sorting
    
    unstableModes.sort() # I do not know if this is required or not. For now sorting it.
    
    return unstableModes

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def _getPhi(fo):

    # Private method that loops through the field values of field output fo
    # and computes the PHI.
    # Returns the total PHI.
    #
    sum = 0
    for val in fo.values:
        if val.precision == SINGLE_PRECISION:
            rData = val.data
            iData = val.conjugateData
        else:
            rData = val.dataDouble
            iData = val.conjugateDataDouble

        for i in range(len(rData)): # loop over non sequence can happen
            mag2 = (rData[i]**2) + (iData[i]**2)
            sum += mag2
    
    return sum

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def _getPhiAndMag2Data(fo):

    # Private method that loops through the field values of field output fo
    # and computes the PHI.
    # Returns the total PHI, nodeLabels and the stored mag2 values.
    #
    realData = []
    imagData = []
    sum = 0
    nodeLabels = []
    for val in fo.values:
        if val.precision == SINGLE_PRECISION:
            rData = val.data
            iData = val.conjugateData
        else:
            rData = val.dataDouble
            iData = val.conjugateDataDouble
        
        nodeLabels.append(val.nodeLabel)
        tempR = []
        tempI = []

        # Loop over non sequence can happen
        #
        for i in range(len(rData)):
            mag2 = (rData[i]**2) + (iData[i]**2)
            tempR.append(rData[i])
            tempI.append(iData[i])
            sum += mag2
        
        realData.append(tempR)
        imagData.append(tempI)
    
    return sum, nodeLabels, realData, imagData

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def _getSumWeightedComponents(odb, ufo):

    # Private method to find a weighting factors for every component
    # Returns dictionary with component as keys and normalized 
    # weighted contribution as values
    #
    componentsDict = ccfutils.getComponentsDict(odb)
    componentWeightedFactorsDict = {}
    componentSpecification = ccfutils.getComponentSpecification(odb)

    # Check if every history output is available
    #
    stepName = odb.steps.values()[0].name
    found = 1
    
    if stepName == '':
        found = 0
    else:
         for componentName in componentsDict.keys():
             hrName = ''

             if componentSpecification == 'NODE_SET_BASED':
                 HRegionKeys = odb.steps[stepName].historyRegions.keys()

                 for RegionName in HRegionKeys:
                     if componentName in RegionName:                         
                         hrName = RegionName                         
             else:
                 # For instances, loop through each elset
                 #                    
                 for elsetName in odb.rootAssembly.instances[componentName].elementSets.keys():
                     if elsetName.startswith(COMP_NSET_START_STRING):
                         hrName = 'ElementSet ' + elsetName
             
             if hrName not in odb.steps[stepName].historyRegions.keys():
                 found = 0
             else:
                 if not odb.steps[stepName].historyRegions[hrName].historyOutputs.has_key('VOL'):
                     found = 0
         
    if not found:        
        print 'Unable to find volume for one or more components. Weigthing scale factor not applied.'

    
    # Populate dictionary with component weighted (if needed) contribution 
    # and keep tab of sum
    #
    weightedSum = 0
    for component in componentsDict.values():
        weightFactor = 1.0
        componentName=component.name
        if found:
                        
            if componentSpecification == 'NODE_SET_BASED':
                HRegionKeys = odb.steps[stepName].historyRegions.keys()
                for RegionName in HRegionKeys:
                     if componentName in RegionName:                             
                         hrName = RegionName
                         break
            else:
                # For instances, loop through each elset
                #
                for elsetName in odb.rootAssembly.instances[componentName].elementSets.keys():
                    if elsetName.startswith(COMP_NSET_START_STRING):
                        hrName = 'ElementSet ' + elsetName

            weightFactor = odb.steps[stepName].historyRegions[
                hrName].historyOutputs['VOL'].data[0][1] / len(component.nodes)
        compUfo = ufo.getSubset(region=component)
        compSum = _getPhi(compUfo)
        componentWeightedFactorsDict[component.name] = weightFactor * compSum
        weightedSum += weightFactor * compSum
    weightedSum /= 100

    # Normalize
    #
    for componentName in componentWeightedFactorsDict.keys():
        componentWeightedFactorsDict[componentName] /= weightedSum

    return componentWeightedFactorsDict
