"""
Module to plot the component contribution factors (CCF) and component mode 
contribution factors (CMCF). 
"""
from abaqus import *
from abaqusConstants import *
import os, types
from cPickle import Unpickler
from abq_BrakeNoise.computeContributionsConstants import *

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def plotCcf(resultsFileName, unstableMode, componentNames):
    """
    Method to plot the CCF.
    Required arguments:
    
    resultsFileName: Name of the results file.
    unstableMode: The unstable mode number.
    componentNames: A sequence of componentNames
    Returns None
    """
    
    # Check args
    #
    _checkResultsFileName(resultsFileName) 
    if type(unstableMode) != types.IntType:
        raise TypeError, 'unstableMode must be an integer; got %s.' % type(unstableMode)

    if type(componentNames) != types.ListType and type(componentNames) != types.TupleType:
        raise TypeError, 'componentNames must be a list or a tuple; got %s/' % type(componentNames)
    
    resultsFile = open(resultsFileName)
    
    try:
        pfile = Unpickler(resultsFile)
        pickleDict = pfile.load()
    except:
        raise ValueError, '"%s" is an invalid results file.' % resultsFileName
      
    try:
        complexFrequencyODBName = pickleDict['complexFrequencyODBName']
        if type(complexFrequencyODBName) != types.StringType:
            raise TypeError
        unstableModesDict = pickleDict['unstableModesDict'] 
        if type(unstableModesDict) != types.DictType:
            raise TypeError
        componentNamesForCCF = pickleDict['componentNamesForCCF']
        if type(componentNamesForCCF) != types.ListType and type(componentNamesForCCF) != types.TupleType: 
            raise TypeError
        ccfDict = pickleDict['ccfDict']
        if type(ccfDict) != types.DictType:
            raise TypeError
    except:
        msg = 'The results file is not in a valid format.'
        msg += '\nPlease check if the file was created using the compute component contributions plug-in.'
        raise ValueError, msg
       
    if not unstableModesDict.has_key(unstableMode):
        raise KeyError, 'No results are available for the specified unstable mode %d.' % unstableMode
    
    frequency = unstableModesDict[unstableMode]
    
    componentsDict = {}
    for cn in componentNames:
        if cn not in componentNamesForCCF:
            raise ValueError, 'No results are available for the component "%s".' % cn        
        componentsDict[cn] = ccfDict[unstableMode][cn]
    
    title = 'CCF for unstable mode: %d (%g Hz)' % (unstableMode, frequency)
    
   
    _makeCcfBargraph(componentsDict, title)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def plotCmcf(resultsFileName, unstableMode, componentName, maxNumModes=10):
    """
    Method to plot the CMCF.
    Required arguments:
    
    resultsFileName: Name of the results file.
    unstableMode: The unstable mode number.
    componentName: The name of the component.
    
    Optional Arguments:
    maxNumModes: maximum number of modes to show. 
    """
    
    # Check args
    #
    _checkResultsFileName(resultsFileName) 
    if type(unstableMode) != types.IntType:
        raise TypeError, 'unstableMode must be an integer; got %s.' % type(unstableMode)
    
    if type(componentName) != types.StringType:
        raise TypeError, 'componentName must be a string; got %s/' % type(componentName)
    
    if type(maxNumModes)!= types.IntType:
        raise TypeError, 'maxNumModes must be an integer; got %s.' % type(maxNumModes)
    
    if maxNumModes < MINIMUM_NUM_MODES:
        raise ValueError, 'maxNumModes must be greater than or equal to %d.' % MINIMUM_NUM_MODES

    resultsFile = open(resultsFileName)
    
    try:
        pfile = Unpickler(resultsFile)
        pickleDict = pfile.load()
    except:
        raise ValueError, '"%s" is an invalid results file.' % resultsFileName
      
    try:
        complexFrequencyODBName = pickleDict['complexFrequencyODBName']
        if type(complexFrequencyODBName) != types.StringType:
            raise TypeError
        unstableModesDict = pickleDict['unstableModesDict'] 
        if type(unstableModesDict) != types.DictType:
            raise TypeError
        componentNamesForCMCF = pickleDict['componentNamesForCMCF']
        if type(componentNamesForCMCF) != types.ListType and type(componentNamesForCMCF) != types.TupleType: 
            raise TypeError
        cmcfDict = pickleDict['cmcfDict']
        if type(cmcfDict) != types.DictType:
            raise TypeError        
    except:
        msg = 'The results file is not in a valid format.'
        msg += '\nPlease check if the file was created using the compute component contributions plug-in.'
        raise ValueError, msg
    
    if len(componentNamesForCMCF) == 0:
        raise ValueError, 'The results file does not contain CMCF data.'
    
    if not unstableModesDict.has_key(unstableMode):
        raise KeyError, 'No results are available for the specified unstable mode %d.' % unstableMode
    
    frequency = unstableModesDict[unstableMode]
  
    if componentName not in componentNamesForCMCF:
        raise ValueError, 'No results are available for the component "%s".' % componentName
    
    
    freqContributions = cmcfDict[unstableMode][componentName]
    
    # Arrange data for sorting
    #
    data = []
    for freq, contribution in freqContributions.items():
        data.append((contribution, freq))
    
    data.sort()
    data.reverse()
    
    dataLen = len(data)
    
    if maxNumModes > dataLen:
        print '\nWARNING: '
        print 'The specified value of maxNumModes(%d) exceeds \nthe available number of frequency modes(%d).' % (maxNumModes, dataLen)
        print 'All the modes will be shown.'
        maxNumModes = dataLen
    
    showData = data[:maxNumModes]
    otherData = data[maxNumModes:]
    
    othersTotal = 0
    
    for contribution, freq in otherData:
        othersTotal += contribution
    
    newData = []
    
    for contribution, freq in showData:
        newData.append((freq, contribution))
    
    newData.append(('Others', othersTotal))
    title = 'CMCF for %s, unstable mode: %d (%g Hz)' % (componentName, unstableMode, frequency)    
    _makeCmcfBargraph(newData, title)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def _makeCcfBargraph(contributions, title):

    # Method to create the bar graphs for the specified contributions.
    # The bar graphs are plotted in the current viewport.    
    # 

    # Get current viewport
    #
    vp = session.viewports[session.currentViewportName]
    
    bgName = vp.name

    # Create XYPlot
    #
    try:
        xyp = session.XYPlot(name =bgName)
    except:
        xyp = session.xyPlots[bgName]
    chartName = xyp.charts.keys()[0]
    chart = xyp.charts[chartName]

    # Now create the appropriate curves               
    #
    start = (0.1,0.0)
    width = .8
    inc = .2    
    curvesToPlot = []

    for cnName in contributions.keys():
        mag = contributions[cnName]
        xyData = []        
        xyData.append(start)
        first = start[0],mag
        xyData.append(first)
        xyData.append(first)
        second = start[0]+width, mag
        xyData.append(second)
        xyData.append(second)
        third = start[0]+width,start[1]
        xyData.append(third)
        name = '%s' % cnName

        # So that the xy data does not interfere with viewports
        #
        xyDataName = '%s-%s' % (bgName, name)
        xyData = session.XYData(name = xyDataName,data = xyData)
        #curve = xyp.Curve(name, xyData)
        curve = session.Curve(xyData=xyData)
        curve.curveOptions.setValues(lineThickness=MEDIUM,
                                     legendTextSource=USER_DEFINED,
                                     legendText=name)
        curvesToPlot.append(curve,)
        start = third[0]+inc,start[1]

    chart.setValues(curvesToPlot=curvesToPlot)
    chart.axes1[0].setValues(labelFrequency=0)
    chart.axes1[0].axisData.setValues(useSystemTitle=False, title='Components')
    chart.axes2[0].axisData.setValues(useSystemTitle=False, title='Contribution(%)')
    #xyp.xyPlotOptions.setValues(xAxisLabelFrequency=0)
#    xyp.setValues(xAxisTitleSource=USER_DEFINED,
#                  yAxisTitleSource=USER_DEFINED, xAxisTitle='Components', 
#                  yAxisTitle='Contribution(%)')
    
    try:
        # Now set the plot title (6.6 only)
        #
        xyp.setValues(plotTitle=title)        
    except:
        # 6.5
        #
        print title
        
    vp.setValues(displayedObject=xyp)
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def _makeCmcfBargraph(cmcfData, title):

    # Method to create the bargraphs for the specified cmcfData
    # The bar graphs are drawn in the current viewport
    #

    # Get current viewport
    #
    vp = session.viewports[session.currentViewportName]
    bgName = vp.name

    # Create XYPlot
    #
    try:
        xyp = session.XYPlot(name =bgName)
    except:
        xyp = session.xyPlots[bgName]
    chartName = xyp.charts.keys()[0]
    chart = xyp.charts[chartName]


    # Now create the appropriate curves
    #
    start = (0.1,0)
    width = .8
    inc = .2    
    curvesToPlot = []
    
    numData = len(cmcfData)

    for i in range(numData):
        mag = cmcfData[i][1]
        xyData = []        
        xyData.append(start)
        first = start[0],mag
        xyData.append(first)
        xyData.append(first)
        second = start[0]+width, mag
        xyData.append(second)
        xyData.append(second)
        third = start[0]+width,start[1]
        xyData.append(third)
        if i == numData - 1: # last item
            name = 'Others'
        else:
            name = 'Mode- %d' %  cmcfData[i][0]
        # So that the xy data does not interfere with viewports
        #
        xyDataName = '%s-%s' % (bgName, name)
        xyData = session.XYData(name = xyDataName,data = xyData)
        curve = session.Curve(xyData=xyData)
        #curve = xyp.Curve(name, xyData)
        curve.curveOptions.setValues(lineThickness=MEDIUM,
                                     legendTextSource=USER_DEFINED,
                                     legendText=name)
        curvesToPlot.append(curve,)

        start = third[0]+inc,start[1]

    chart.setValues(curvesToPlot=curvesToPlot)
    chart.axes1[0].setValues(labelFrequency=0)
    chart.axes1[0].axisData.setValues(useSystemTitle=False, title='Frequency Modes')
    chart.axes2[0].axisData.setValues(useSystemTitle=False, title='Contributions')
#    xyp.setValues(curvesToPlot=curvesToPlot)
#    xyp.xyPlotOptions.setValues(xAxisLabelFrequency=0)
#    xyp.setValues(xAxisTitleSource=USER_DEFINED,
#                  yAxisTitleSource=USER_DEFINED, xAxisTitle='Frequency Modes', 
#                  yAxisTitle='Contributions')
    
    try:
        # Now set the plot title (6.6 only)
        #
        xyp.setValues(plotTitle=title)       
    except:
        # 6.5
        #
        print title
        
    vp.setValues(displayedObject=xyp)
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def _checkResultsFileName(resultsFileName):

    # Private method to check the resultsFileName
    #
    if type(resultsFileName) != types.StringType:
        raise TypeError, 'resultsFileName must be a string; got %s.' % type(resultsFileName)
    
    if not resultsFileName.strip():
        raise ValueError, 'resultsFileName cannot be an empty string.'
    
    if not os.path.isfile(resultsFileName):
        raise OSError, 'There is no file with name "%s".' % resultsFileName
    







