from abaqus import *
from abaqusConstants import *
import part
import regionToolset
import section
import customKernel
import numpy as np
       
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def tol(val):
    """
    Returns a value that is zero if the input is less than zero tolerance
    """
    
    zeroTol = 1e-9
    
    if (abs(val) < zeroTol):
        result = 0
    else:
        result = val
        
    return result    

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def calcReducedStiffness(matType,listofMatData):
#---------------
#This function takes in a list of anistoropic or orthotropic material parameters and returns a reduced C Matrix.
#This function is added as a part of SR00318577
#---------------
    stiffness=np.eye(6)

    ListmatAnis=[0]*21
    Listmat = listofMatData

    if matType =="ANISOTROPIC":
        Datacnt = len(Listmat)
        if Datacnt <21:
            raise ValueError ("Insufficient data in *Elastic,Anistoropic keyword for one of the materials assigned")
        for i in range(21):
            ListmatAnis[i]=float(Listmat[i])

    elif matType=='ORTHOTROPIC':
        Datacnt = len(Listmat)
        if Datacnt <9:
            raise ValueError ("Insufficient data in *Elastic,Elastic keyword")
        
        Listmat =[float(x) for x in Listmat]
        ListmatAnis[0:6]= Listmat[0:6]
        ListmatAnis[9]=Listmat[6]
        ListmatAnis[14]=Listmat[7]
        ListmatAnis[20]=Listmat[8]

    for i in range(6):
        for j in range(6):        
            cnt= i*(i+1)/2
            stiffness[j][i]=ListmatAnis[cnt+j]
            stiffness[i][j]= stiffness[j][i]
            if (j==i):
                break

    #---------------------
    #Creating compliance and reduced compliance
    #---------------------
    
    try:
        compliance = np.linalg.inv(stiffness)
    except:
        raise ValueError ("The stiffness matrix for the given Anistropic/orthotropic data cannot be inverted. Please verify the defined material properties")

    reducedCompliance = np.delete(compliance,[2,4,5],0)    
    reducedCompliance = np.delete(reducedCompliance,[2,4,5],1)
    
    try:
        reducedStiffness = np.linalg.inv(reducedCompliance)        
    except:
        raise ValueError ("The stiffness matrix for the given Anistropic/orthotropic data cannot be inverted. Please verify the defined material properties")    
    return reducedStiffness


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
##    The code is a port to Python from Mike Mathers' C source (thanks Mike!)
def calcABD():###specName, laminae, m, s, offset_ratio, description):

    # Ply data stored in custom data.
    # laminae attribute is a list of lists. Only one list if single ply stack
    # Two lists if face sheets.
    #
    cd = session.customData.compositePluginData['laminae']
    
    laminae = cd.laminae
    specName = cd.specName
    m,c,s = cd.mcs
    offset_ratio = cd.offsetRatio
    
    description = cd.description
    txtStr = ''
    
    # Create list of lamina data
    #

    #Bug fix for adding support for anisotropy
    suppMatType=[ISOTROPIC,LAMINA,ENGINEERING_CONSTANTS,ANISOTROPIC,ORTHOTROPIC]
    cMatrixforAnis = {}
    stackCount=0

    for stack in laminae:

        # Initialize the lists                
        EX=[]
        EY=[]
        PRXY=[]
        GXY=[]
        thick=[]
        theta=[]
        plyMat=[]
        plyName = []

        for l in stack:

            #Bug fix for adding support for anisotropy
            stackCount+=1
            stackKey = 'S'+str(stackCount)
            
            if (m[l.material].elastic.type == ISOTROPIC): 
                data=[ m[l.material].elastic.table[0][0],
                       m[l.material].elastic.table[0][0],
                       m[l.material].elastic.table[0][1],
                       m[l.material].elastic.table[0][0]/(2*(1+m[l.material].elastic.table[0][1])),]                  
            elif (m[l.material].elastic.type == LAMINA): 
                data=[ m[l.material].elastic.table[0][0],
                       m[l.material].elastic.table[0][1],
                       m[l.material].elastic.table[0][2],
                       m[l.material].elastic.table[0][3],]
            elif (m[l.material].elastic.type == ENGINEERING_CONSTANTS): 
                data=[ m[l.material].elastic.table[0][0],
                       m[l.material].elastic.table[0][1],
                       m[l.material].elastic.table[0][3],
                       m[l.material].elastic.table[0][6],]
            elif (m[l.material].elastic.type == ANISOTROPIC):            
                data=[stackKey]*4
                matList = m[l.material].elastic.table[0] 
                matType='ANISOTROPIC'
                RedStiff=calcReducedStiffness(matType,matList)
                cLocal = [RedStiff[0][0],RedStiff[0][1],RedStiff[1][1],RedStiff[2][2]]
                cMatrixforAnis[stackKey]=cLocal

            elif (m[l.material].elastic.type == ORTHOTROPIC):
                data=[stackKey]*4
                matList = m[l.material].elastic.table[0] 
                matType='ORTHOTROPIC'
                RedStiff=calcReducedStiffness(matType,matList)
                cLocal = [RedStiff[0][0],RedStiff[0][1],RedStiff[1][1],RedStiff[2][2]]
                cMatrixforAnis[stackKey]=cLocal

            elif (m[l.material].elastic.type not in suppMatType):
                plyMat = l.material
                plyName = l.plyName
                raise ValueError ('Applied elastic material property for the ply "'+str(plyName) + '" with the material "' +str(plyMat) +'" is inadequate for ABD matrix calculation.\
                                  \nKindly review the material properties and try')

            if (type(l).__name__ == 'SectionLayer'):
                thetaVal = l.orientAngle
            else:
                if (l.orientationType==ANGLE_90):
                	thetaVal = 90
                elif (l.orientationType==ANGLE_45):
                	thetaVal = 45
                elif (l.orientationType==ANGLE_0):
                	thetaVal = 0
                elif (l.orientationType==ANGLE_NEG45):
                	thetaVal = -45
                else:
                	thetaVal = l.orientationValue

            
            EX.append(data[0])
            EY.append(data[1])
            PRXY.append(data[2])
            GXY.append(data[3])
            theta.append(thetaVal)
            thick.append(l.thickness)
            plyMat.append(l.material)
            plyName.append(l.plyName)

        # Account for section layup symmetry switch
        #
        if (type(l).__name__ =='SectionLayer' and s[specName].symmetric):
            count=len(EX)
            for isym in range(count):
                EX.append(EX[0:count][-(isym+1)])
                EY.append(EY[0:count][-(isym+1)])
                PRXY.append(PRXY[0:count][-(isym+1)])
                GXY.append(GXY[0:count][-(isym+1)])
                theta.append(theta[0:count][-(isym+1)])
                thick.append(thick[0:count][-(isym+1)]) 
                plyMat.append(plyMat[0:count][-(isym+1)])
                plyName.append(plyName[0:count][-(isym+1)])

        # Extract the material data
        #
        nn_layers = len(EX)

        # Initialize the arrays
        #
        abbd    = [[0 for i in range(6)] for j in range(6)]
        clayer  = [[0 for i in range(3)] for j in range(3)]
        tr      = [[0 for i in range(6)] for j in range(6)]
        cglobal = [0 for j in range(6)]

        total_t = 0.0

        for ixxx in range(nn_layers):            
            total_t += thick[ixxx]

        # offset_ratio is OFFSET in the ABAQUS sense (offset/thickness) 
        #
        offset_length = offset_ratio * total_t

        zlbot = -.5*total_t - offset_length

        for i_layer in range(nn_layers):

            if ( EX[i_layer] == 0. ):                
                print ("Note that the E1 value for the ply '" + plyName[i_layer] + "' is found to be 0. This ply will be skipped for ABD Matrix calculation" )
                continue
            
            if (isinstance(EX[i_layer],basestring)):                
                clocal = cMatrixforAnis[EX[i_layer]]                
            else:
                PRYX = PRXY[i_layer] * EY[i_layer]/EX[i_layer]
                #clocal =  Listclocal[[i_layer]
                clocal =[ EX[i_layer]/(1.-PRXY[i_layer]*PRYX), PRXY[i_layer]*EY[i_layer]/(1.-PRXY[i_layer]*PRYX), EY[i_layer]/(1.-PRXY[i_layer]*PRYX), GXY[i_layer] ]
            
            #theta still defined (orientationValue). But orientation attribute can point to an Datum Csys,
            #
            angle_r = theta[i_layer]*3.14159265358979/180. #TODO: Error thrown here when *ply* has local CSYS as orientation! (i.e. other than a float)
            
            C1T = cos(angle_r)
            S1T = sin(angle_r)
            C2T = C1T*C1T
            C3T = C2T*C1T
            C4T = C2T*C2T
            S2T = S1T*S1T
            S3T = S2T*S1T
            S4T = S2T*S2T

            tr[0][0] = C4T
            tr[1][0] = S4T
            tr[2][0] = S2T*C2T
            tr[3][0] = S2T*C2T
            tr[4][0] = S1T*C3T
            tr[5][0] = S3T*C1T
            tr[0][1] = 2.*S2T*C2T
            tr[1][1] = 2.*S2T*C2T
            tr[2][1] = S4T + C4T
            tr[3][1] = (-2.)*C2T*S2T
            tr[4][1] = S3T*C1T - S1T*C3T
            tr[5][1] = S1T*C3T - S3T*C1T
            tr[0][2] = S4T
            tr[1][2] = C4T
            tr[2][2] = S2T*C2T
            tr[3][2] = S2T*C2T
            tr[4][2] = (-1.)*S3T*C1T
            tr[5][2] = (-1.)*S1T*C3T
            tr[0][3] = 4.*S2T*C2T
            tr[1][3] = 4.*S2T*C2T
            tr[2][3] = (-4.)*S2T*C2T
            tr[3][3] = S4T + C4T - 2.*S2T*C2T
            tr[4][3] = 2.*(S3T*C1T-S1T*C3T)
            tr[5][3] = 2.*(S1T*C3T-S3T*C1T)

            cglobal[0]  = tr[0][0] * clocal[0] + tr[0][1] * clocal[1] + tr[0][2] * clocal[2] + tr[0][3] * clocal[3]
            cglobal[1]  = tr[1][0] * clocal[0] + tr[1][1] * clocal[1] + tr[1][2] * clocal[2] + tr[1][3] * clocal[3]
            cglobal[2]  = tr[2][0] * clocal[0] + tr[2][1] * clocal[1] + tr[2][2] * clocal[2] + tr[2][3] * clocal[3]
            cglobal[3]  = tr[3][0] * clocal[0] + tr[3][1] * clocal[1] + tr[3][2] * clocal[2] + tr[3][3] * clocal[3]
            cglobal[4]  = tr[4][0] * clocal[0] + tr[4][1] * clocal[1] + tr[4][2] * clocal[2] + tr[4][3] * clocal[3]
            cglobal[5]  = tr[5][0] * clocal[0] + tr[5][1] * clocal[1] + tr[5][2] * clocal[2] + tr[5][3] * clocal[3]

            clayer[0][0] = cglobal[0]
            clayer[1][0] = cglobal[2]
            clayer[2][0] = cglobal[4]
            clayer[0][1] = cglobal[2]
            clayer[1][1] = cglobal[1]
            clayer[2][1] = cglobal[5]
            clayer[0][2] = cglobal[4]
            clayer[1][2] = cglobal[5]
            clayer[2][2] = cglobal[3]

            zltop = zlbot + thick[i_layer]
            wgt1 = zltop - zlbot
            wgt2 = (zltop*zltop - zlbot*zlbot)*.5
            wgt3 = (zltop*zltop*zltop - zlbot*zlbot*zlbot)/3.
            zlbot = zltop


            for ixxx in range(3):

                for iyyy in range(3):

                    coeff = clayer[ixxx][iyyy]
                    var = coeff*wgt1
                    abbd[ixxx][iyyy] += coeff*wgt1
                    abbd[ixxx+3][iyyy] += coeff*wgt2
                    abbd[ixxx][iyyy+3] += coeff*wgt2
                    abbd[ixxx+3][iyyy+3] += coeff*wgt3

        # Print the results
        #
        txtStr = '%sLaminate specification %s\n' % (txtStr, description)
        txtStr += '%15s %15s %12s %7s\n' % ('Ply', 'Material', 'Thickness', 'Local Orientation Angle')
        txtStr += '-------------------------------------------------------------------------------------\n'
        for pcnt in range(len(EX)):
            ##txtStr += '%5s %15s %12.5f %15.2f\n' % (pcnt+1, plyMat[pcnt], thick[pcnt], theta[pcnt]) 
            txtStr += '%15s %15s %12.5f %15.2f\n' % (plyName[pcnt], plyMat[pcnt], thick[pcnt], theta[pcnt]) 

        txtStr += "\nABD Matrix Calculation %s\n" % (description)

        for row in range(6):
            if (row == 3):
                txtStr += "-------------------------------------------------------------------------------------\n"

            txtStr += "%13.4E %13.4E %13.4E | %13.4E %13.4E %13.4E\n"%(tol(abbd[row][0]),
                                                                       tol(abbd[row][1]),
                                                                       tol(abbd[row][2]),
                                                                       tol(abbd[row][3]),
                                                                       tol(abbd[row][4]),
                                                                       tol(abbd[row][5]))


        print 'Laminate specification ', description
        print '%5s %15s %12s %7s' % ('Ply', 'Material', 'Thickness', 'Local Orientation Angle')
        print '-------------------------------------------------------------------------------------'
        for pcnt in range(len(EX)):
            print '%15s %15s %12.5f %15.2f\n' % (plyName[pcnt], plyMat[pcnt], thick[pcnt], theta[pcnt]) 
            ##print '%5s %15s %12.5f %15.2f' % (pcnt+1, plyMat[pcnt], thick[pcnt], theta[pcnt]) 

        print "\nABD Matrix Calculation ", description

        for row in range(6):
            if (row == 3):
                print "-------------------------------------------------------------------------------------"
            print("%13.4E %13.4E %13.4E | %13.4E %13.4E %13.4E"%(tol(abbd[row][0]),
                tol(abbd[row][1]),tol(abbd[row][2]),tol(abbd[row][3]),tol(abbd[row][4]),tol(abbd[row][5])))



        ##### Print the inverse of ABD (this is another interesting output...)
        

        #AINV = LinearAlgebra.inverse(abbd)

        #
        #     print "\nABD Inverse Matrix Calculation for", specName
        #     
        #     for row in range(6):
        #         if (row == 3):
        #             print "-------------------------------------------------------------------------------------"
        #         print("%13.4E %13.4E %13.4E | %13.4E %13.4E %13.4E"%(abbd_inv[row][0],
        #             abbd_inv[row][1],abbd_inv[row][2],abbd_inv[row][3],abbd_inv[row][4],abbd_inv[row][5]))           


    
        #--------------------------------------------
        # Calculate and Display the equivalent moduli
        #--------------------------------------------

        #Make sure the composite is symmetric
        tolerance=1e-6
        isSymmetric = True
        for i1 in range(3,6):
            for j1 in range(3):
                if abs(abbd[i1][j1])>tolerance or abs(abbd[j1][i1])> tolerance:
                    isSymmetric = False
                    break


        #------------------
        #Get the inverse of the ABD matrix
        #------------------
        abbd_inv = None
        try:
            abbd_inv = np.linalg.inv(abbd) #abbd is the abd matrix
        except:
            abbd_inv = None
        if abbd_inv==None: isSymmetric= False


        #------------------
        #Calculate the equivalent moduli only if it is symmetric
        #------------------
        if isSymmetric:
            NUXYT = -1.0*abbd_inv[1][0]/abbd_inv[0][0]
            NUYXT = -1.0*abbd_inv[1][0]/abbd_inv[1][1]
            EXT = 1.0/(total_t*abbd_inv[0][0])
            EYT = 1.0/(total_t*abbd_inv[1][1])
            GXYT = 1.0/(total_t*abbd_inv[2][2])
            NUXYB = -1.0*abbd_inv[4][3]/abbd_inv[3][3]
            NUYXB = -1.0*abbd_inv[4][3]/abbd_inv[4][4]
            EXB = 12.0/(pow(total_t,3)*abbd_inv[3][3])
            EYB = 12.0/(pow(total_t,3)*abbd_inv[4][4])
            GXYB = 12.0/(pow(total_t,3)*abbd_inv[5][5])

            txtStr += """\nEquivalent extensional moduli for %s
            EXT   = %13.4E
            EYT   = %13.4E
            GXYT  = %13.4E
            NUXYT = %8.4F
            NUYXT = %8.4F
            \nEquivalent bending moduli for %s
            EXB   = %13.4E
            EYB   = %13.4E
            GXYB  = %13.4E
            NUXYB = %8.4F
            NUYXB = %8.4F\n\n\n\n """ % (description,EXT,EYT,GXYT,NUXYT,NUYXT,description,EXB,EYB,GXYB,NUXYB,NUYXB)


            
            print "\nEquivalent extensional moduli ", description
            print "EXT   = %13.4E"%EXT
            print "EYT   = %13.4E"%EYT
            print "GXYT  = %13.4E"%GXYT
            print "NUXYT = %8.4F"%NUXYT
            print "NUYXT = %8.4F"%NUYXT

            print "\nEquivalent bending moduli " , description
            print "EXB   = %13.4E"%EXB
            print "EYB   = %13.4E"%EYB
            print "GXYB  = %13.4E"%GXYB
            print "NUXYB = %8.4F"%NUXYB
            print "NUYXB = %8.4F\n\n\n"%NUYXB
            
        session.customData.compositePluginData['text'].text = txtStr       
        key = 'text'
        if key not in session.customData.compositePluginData.keys():
            a = tmp()
        
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
class CompositePluginData(customKernel.CommandRegister):
    #class for creating kernel data.
    #Used to store data used in dialogs
    #Also used to calculate mappings of element to sets.
    #Once carried out, no need if querying another element.
    def __init__(self, name, obj):
        self.obj = obj


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
class tmp:
    #Dummy class for creating instances on customData
    pass

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createCustomRepository():
    if 'CompositePluginData' not in dir(session.customData):
        session.customData.Repository('compositePluginData', CompositePluginData)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
def getElementMapping(regions):

    # Element to sets
    #
    vpname = session.currentViewportName
    createCustomRepository()
    p = session.viewports[vpname].displayedObject
    model = mdb.models[p.modelName]
    key = '%s_%s' % (p.modelName, p.name)
    if key not in session.customData.compositePluginData.keys():
        a = tmp()
        session.customData.CompositePluginData(key, a)
        elementMapping = {}
        for set in regions:
            #
            if p.sets.has_key(set):
                for element in p.sets[set].elements:
                    if not elementMapping.has_key(element.label):
                        elementMapping[element.label] = []
                    elementMapping[element.label].append(set)
        session.customData.compositePluginData[key].obj.elementMapping = elementMapping
    else:
        elementMapping = session.customData.compositePluginData[key].obj.elementMapping
    return elementMapping

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getUsedRegions():

    vpname = session.currentViewportName
    p = session.viewports[vpname].displayedObject
    m,c,s = getMCS()
    regions = {}
    for sa in p.sectionAssignments:
        regions[sa.region[0]] = []
    for name, layup in c.items():
        for ply, value in layup.plies.items():
            regions[value.region[0]] = []
    return regions.keys()

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getMCS():

    # Function to get materials, composite layups and sections
    #
    vpname = session.currentViewportName
    p = session.viewports[vpname].displayedObject
    model = mdb.models[p.modelName]
    m = model.materials
    c = p.compositeLayups
    s = model.sections
    return m, c, s

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getMapping():

    # Maps faces or cells of part's sets
    #
    vpname = session.currentViewportName
    p = session.viewports[vpname].displayedObject
    sets = p.sets
    intSets = p.allInternalSets
    mySets={}


    #--------------
    # Bug fix on 5_17_2016 (SR00338768)
    # Adding support for internal sets
    #--------------
    for set in sets.keys():
        mySets[set] = sets[set]


    for intSet in intSets.keys():
        if intSet not in mySets:
            mySets[intSet] = intSets[intSet]

    faceMapping = {}
    cellMapping = {}
    for face in p.faces:
        faceMapping[face.index] = []
        for set in mySets.keys():
            setFaces = mySets[set].faces
            for setFace in setFaces:
                if setFace.index == face.index:
                    faceMapping[face.index].append(set)
                    break
    #---------------------
    # End of bug fix
    #--------------------

    for cell in p.cells:
        cellMapping[cell.index] = []
        for set in sets.keys():
            setCells = sets[set].cells
            for setCell in setCells:
                if setCell.index == cell.index:
                    cellMapping[cell.index].append(set)
                    break
    return faceMapping, cellMapping

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getOffsetValue(tp, val=0.0):

    if tp == SINGLE_VALUE:
        val = val
    elif tp == TOP_SURFACE:
        val = 0.5
    elif tp == BOTTOM_SURFACE:
        val = -0.5

    return val
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def collectLayUpData(region, core=''):

    # Check if displayed object contains any section assignment or 
    # lay ups
    #
    p = session.viewports[session.currentViewportName].displayedObject
    if len(p.sectionAssignments) == 0 and len(p.compositeLayups) == 0:
        msg = 'Part does not contain any section assigments or composite layups'
        ValueError, msg
        
    # Find sets
    #
    faceMapping, cellMapping = getMapping()
    offsetValue = 0.0 #Default
    try:

        if type(region) == part.FaceType:
            sets = faceMapping[region.index]
        elif type(region) == part.CellType:
            sets = cellMapping[region.index]
        elif type(region) == part.MeshElementType:
            #Element......
            #New mapping. Need to find all sets that are used in section assignments or layups
            #
            regions = getUsedRegions()
            #
            elementMapping = getElementMapping(regions)
            #
            sets = elementMapping[region.label]
    except:
        #Error working out sets.
        msg = 'Error when working out sets used. Please check part\n'\
              'contains section assigments or composite layups'
        raise ValueError, msg

    m,c,s = getMCS()
    layups = c.keys()
    layups.reverse()
    #Layups used alphabetically. Look in reverse order. If found, use that one.
    laminae=[[]]
    for layup in layups:
        coreName = core
        if coreName:
            laminae = [[], []]
        else:
            laminae = [[]]
        count = 0
        for ply in c[layup].plies.values():
            if ply.plyName == coreName:
                count = 1
                continue
            if ply.region[0] in sets:
                if not ply.suppressed:
                    laminae[count].append(ply)

        lType = type(c[layup]).__name__
        if (c[layup].symmetric):
            topLam=laminae[0]
            topLam.reverse()
            botLam = []
            for ply in c[layup].plies.values():
                if ply.region[0] in sets:
                    if not ply.suppressed:
                        botLam.append(ply)
                #
                #botLam = c[layup].plies.values()
                laminae[0] = botLam + topLam

        if laminae[0]:
            description = 'for picked region using layup %s' % layup
            if coreName:
                description = '%s for facesheet' % description
            layupOrSectionName = layup
            if c[layup].offsetType == SINGLE_VALUE:
                offsetValue = getOffsetValue(c[layup].offsetType,
                                             c[layup].offsetValues[0])
            else:
                offsetValue = getOffsetValue(c[layup].offsetType,)                
            break

    # If no layups found, look for sections
    #
    if not laminae[0]:
        for sectionAssignment in p.sectionAssignments:
            set = sectionAssignment.region[0]
            sectionName = sectionAssignment.sectionName
            sectionOffsetType = sectionAssignment.offsetType
            sectionOffset = sectionAssignment.offset
            laminae=[[]]
            if set in sets:
                if core:
                    coreName = core
                    laminae = [[],[]]
                    count = 0
                    for layup in s[sectionName].layup:
                        if layup.plyName == coreName:
                            count += 1
                        else:
                            laminae[count].append(layup)
                else:
                    laminae = [s[sectionName].layup]
            if laminae[0]:
                description = 'for picked region using section %s' % sectionName
                if core:
                    description = '%s for facesheet' % description
                layupOrSectionName = sectionName
                offsetValue = getOffsetValue(sectionOffsetType, sectionOffset)

    if not laminae[0]:

        # Composite layup not found for region selected.
        # Raise error
        msg = 'No composite layup information \nfound in selected region.'
        raise ValueError, msg

    # Check if core chosen left one laminae with no ply information
    #
    if core:
         if not len(laminae[0]) or\
            not len(laminae[1]):
             msg = 'No composite layup information '\
                   'found in one side of the core region.'
             raise ValueError,msg
             
             
        
    cd = session.customData.compositePluginData['laminae']
    
    cd.laminae = laminae
    cd.specName = layupOrSectionName
    cd.mcs = (m,c,s)
    cd.offsetRatio = offsetValue
    cd.description = description
    
    if laminae[0]:       

        # Call caclABD()
        #
        calcABD()
            

        # Create viewport and display Ply Stack Plot
        #
        if not session.viewports.has_key('Composite plug-in'):
            vpn = session.currentViewportName
            width = 0.5*session.drawingArea.width
            height = session.drawingArea.height
            origin = session.drawingArea.origin
            vp = session.viewports[vpn]
            vp.setValues(origin=origin, width=width, height=height)
            if vp.windowState == MAXIMIZED:
                vp.restore()
            cpVp = session.Viewport(name='Composite plug-in', origin=(origin[0]+width, origin[1]),
                                    width=width, height=height)
        cpVp = session.viewports['Composite plug-in']
        faces = p.faces
        cells = p.cells
        elements = p.elements
        if type(region) == part.FaceType:
            faces = faces[region.index:region.index+1]
            region = regionToolset.Region(faces=faces)
        elif type(region) == part.CellType:
            cells = cells[region.index:region.index+1]
            region = regionToolset.Region(cells=cells)
        elif type(region) == part.MeshElementType:
            index = elements.index(region)
            elements = elements[index:index+1]
            region = regionToolset.Region(elements=elements)
        detp = section.MdbPlyStackPlot(part=p, region=region)
        cpVp.setValues(displayedObject=detp)
        cpVp.viewportAnnotationOptions.setValues(triadColor='Yellow', triadLabels=NUMBERS)
        cpVp.view.setViewpoint(viewVector=(1, -1, 1), cameraUpVector=(0, 0, 1))

########################################################################
#                                                                      #
# When plug-in starts create custom data. This will be used to store   # 
# face/cell->set mapping.                                              #
# Also used to store results for calculation. Main dialog registers a  # 
# a query on this and updated when changed.                            #
#                                                                      #
########################################################################
createCustomRepository()
session.customData.text = None
#Unique key:
key = 'text'
a = tmp()
session.customData.CompositePluginData(key, a)
session.customData.compositePluginData['text'].text = None
key = 'laminae'
a = tmp()
session.customData.CompositePluginData(key, a)
session.customData.compositePluginData['laminae'].laminae = None


