"""
Module to compute modal assurance component (MAC) and
generate a graphical representation of it.
"""

#
# abaqus imports
from abaqus import *
from abaqusConstants import *
from odbAccess import *
import uti,customKernel
from copy import deepcopy

#
# python imports
#
import math
import os, sys, types

from time import clock

from abq_CampbellDiagram.setConstants import (DEBUG_RUN_MODE, BLK2,
                                                   USE_NORMALIZATION)
from abq_Mac import macComplexTools
#
from abq_CampbellDiagram import macUtilities
from abq_CampbellDiagram import campbellDiagram

# be verbose if:
_verbose = (DEBUG_RUN_MODE >= 1)
_verboseDev = (DEBUG_RUN_MODE >= 2)
_namePrefix = __name__.split('.')[-1]
_verboseBase = BLK2+_namePrefix+':'
MAC_Value = 0.6

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def initCustomData():
    """
    Method to open create custom data for campbell diagrams
      in the session.customData object.
      
    @return the campbell dict in customData
    """
    if _verbose:
        print _verboseBase,'initCustomData'
        
    # Check for existance
    #
    if not(hasattr(session.customData, 'campbell')):
        campbell = {'odbs': {},
                    'useSteps': [],
                    }
        session.customData.campbell = campbell
    else:
        if _verbose:
            print 'CustomData already exists:',session.customData.campbell.keys()
        pass


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def openOdb(odbPath):
    """
    Method to open *COMPLEX FREQUENCY ODBs and store step names
      in the session.customData object.
      
    Required arguments:
    odbPath: The path to the *COMPLEX FREQUENCY ODB

    @return the odb step information in customData
    """
    
    if _verbose:
        print _verboseBase,'openOdb'
        
    # Check the steps
    #
    session.customData.campbell['odbs'][odbPath] = {'odbSteps': [],
                                                    'odbStepFreqs' : [],
                                                    'odbStepNorms' : {},
                                                    'odbNodeSets': [],
                                                    'massStatus': None,
                                                    'odbStatus': False
                                                    }
    custOdbData = session.customData.campbell['odbs'][odbPath]
    odbSteps = []
    pathIndex = 0
    #
    if type(odbPath) != types.StringType:
        raise TypeError, 'odbPath must be a string; got %s.' % type(odbPath)

    if odbPath.strip() == '':
        raise ValueError, 'Path cannot be an empty string.'

    if session.isUpgradeRequiredForOdb(upgradeRequiredOdbPath=odbPath):
        raise ValueError, ('%s\nis from a previous '
            'version of Abaqus.\nPlease close the plug-in and upgrade '
            'database before continuing.' % odbPath)

    odb = session.openOdb(odbPath)

    # Do odb type and step checks
    #
    if type(odb) != OdbType:
        raise TypeError, 'odb must be of type OdbType; got %s.' % type(odb)
    for step in odb.steps.values():
        stepData = []
        if step.procedure == '*COMPLEX FREQUENCY':
            spinSpeed = getattr(step, 'propertyEvaluation', None)
            if spinSpeed == None:
                desc = step.description.replace(' ','').upper()
                location = desc.find('PROPERTYEVALUATION=')
                if location > -1:
                    propDesc = desc[location:].split(',')[0]
                    try:
                        spinSpeed = float(propDesc.split('=')[1])*60.
                    except ValueError:
                        spinSpeed = None
            else:
                spinSpeed = spinSpeed*60.
            #
            if spinSpeed == None:
                spinSpeed = ''
            stepData += [1, step.name, spinSpeed]
            odbSteps += [stepData]

    # Make sure variables exist
    #
    for stepData in odbSteps:
        useStep, stepName, spinSpeed = stepData
        if not hasattr(odb.steps[stepName], "frames"):
            continue
        elif (len(odb.steps[stepName].frames) == 0 or
              not hasattr(odb.steps[stepName].frames[-1], "fieldOutputs")):
            continue
        elif not odb.steps[stepName].frames[-1].fieldOutputs.has_key('U'):
            continue
        elif not hasattr(odb.steps[stepName].frames[-1].fieldOutputs['U'], "values"):
            continue
        #
        custOdbData['odbSteps'] += [stepData]
        #
        # Scan frequencies:
        freqs = [_f.frequency for _f in odb.steps[stepName].frames
                              if _f.frequency > 0.0]
        custOdbData['odbStepFreqs'] += [freqs]

    #
    # Read set names:
    nsets = []
    a = odb.rootAssembly
    if a.instances.has_key('PART-1-1'):
        nsets = a.instances['PART-1-1'].nodeSets.keys()
    custOdbData['odbNodeSets'] += nsets
    #
    if not custOdbData['odbSteps']:
        errText = ('The ODB "%s"\n does not contain a *COMPLEX FREQUENCY step'%(odbPath)+
                   ' with field output "U" values.')
        raise ValueError, errText
        return None

    return 1


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def createCampbellDiagram(odbPath,
                          useSteps, modes, modesToPlot, nodeSet=None, useMass=True, 
                          xyPrefix='', calcCritPtIntersection=True,
                          refMultiplier=[], spinDirn=None, refFrame=None):
    """
    Start a Campbell run
    """
    if _verbose:
        print _verboseBase,'.createCampbellDiagram'
    #
    print '\n\n\n START\n'
    
    spin_axis=spinDirn
    if spin_axis=='X' or spin_axis=='-X':
        ID1 = 1; ID2 = 2
    elif spin_axis=='Y' or spin_axis=='-Y':
        ID1 = 2; ID2 = 0
    elif spin_axis=='Z' or spin_axis=='-Z':
        ID1 = 0; ID2 = 1
    
    
    # len useSteps must always be more than 1
    useSteps = eval(useSteps)
    session.customData.campbell['useSteps'] = useSteps
    
    # identify steps to work on:
    useStepNames = [stepData for stepData in useSteps
                             if stepData[0]]
    
    
    stepNames=[ step[1] for step in useStepNames]
    odb = session.odbs[odbPath]
    #
    
    modes = eval(modes)

   
    #
    # Reset status:
    session.customData.campbell['odbs'][odbPath]['odbStatus'] = ''
    #
    if useMass:
        session.customData.campbell['odbs'][odbPath]['massStatus'] = ''
    else:
        session.customData.campbell['odbs'][odbPath]['massStatus'] = None
    #
    #
    refMultiplier = eval(refMultiplier)
    #
    #
    # Generate mode correlations:
    modeCorrelation = combineSteps(odbPath, useSteps, modes, nodeSet, useMass)
    whirlDirDict=getWhirlDir(odb, ID1, ID2, stepNames, spinDirn, modeCorrelation)
    
    
    if refFrame=='Moving':
        for i in range(len(whirlDirDict)):
            for x in range(len(useStepNames)):
                if whirlDirDict[i]=='BACKWARD':
                    modeCorrelation[i][x]=[modeCorrelation[i][x][0], (modeCorrelation[i][x][0]/60)-modeCorrelation[i][x][1]]
                elif whirlDirDict[i]=='FORWARD':
                    modeCorrelation[i][x]=[modeCorrelation[i][x][0], (modeCorrelation[i][x][0]/60)+modeCorrelation[i][x][1]]
                else:
                    print 'There is either no whirl or the whirl direction is mixed.'
    
    
    #
    # store mode correlation for later reusage:
    session.customData.campbell['odbs'][odbPath]['modeCorrelation'] = modeCorrelation
    
    #
    # plot frequencies and crossPoints, if required.
    campbellDiagram.createXYData(odbPath,
                                 modeCorrelation,
                                 modesToPlot,
                                 xyPrefix,
                                 calcCritPtIntersection,
                                 refMultiplier,)
    return

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def getWhirlDir(odb, ID1, ID2, stepNames, spin_axis, modeCorrelation):
    ref_real = 2
    pos_real = 1
    ref_whirl =  1E-16
    
    
    whirlDirDict={}
    for step in stepNames:
        for frame in range(len(odb.steps[step].frames)-1):
            tmp_freq_frames=odb.steps[step].frames[frame+1]
            temp2 = []
            tmp_whirl = 0.
            tmp_whirl_1 = 0.
            outputVals = tmp_freq_frames.fieldOutputs['U'].values
            outputValsLen=len(outputVals)
            for iii in xrange(outputValsLen):
                outputVal=outputVals[iii]
                n = outputVal.nodeLabel
                a = outputVal.data
                b = outputVal.conjugateData
                Uc = float(a[ID1]); Us = -float(b[ID1])
                Vc = float(a[ID2]); Vs = -float(b[ID2])
                if Uc+Us+Vc+Vs != 0.:
                    W_f = 0.5*sqrt((Uc+Vs)**2 + (-Us+Vc)**2)
                    W_b = 0.5*sqrt((Uc-Vs)**2 + (Us+Vc)**2)

                    semimajor = W_f + W_b
                    semiminor = W_f - W_b

                    tmp_whirl +=semiminor
                    if semiminor == 0.0:
                        tmp_whirl_1 = tmp_whirl_1 + 1.
                    temp2.append((n, semimajor, semiminor))

            if(tmp_whirl > 0. and abs(tmp_whirl) > ref_whirl):
                if spin_axis.startswith('-'):
                    wDir="BACKWARD"
                else:    

                    wDir="FORWARD"
            elif (tmp_whirl < 0. and abs(tmp_whirl) > ref_whirl):
                if spin_axis.startswith('-'):
                    wDir="FORWARD"
                else:

                    wDir="BACKWARD"
            else:
                if tmp_whirl_1 == float(len(temp2)):
                    wDir="No Whirl"
                else:           
                    wDir="MIXED"
            whirlDirDict[frame]=wDir
            
    return whirlDirDict
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def updatePlot(odbPath, modesToPlot,
               xyPrefix='', calcCritPtIntersection=True,
               refMultiplier=[]):
    """
    Update the xy-plot after a complete campbell run was done before.
    """
    if _verbose:
        print _verboseBase,'.updatePlot'
    if _verboseDev:
        print 'odbPath',odbPath
        print 'xyPrefix',xyPrefix
        print 'calcCritPtIntersection',calcCritPtIntersection
        print 'refMultiplier',refMultiplier
        print 'showArrows',showArrows
    #
    refMultiplier = eval(refMultiplier)
    #
    # read mode correlations from session data
    modeCorrelation = session.customData.campbell['odbs'][odbPath]['modeCorrelation']
    #
    # plot frequencies and crossPoints, if required.
    campbellDiagram.createXYData(odbPath,
                                 modeCorrelation,
                                 modesToPlot,
                                 xyPrefix,
                                 calcCritPtIntersection,
                                 refMultiplier)
    return


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def combineSteps(odbPath, useSteps, modes, nodeSet, useMass):
    """
    Compare all steps os useSteps each one with the preceeding one.

    @param odbPath       STRING name of the ODB currently worked on.
    @param useSteps      LIST of LISTs [bool, name, float]. bool indicates, if step is
                         worked on. float is spin speed in current step.
    @param modes         LIST of freq
    @param nodeSet       STRING name of ODB_nset if not None, the values are only taken for this set.
    @param useMass       Bool

    @return modeCorrelation DICT of LISt of TUPLEs {modeId:[(spinSpeed,frequency)]}
    """
    
    # Open odbs
    #
    if _verbose:
        startTime = clock()
        print '\n\nStart combine steps: time = %6.3f'%(startTime)
    #
    odb = session.odbs[odbPath]
    inst1 = odb.rootAssembly.instances['PART-1-1']
    if not(inst1.nodeSets.has_key(nodeSet)):
        nodeSet = None
    if nodeSet:
        nodeSet = inst1.nodeSets[nodeSet]
    #
    #
    # identify steps to work on:
    useStepNames = [stepData for stepData in useSteps
                             if stepData[0]]
    #
    numberOfModes = len(modes)

    referenceStepName = useStepNames[0][1]
    referenceSpinSpeed = float(useStepNames[0][2])
    
    succeedMsg = '-->Steps: "%s" - "%s", SUCCEED: %s\n'
    failMsg    = '-->Steps: "%s" - "%s", FAILED!: %s\n'

    #
    # extract element mass at element nodes:
    status = False
    if useMass:
        status,nodeMassDict = macComplexTools._extractMassValues(odb, referenceStepName)
        if not(status):
            statusMsg = '-->Step: "%s", FAILED!: Mass Matrix could not be evaluated\n'
        else:
            statusMsg = '-->Step: "%s", SUCCEED: Mass Matrix evaluated\n'
        statusText = statusMsg%(referenceStepName)
        session.customData.campbell['odbs'][odbPath]['massStatus'] += statusText
    #
    if not(status):
        nodeMassDict = dict([(no.label,1.0) for no in inst1.nodes])

    # Dictionary modeCorrelation collects data
    # Keys are modes sorted by first step
    # Values are (spin speed, frequency, last mode)
    # "Spin speed" and "frequency" are used for plotting relating to mode
    # "last mode" is used to keep track of which mode is found after 
    # each MAC calculation
    #
    modeCorrelation = {}
    spinSpeedFirstStep = float(useStepNames[0][-1])
    i=0
    for freq in modes:
        modeCorrelation[i] = [(spinSpeedFirstStep,freq, i),]
        i+=1        

    # Need to keep track which step we are handling
    #
    stepNumber = 0
    
    # Loop through step values
    #
    for stepData in useStepNames[1:]:
        stepName = stepData[1]
        freqValues = []
        for frame in odb.steps[stepName].frames:
            if frame.frequency == 0.0: continue
            freqValues += [frame.frequency]
        spinSpeed = float(stepData[2])
        #
        stepStatus = True
        #
        # calculate MAC values:
        startTime = clock()
        if _verbose:
            print '\nStart ComplexMacCorrelation: time = %6.3f'%(startTime)
        #
        try:
            stepA = odb.steps[referenceStepName]
            stepB = odb.steps[stepName]
            #
            swap, takeList, valA, valB = macComplexTools.extractNodeList(stepA, stepB, nodeSet)

            nameAStep = referenceStepName
            nameBStep = stepName
            if swap:
                stepA, stepB = stepB,stepA
                nameAStep, nameBStep = nameBStep, nameAStep
            #
            #
            arrayA, arrayB,isComplex = macComplexTools.accessNodeData(takeList,
                                                                      stepA,
                                                                      stepB,
                                                                      nodeSet,
                                                                      numberOfModes)
            if useMass:
                nodeMassList = macComplexTools.rearrangeNodeMass(nodeMassDict,
                                                                 takeList,
                                                                 valA,
                                                                 nodeSet)
            else:
                nodeMassList = None
            #
            matrixLen = len(arrayA)
            #
            macMatrix = macComplexTools.calculateMacMatrix(arrayA, arrayB, matrixLen,
                                                           nodeMassList, isComplex,
                                                           genTextOutput=True)

            if _verbose:
                endTime = clock()
                print 'End MacCorrelation: complete time = %6.3f, (%6.3f)'%(endTime,endTime-startTime)

        except:
            if _verbose:
                _info = sys.exc_info()
                print _info[1]
            # something failed during MAC calculation
            
            statusText = failMsg%(referenceStepName, stepName, "MAC correlation")
            session.customData.campbell['odbs'][odbPath]['odbStatus'] += statusText
            stepStatus = False
        #
        # get mode order:
        if stepStatus:
            try:
                modeOrder = macUtilities.getModeOrder(macMatrix, printResults=_verbose)
            except:
                # something failed during mode correlation
                statusText = failMsg%(referenceStepName, stepName, "Mode correlation")
                session.customData.campbell['odbs'][odbPath]['odbStatus'] += statusText
                stepStatus = False
                raise
        #
        # add results into modeCorrelation:
        if stepStatus:
            for iMode,modeData in enumerate(modeOrder):
                modeDataKey = modeData[0][1]-1

                found = 0
                # Find last entry that matches with modeDataKey found in this mode,
                # then grab original (first step) mode key.
                #
                for key in modeCorrelation.keys():
                    if modeCorrelation[key][stepNumber][2] == iMode:
                        lastStepKey = key
                        found = 1

                # If entry found, add to modeCorrelation
                # If it does not pass MAC requirement, make it not found
                #
                if MAC_Value >= modeData[0][0]:
                    found = 0
                
                if found:
                    modeCorrelation[lastStepKey] += [(spinSpeed,
                                                      freqValues[modeDataKey],
                                                      modeDataKey),]
                else:
                    print 'Step "%s", mode %s not found or MAC value '\
                          'is not greater than %s'% (stepName, iMode, MAC_Value)

            # We cannot tell which mode did not have an entry.
            # Go through each one and make the number of entries the same 
            # for each, using the previous step entry
            #
            for key in modeCorrelation.keys():
                if stepNumber + 2 > len(modeCorrelation[key]):
                    modeCorrelation[key] += [(spinSpeed,
                                      modeCorrelation[key][-1][1],
                                      modeCorrelation[key][-1][2]),]
                    
            statusText = succeedMsg%(referenceStepName, stepName, "for MAC correlation")
            session.customData.campbell['odbs'][odbPath]['odbStatus'] += statusText
            print statusText
        #

        referenceStepName = stepName
        # increase number of steps
        #
        stepNumber+=1
    #
    # all steps done. Remove unnecessary index from modeCorrelation so we can plot them
    #
    for key in modeCorrelation.keys():
        modeCorrelation[key] = [(plot[0],plot[1]) for plot in modeCorrelation[key]]
    return modeCorrelation

# end: combineSteps
