from abaqusGui import *
from kernelAccess import mdb, session
from symbolicConstants import SymbolicConstant
ONEOCTAVE = SymbolicConstant('ONEOCTAVE')
THIRDOCTAVE = SymbolicConstant('THIRDOCTAVE')
import octaveData


###########################################################################
# Class definition
###########################################################################
class CreateSSDStepForm(AFXForm):
    def __init__(self, owner):
        AFXForm.__init__(self, owner)
        self.owner = owner

        self.cmd = AFXGuiCommand(mode=self, method='SteadyStateDirectStep',
                                 objectName='mdb.models[%s]')

        # kewwords sent with the command
        #
        self.stepNameKw = AFXStringKeyword(command=self.cmd, name='name',
                                           isRequired=True)

        self.previousNameKw = AFXStringKeyword(command=self.cmd,
            name='previous', isRequired=True)

        self.frequencyRangeKw = AFXTableKeyword(command=self.cmd,
            name='frequencyRange', isRequired=True)
        
        # keywords not sent with the command
        #
        self.octaveTypeKw = AFXSymConstKeyword(command=None,
            name='octaveType', isRequired=False,
            defaultValue=THIRDOCTAVE.getId())
        
        self.fromKw = AFXFloatKeyword(command=None, name='from',
                                      isRequired=False, defaultValue=16.)
        
        self.toKw = AFXFloatKeyword(command=None, name='to',
                                    isRequired=False, defaultValue=20000.)
        
        self.numPointsKw = AFXIntKeyword(command=None, name='numPoints',
                                         isRequired=False, defaultValue=30)
        self.biasKw=AFXFloatKeyword(command=None, name='Bias',
            isRequired=False, defaultValue=1)                                         
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        return CreateSSDStepDb(self)

    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def calculateRange(self):
        """Look up the frequency data from the tables in octaveData.py"""

        if self.octaveTypeKw.getValue() == ONEOCTAVE.getId():
            keys = octaveData.oneOctaveKeys
            dict = octaveData.oneOctaveDict
        else:
            keys = octaveData.thirdOctaveKeys
            dict = octaveData.thirdOctaveDict

        kwStr = ''
        fromVal = self.fromKw.getValue()
        toVal = self.toKw.getValue()
        numPoints = self.numPointsKw.getValue()
        Bias=self.biasKw.getValue()

        #Data lines for the SSD direct Step
        #
        for key in keys:
            if key >= fromVal and key <= toVal:
                data = dict[key]

                if key==toVal:
                    kwStr += '(%s,%s,%s,%s)' % (data[0], data[2], numPoints, Bias)
                else:
                    kwStr += '(%s,%s,%s,%s),' % (data[0], data[2], numPoints, Bias)

        self.frequencyRangeKw.setValues(kwStr)
        return kwStr
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        # get previous step name (always last step)
        #

        currentViewport = session.currentViewportName
        modelName = session.sessionState[currentViewport]['modelName']
        existingSteps = mdb.models[modelName].steps.keys()
        self.previousNameKw.setValue(existingSteps[-1])

        if self.stepNameKw.getValue() in existingSteps:
            showAFXErrorDialog(self.getCurrentDialog(),
                '"%s" already exists and cannot be overwritten.' % self.stepNameKw.getValue())
            return False

        if self.fromKw.getValue() >= self.toKw.getValue():
            showAFXErrorDialog(self.getCurrentDialog(),
                'Lower frequency value must be less than Upper frequency value.')
            return False

        if self.numPointsKw.getValue() < 2:
            showAFXErrorDialog(self.getCurrentDialog(),
                'Number of points must be greater than 2.')
            return False
        
        if self.biasKw.getValue()==0 or self.biasKw.getValue()<0:
            showAFXErrorDialog(self.getCurrentDialog(),
                'Bias parameter must be greater than zero.')
            return False
        
        kwStr = self.calculateRange()        
        if kwStr == '':
            showAFXErrorDialog(self.getCurrentDialog(),
                'No octave data found in given range.\nTry a different range.')
            return False
            
        return AFXForm.verifyKeywordValues(self)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def handleException(self, exc):
        """Overload the exception handling method"""

        errorType = exc[0]
        errorValue = exc[1]

        # Then post error dialog
        #
        showAFXErrorDialog(self.getCurrentDialog(), str(errorValue))
        
###########################################################################
# Class definition
###########################################################################
class CreateSSDStepDb(AFXDataDialog):

    def __init__(self, form):
        self.form = form
        
        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form,
                               title='Create SSD Direct Step',
                               actionButtonIds=self.OK|self.CANCEL,
                               opts=DATADIALOG_BAILOUT|DECOR_RESIZE|DIALOG_ACTIONS_SEPARATOR)
        
        #Populate unused step name in the text field
        #
        AFXTextField(p=self, ncols=10, labelText='Name:',
                     tgt=self.form.stepNameKw, sel=0, opts=LAYOUT_FILL_X)
        
        currentViewport = session.currentViewportName
        modelName = session.sessionState[currentViewport]['modelName']
        existingSteps = mdb.models[modelName].steps.keys()
        
        count=1        
        while 1:
            if 'Step_SSD'+str(count) in existingSteps:
                count+=1
                continue
            else:
                self.form.stepNameKw.setValue('Step_SSD'+str(count))
                break
        
        # Data group box and widgets
        #
        gb = FXGroupBox(p=self, text='Data',
                        opts=FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        hf = FXHorizontalFrame(p=gb, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)                        
        
        FXLabel(p=hf, text='Octave:', opts=JUSTIFY_LEFT)
        
        FXRadioButton(p=hf, text='One', tgt=self.form.octaveTypeKw, sel=ONEOCTAVE.getId())
        
        FXRadioButton(p=hf, text='One third', tgt=self.form.octaveTypeKw, sel=THIRDOCTAVE.getId())
        
        vf = AFXVerticalAligner(p=gb, opts=LAYOUT_FILL_X, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        
        AFXTextField(p=vf, ncols=12, labelText='Lower frequency:',
            opts=LAYOUT_FILL_X, tgt=self.form.fromKw, sel=0)
        AFXTextField(p=vf, ncols=12, labelText='Upper frequency:',
            opts=LAYOUT_FILL_X, tgt=self.form.toKw, sel=0)
        AFXTextField(p=vf, ncols=12, labelText='Number of points \nper octave bin:',
            opts=LAYOUT_FILL_X, tgt=self.form.numPointsKw, sel=0)    
        AFXTextField(p=vf, ncols=12, labelText='Bias parameter:',
            opts=LAYOUT_FILL_X, tgt=self.form.biasKw, sel=0)      
        AFXNote(p=gb, message='Frequency values outside of the\nrange 16 - 20000 Hz will be ignored.')
        
