from abaqus import *
from abaqusConstants import *
from dymolaConstants import *
import os, uti

def keywordPosUsingEq(modelName, searchString):
    """

    Method to find the index of a keyword search string in the
    sieBlocks object.
    Attributes:
        modelName - Name of the model.
        searchString - Keyword whose occurence is to be found.

    """
    searchIndex = []
    lcSearchString = searchString.lower()
    keyIndex = -1
    sieBlocksObj = mdb.models[modelName].keywordBlock.sieBlocks
    for keyBlocks in sieBlocksObj:
        keyIndex = keyIndex + 1
        blocks = keyBlocks.split('\n')
        for block in blocks:
            subBlocks = block.split(',')
            for subBlock in subBlocks:
                subBlock = subBlock.strip('')
                lcSubBlock = subBlock.lower()
                if lcSubBlock == lcSearchString:
                    searchIndex.append(keyIndex)
    return searchIndex

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def keywordPosUsingIndex(modelName, searchString):
    """

    Method to find the index of a case sensitive parameter
    search string in the sieBlocks object.

    """
    searchIndex = []
    keyIndex = -1
    fVal = -1
    sieBlocksObj = mdb.models[modelName].keywordBlock.sieBlocks
    for keyBlocks in sieBlocksObj:
        keyIndex = keyIndex + 1
        blocks = keyBlocks.split('\n')
        for block in blocks:
            subBlocks = block.split(',')
            for subBlock in subBlocks:
                fVal = subBlock.find(searchString)
                #Need for additional checks to get exact match
                if (fVal != -1):
                    try:
                        subList = subBlock.split('=')
                        subString = subList[1].strip('')
                        subString = subString.strip('"')
                        subString = subString.strip('')
                        if subString.lower() == searchString.lower():
                            searchIndex.append(keyIndex)
                    except:
                        pass
    return searchIndex

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def restoreKeywords(modelName):
    """
    Method to check for co-simulation keywords. If found replace with ''
    """
    keywordBlockObject = mdb.models[modelName].keywordBlock
    i=0
    for sb in keywordBlockObject.sieBlocks:
        if sb.lower().startswith('*co-simulation'):
           keywordBlockObject.replace(i,'')
        i=i+1
    keywordBlockObject.synchVersions()

    return 1

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def multiLineFunc(signalList):
    """
    Function to write data line or lines for any keyword.
    """
    dataList = []
    if len(signalList) < 8:
        cstring = ', '.join(signalList) + '\n'
        dataList.append(cstring)
    else:
        numAct = len(signalList) / 8
        remAct = len(signalList) % 8
        for i in range(numAct):
            tempList = signalList[i*8:(i*8) + 8]
            cstring = ', '.join(tempList) + ',\n'
            dataList.append(cstring)
        if remAct:
            tempList = signalList[numAct*8:]
            cstring = ', '.join(tempList) + '\n'
            dataList.append(cstring)

    return dataList

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def writeMapFile(jobName, stepName, senList, actList,
                 schmType, stepSize, dymSolver):
    """
    Utility to write mapping file.
    """

    dymSolvDict = {}
    dymSolvDict['Dassl'] = '8'
    dymSolvDict['Lsodar'] = '4'
    dymSolvDict['Euler'] = '11'
    dymSolvDict['Rkfix2'] = '12'
    dymSolvDict['Rkfix3'] = '13'
    dymSolvDict['Rkfix4'] = '14'

    mapFileName = jobName + '.sgn'
    modelName = session.viewports[session.currentViewportName].displayedObject.modelName
    fileObj = open(mapFileName, 'w')
    timePeriod = mdb.models[modelName].steps[stepName].timePeriod
    if schmType == 'VARIABLE':
        stepSize = timePeriod
    try:
        solvNo = dymSolvDict[dymSolver]
    except:
        solvNo = '8'

    writeStr = 'Analysis: ' + str(timePeriod) + ' ' + str(stepSize) + ' 1 ' + solvNo + '\n'
    fileObj.write(writeStr)

    fileObj.close()
    return mapFileName

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def runJobs(jobName, stepName, senList, actList,
            enforceType, schemeType, cosimStepTime,
            dymSolver, precision, nCpus, multiProc,
            jobType):
    """
    """

    senList = senList[:-1]
    actList = actList[:-1]



    modelName = session.viewports[session.currentViewportName].displayedObject.modelName
    currModel = mdb.models[modelName]
    keywordBlockObject = currModel.keywordBlock
    #Call the restore utility to undo any existing keyword definitions
    #
    restoreKeywords(modelName)
    keywordBlockObject.synchVersions(storeNodesAndElements = FALSE)
    searchString = '*Step'
    stepIndexList = keywordPosUsingEq(modelName, searchString)
    searchString = stepName
    snameIndex = keywordPosUsingIndex(modelName, searchString)
    #find if the step name corresponds to *step keyword
    #
    coStepIndex = 0
    for iStep in stepIndexList:
        for iSName in snameIndex:
            if iStep == iSName:
                coStepIndex = iSName
                break
        if coStepIndex:
            break

    searchString = '*Restart'
    resIndex = keywordPosUsingEq(modelName, searchString)
    if len(resIndex) == 0:
        errorMessage = 'Error: No *Restart definitions found '\
                       'in the model.\n'
        raise ValueError, errorMessage

    #Generating a string for co-simulation keywords that will be inserted in the
    #keyword block
    #
    cosimList = []
    cstring = '*CO-SIMULATION, NAME=' + jobName + ', PROGRAM=MULTIPHYSICS, CONTROLS=co-ctrl\n'
    cosimList.append(cstring)
    cstring = '*CO-SIMULATION REGION, TYPE=DISCRETE, IMPORT\n'
    cosimList.append(cstring)
    importList = multiLineFunc(actList)
    cstring = ''.join(importList)
    cosimList.append(cstring)
    cstring = '*CO-SIMULATION REGION, TYPE=DISCRETE, EXPORT\n'
    cosimList.append(cstring)
    exportList = multiLineFunc(senList)
    cstring = ''.join(exportList)
    cosimList.append(cstring)

    cstring = '*CO-SIMULATION CONTROLS, NAME=co-ctrl, '
    if schemeType == 'FIXED':
        tString = 'STEP SIZE=' + str(cosimStepTime) + ', '
        cstring += tString
    cstring += 'TIME INCREMENTATION=SUBCYCLE, '
    if enforceType == 'EXACT':
        cstring += 'TIME MARKS=YES\n'
    else:
        cstring += 'TIME MARKS=NO\n'
    cosimList.append(cstring)
    cosimString = ''.join(cosimList)

    for rindex in resIndex:
        if rindex > coStepIndex:
            try:
                keywordBlockObject.insert(position=rindex - 2,
                                          text=cosimString)
            except:
                errorMessage = 'Error in adding *Co-simulation data to '\
                               'keywordBlock object.\n'
                restoreKeywords(modelName)
                raise ValueError, errorMessage
            break

    mapFileName = writeMapFile(jobName, stepName, senList, actList,
                               schemeType, cosimStepTime, dymSolver)

    description = 'Abaqus to Dymola Co-simulation'
    mdb.Job(name=jobName, model=modelName, description=description)
    mdb.jobs[jobName].writeInput(consistencyChecking=OFF)

    if jobType == WRITE_INPUT:
        print "Input file '%s' written to '%s'" %(jobName, os.getcwd())
        return 1

    milestone('Getting the Abaqus executable...')
    try:
        abqCmd = os.environ['ABA_COMMAND']
    except KeyError:
        abqCmd = 'abq'

    import random
    portNo = random.randint(4e4,7e4)

    if os.sys.platform.startswith('win'):
        machineName = os.getenv('COMPUTERNAME')
    else:
        machineName = os.getenv('HOSTNAME')


    milestone('Submitting Dymola job...')
    print 'Submitting Dymola job...'

    dymCmd = '%s dymola -input "%s" -port %s' %(abqCmd, mapFileName[:-4], portNo)


    uti.Spawn(dymCmd)

    milestone('Submitted Dymola job...')
    print 'Submitted Dymola job...'
    milestone('Submitting Abaqus job...')
    print 'Submitting Abaqus job...'
    if jobType == FULL_ANALYSIS:

        abaCmd = '%s -j "%s" -ask_delete off -port %s -host %s' %(abqCmd, jobName, portNo, machineName)
    elif jobType == DATACHECK:
        abaCmd = '%s -j "%s" datacheck -ask_delete off -port %s -host %s' %(abqCmd, jobName, portNo, machineName)

    if precision == "Double":
        abaCmd += ' -double explicit'

    if nCpus > 1 and multiProc == 1:
        abaCmd += ' -cpus %s' %(str(nCpus))


    uti.Spawn(abaCmd)
    milestone('Submitted Abaqus job...')
    print 'The job input file "%s" has been submitted for analysis.' %jobName
    del mdb.jobs[jobName]
