import string
from abaqus import *
from abaqusConstants import *
from caeModules import *
from engFeaturesSymConsts import *
import regionToolset

try:
   from Numeric import *
except:
   pass


MPC_LIB = {BEAM_MPCS.getId():BEAM_MPC,
           TIE_MPCS.getId():TIE_MPC,
           LINK_MPCS.getId():LINK_MPC,
           PIN_MPCS.getId():PIN_MPC,
           ELBOW_MPCS.getId():ELBOW_MPC}
MPC_NAME = {BEAM_MPC:"BEAM", TIE_MPC:"TIE", LINK_MPC:"LINK",
            PIN_MPC:"PIN", ELBOW_MPC:"ELBOW"}


def main(featType, featName, nset1Name, nset2Name, wireSet=False):
    """
    Create 2-noded entities connecting every node in node set nset1Name
    with the closest node in node set nset2Name.  If featType is
    SPRINGS_DASHPOTS, the entities are to be springs/dashpots in a single
    engineering feature.  If featType is WIRES_FOR_CONNECTORS, the entities
    are to be wires that can be used for connectors, and will all be part
    of a wire feature.  A wire set will also be created if wireSet==True.
    If featType is any of BEAM_MPCS, TIE_MPCS, LINK_MPCS, PIN_MPCS, or
    ELBOW_MPCS, the entities are to be MPCs of the given type.

    This function checks the input for validity and fetches model parameters
    before sending everything to createFeats() which creates the new entities.

    Input:
        featType                SymConst    ID of SPRINGS_DASHPOTS, BEAM_MPCS,
                                            TIE_MPCS, LINK_MPCS, PIN_MPCS,
                                            ELBOW_MPCS or WIRES_FOR_CONNECTORS
        featName                string      Feature name prefix
        nset1Name               string      Name of first node set
        nset2Name               string      Name of second node set
        wireSet                 bool        If True, create a wire set.
                                            Ignore this if
                                            featType != WIRES_FOR_CONNECTORS.

    Return: None
    Unhandled Exceptions: ValueError
    """

    curVPName = session.currentViewportName
    curVP = session.viewports[curVPName]
    modelName = curVP.displayedObject.modelName
    model = mdb.models[modelName]
    assy = model.rootAssembly
    curAssyDispGeoOptions = curVP.assemblyDisplay.geometryOptions

    for setName in nset1Name, nset2Name:
        if setName not in assy.sets.keys():
            raise ValueError, 'Node set "%s" does not exist in the assembly.' \
                              % setName
        # assy.sets[setName].instances is an undocumented (as of Abaqus 2016)
        # array of integers, apparently the number of entities from each
        # instance.
        elif len(assy.sets[setName].instances) > 1:
            msg = 'Node set "%s" contains nodes from a mixture of ' + \
                  'instances.\nThis is currently not supported.' % setName
            raise ValueError, msg
        elif not len(assy.sets[setName].nodes):
            raise ValueError, 'Node set "%s" contains zero nodes.' % setName
    createFeats(featName, nset1Name, nset2Name, featType, wireSet, modelName,
                curAssyDispGeoOptions)
    return 0


def meanXYZ(xyz1, xyz2): 
    """Find the midpoint between 2 points."""
    xyz = []
    for p1,p2 in zip(xyz1, xyz2):
        xyz.append( 0.5*(p1+p2) )
    return xyz


def getAssySetNameFromRoot(assy, root):
    """
    Get the next eligible assembly set name that starts with the string
    in root.
    """
    i = 1
    while True:
        name = root + str(i)
        if assy.sets.has_key(name):
            i += 1
        else:
            return name


def findClosestNodesToSet(nset1Name, nset2Name, assy):
    """
    Given two node set names and the assembly object, compute the node in
    node set 2 closest to each node in node set 1, and return the result
    in 'match', a dictionary with nset1Name node label as key and nset2Name
    node label as value.  This function assumes that the node sets exist.
    """
    match = {}
    for n1 in assy.sets[nset1Name].nodes:
        n1_coord = n1.coordinates
        n1_label = n1.label
        first = True
        for n2 in assy.sets[nset2Name].nodes:
            n2_coord = n2.coordinates
            n2_label = n2.label
            dx = n2_coord[0] - n1_coord[0]
            dy = n2_coord[1] - n1_coord[1]
            dz = n2_coord[2] - n1_coord[2]
            r = sqrt(dx*dx + dy*dy + dz*dz)
            if first:
                minValue = r
                minLabel = n2_label
                first = False
            elif r < minValue:
                minValue = r
                minLabel = n2_label
        match[n1_label] = minLabel
    return match


def createFeats(featName, nset1Name, nset2Name, featType, wireSet, modelName,
                curAssyDispGeoOptions):
    """
    Create 2-noded entities connecting every node in node set nset1Name
    with the closest node in node set nset2Name.  If featType is
    SPRINGS_DASHPOTS, the entities are to be springs/dashpots in a single
    engineering feature.  If featType is WIRES_FOR_CONNECTORS, the entities
    are to be wires that can be used for connectors, and will all be part
    of a wire feature.  A wire set will also be created if wireSet==True.
    If featType is any of BEAM_MPCS, TIE_MPCS, LINK_MPCS, PIN_MPCS, or
    ELBOW_MPCS, the entities are to be MPCs of the given type.

    Input:
        featName                string      Feature name prefix
        nset1Name               string      Name of first node set
        nset2Name               string      Name of second node set
        featType                SymConst    ID of SPRINGS_DASHPOTS, BEAM_MPCS,
                                            TIE_MPCS, LINK_MPCS, PIN_MPCS,
                                            ELBOW_MPCS or WIRES_FOR_CONNECTORS
        wireSet                 bool        If True, create a wire set.
                                            Ignore this if
                                            featType != WIRES_FOR_CONNECTORS.
        modelName               string      Model Name
        curAssyDispGeoOptions   geometryOptions object for assembly display

    Return: None
    """
    model = mdb.models[modelName]
    assy = model.rootAssembly
    match = findClosestNodesToSet(nset1Name, nset2Name, assy)
    
    # Sets should contain nodes from one instance only.
    # Take instance name from first object
    #
    instanceName1 = assy.sets[nset1Name].nodes[0].instanceName
    instanceName2 = assy.sets[nset2Name].nodes[0].instanceName
    inst1Nodes = assy.instances[instanceName1].nodes
    inst2Nodes = assy.instances[instanceName2].nodes
    
    # Generate the correct elements/constraints
    #
    if featType.getId() == SPRINGS_DASHPOTS.getId():
        regions = []
        for j in match:
            node1 = inst1Nodes[j-1:j]   # Region constructor needs a sequence
            node2 = inst2Nodes[match[j]-1:match[j]]
            region1 = regionToolset.Region(nodes=node1)
            region2 = regionToolset.Region(nodes=node2)
            regions.append((region1, region2))
        # Create unique name
        #
        i = 1
        springDashpots = assy.engineeringFeatures.springDashpots
        while featName+str(i) in springDashpots.keys():
            i+=1
        
        sd = assy.engineeringFeatures.TwoPointSpringDashpot(
            name=featName+str(i), regionPairs=tuple(regions), axis=NODAL_LINE,
            springBehavior=ON, springStiffness=1.0, dashpotBehavior=OFF,
            dashpotCoefficient=0.0)
        msg = "Spring/Dashpot Engineering Feature '%s' has been created " + \
              "with %d spring/dashpot elements.  You may want to edit the " + \
              "feature to specify a non-default stiffness or damping " + \
              "constant or both."
        print msg % (sd.name, len(match))
    elif featType.getId() == WIRES_FOR_CONNECTORS.getId():
        nWires = len(match.keys())
        if curAssyDispGeoOptions.referencePointLabels == ON or \
           curAssyDispGeoOptions.referencePointSymbols == ON:
            curAssyDispGeoOptions.setValues(referencePointLabels=OFF,
                                            referencePointSymbols=OFF)
            msg = "Display of Reference Points has been disabled in order " + \
                  "to minimize graphical clutter from %d new Reference " + \
                  "Points.  You can re-enable their display using the " + \
                  "Datum tab of the Assembly Display Options dialog box."
            print msg % (nWires*2)

        # Wires do not connect to Regions, but to Reference Points.
        # Reference points must be created at the ends of each wire.
        rpPairs = []
        midPts = []
        refPts = assy.referencePoints
        for j in match.keys():
            node1 = inst1Nodes[j-1]
            node2 = inst2Nodes[match[j]-1]
            if wireSet:
                midXYZ = meanXYZ(node1.coordinates, node2.coordinates)
                midPts.append(midXYZ)
            rp1 = assy.ReferencePoint(point=node1)
            rp2 = assy.ReferencePoint(point=node2)
            rpPairs.append( (refPts[rp1.id], refPts[rp2.id]) )
        w = assy.WirePolyLine(points=rpPairs, mergeType=IMPRINT, meshable=OFF)
        print ("Wire Feature '%s' for connectors, with %d Wires and %d Ref" +
               "erence Points, has been created.") \
              % (w.name, nWires, nWires*2)
        if wireSet:
            e1 = assy.edges
            edges1 = e1.findAt(coordinates=midPts)
            setRootName = w.name + '-Set-'
            setName = getAssySetNameFromRoot(assy, setRootName)
            assy.Set(edges=edges1, name=setName)
            print "Wire Set %s has been created." % setName
    else:
        type = MPC_LIB[featType.getId()]
        for j in match:
            node1 = inst1Nodes[j-1:j]
            node2 = inst2Nodes[match[j]-1:match[j]]
            region1 = regionToolset.Region(nodes=node1)
            region2 = regionToolset.Region(nodes=node2)

            # This creates constraints whose names begin with the string in
            # featName, but they are not features.
            model.MultipointConstraint(name=featName+str(j),
                                       controlPoint=region1, surface=region2,
                                       mpcType=type, csys=None)
        print "%d %s MPCs have been created." % (len(match), MPC_NAME[type])
    return 0
