"Form for Butterworth high-pass filter"

import highPassDB
from abaqusGui import *
from kernelAccess import session

# This function checks if two values are same within a given
# relative tolerance
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def areTwoValuesAreSame(value1, value2, tol=1E-6): # tol is arbitrily chosen
    "This function checks if two values are same within a given relative tolerance"
    
    if abs(value1) <= 1E-16:  # Any value smaller than 1E-16 in magnitude is considered as zero
        if abs(value2) <= 1E-16:
            return True
        else:
            return False
    elif abs(value2) == 1E-16:
        return False
    if abs((value1 - value2) / value1) < tol:
        return True
    else:
        return False

###########################################################################
# Class definition
###########################################################################

class HighPassForm(AFXForm):
    "The class for storing the form for Butterworth high-pass filter"

    [ID_WARNING_SPACING, ] = range(AFXForm.ID_LAST, AFXForm.ID_LAST+1)

    # The constructor for the form
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
        "The constructor for the form"
        
        AFXForm.__init__(self, owner)
        FXMAPFUNC(self,
                  SEL_COMMAND,
                  self.ID_WARNING_SPACING,
                  HighPassForm.onCmdWarningSpacing)
        
        cmd = AFXGuiCommand(mode=self, method='createFilteredXYData', objectName='abq_HighPass.highPassKernel')

        # Keywords
        self.inpXYKw = AFXStringKeyword(command=cmd,
                                        name='inpxyname',
                                        isRequired=TRUE)
        
        self.outXYKw = AFXStringKeyword(command=cmd,
                                        name='outxyname',
                                        isRequired=TRUE)
        
        self.fcKw = AFXFloatKeyword(command=cmd,
                                    name='fc',
                                    isRequired=TRUE)

        self.displayWarningSpacing = True
        self.kernelImported = False

    # Getfirst Dialog function
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        "Getfirst dialog function"
        
        return highPassDB.HighPassDB(self)

    # This function validates the user input
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):
        "This function validates the use input"

        db = self.getCurrentDialog()
        inpxyname = self.inpXYKw.getValue()
        outxyname = self.outXYKw.getValue()
        fc = self.fcKw.getValue()

        # Valiadtion Steps
        xynames = session.xyDataObjects.keys()

        # inpxyname validation
        if inpxyname == '':
            showAFXErrorDialog(db,
                               "Input XYData name must be specified.")
            return FALSE
        
        if inpxyname not in xynames:
            showAFXErrorDialog(db,
                               "The input XYData does not exist.")
            return FALSE

        # outxyname validation
        if outxyname == '':
            showAFXErrorDialog(db,
                               "Output XYData name must be specified.")
            return FALSE

        # Check to see if input and output names are not same
        if outxyname == inpxyname:
            showAFXErrorDialog(db,
                               "Input and Output XYData cannot be same.")
            return FALSE

        # Input Validation
        # Access to attributes of attributes may not be possible in GUI script for version 6.6
        from highPass_plugin import verAbaqus
        if verAbaqus > 6.6:
            data = session.xyDataObjects[inpxyname].data
        else:
            sendCommand("import customKernel") 
            sendCommand("session.customData.data = session.xyDataObjects['"\
                        + str(inpxyname) + "'].data")
            data = session.customData.data
            
        ld = len(data)
        if ld < 2:
            showAFXErrorDialog(db,
                               "At least two data values must be present in the XYData.")
            return FALSE

        # fc validation
        if fc <= 0:
            showAFXErrorDialog(db,
                               "The cut-off frequency must be greater than zero.")
            return FALSE

        # dt validation
        numData = len(data)
        dt = (data[-1][0] - data[0][0]) / (numData - 1)
        if dt <= 0:
            showAFXErrorDialog(db,
                               "The time-increment must be greater than zero.")
            return FALSE

        # Constant time increment warning
        if self.displayWarningSpacing:
            for index in range(numData - 1):
                if not areTwoValuesAreSame(data[index + 1][0] - data[index][0], dt):
                    self.warningBoxSpacing = showAFXDismissableWarningDialog(db,
                                             "The X values in XYData should be uniformly spaced.",
                                             tgt=self,
                                             sel=self.ID_WARNING_SPACING)
                    return FALSE
            
        return TRUE

    # This function handles the uniform time spacing warning
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdWarningSpacing(self, sender, sel, ptr):
        "This function handles the time-spacing warnings"

        if self.displayWarningSpacing:
            if not sender.getCheckButtonState():
                self.displayWarningSpacing = False
        self.issueCommands()

    # This function plots the curves for XYData after Kernel is executed
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomTasks(self):
        "This function plots the curves for XYData after Kernel is executed"
        
        inpxyname = self.inpXYKw.getValue()
        outxyname = self.outXYKw.getValue()

        # Avoid reloading the Kernel module a number of times
        if not self.kernelImported:
            sendCommand("import abq_HighPass.highPassKernel")
            self.kernelImported = True
            
        sendCommand("abq_HighPass.highPassKernel.plotXYData(inpxyname='" + inpxyname + "', outxyname='" + outxyname + "')")
