"Kernel to perform Butterworth high-pass filtering of the XY-Data"

try:
    import math, Numeric
except:
    import numpy.oldnumeric as Numeric
from abaqus import *

# This function pre-processes the input to return dt
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getDt(inpxyname):
    "This function pre-processes the input to return dt"

    data = session.xyDataObjects[inpxyname].data
    numData = len(data)
    dt = (data[-1][0] - data[0][0]) / (numData - 1)
    return dt

# High-pass filter for the input
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def highPass(inpxyname, fc):
    "This function takes the input XYData with the name inpxyname and processes it according to the plugin"

    dt = getDt(inpxyname)

    pi = math.pi
    data = session.xyDataObjects[inpxyname].data
    NPTS = len(data)
    t  = [tmp[0] for tmp in data]
    x = [tmp[1] for tmp in data]
    y1 = Numeric.array(range(4*NPTS), Numeric.Float)
    y  = Numeric.resize(y1, (4, NPTS))

    # digital frequency [prewarped]
    w = math.tan(pi * fc * dt)

    # Filter coefficients
    # Order
    L = 4
    # Number of sections
    NS = int(L / 2)

    for k in range(NS):
        y[k][0] = x[0]
    y[k][1] = x[0]

    for k in range(NS):
        # each section is a 2nd order butterworth;
        # the analog coefficients;
        d0 = 1.0
        d1 = 0.0
        d2 = 0.0
        c0 = 1.0
        c1 =-2.0 * math.cos((2.0 * (k + 1) + L - 1) * pi / (2.0 * L))
        c2 = 1.0

        # scale to s/w
        d0 = d0 / (w * w)
        d1 = d1 / w
        d2 = d2
        c0 = c0 / (w * w)
        c1 = c1 / w
        c2 = c2

        # High Pass
        denom = c2 + c1 + c0
        b0 = (d2 + d1 + d0)/denom
        b1 = 2.0 * (d2 - d0)/denom
        b2 = (d2 - d1 + d0)/denom
        a1 = 2.0 * (c2 - c0)/denom
        a2 = (c2 - c1 + c0)/denom

        # finite difference formula
        if k == 0:
            for j in range(NPTS-2):
                i = j + 2
                y[k][i] = b0 * x[i] + b1 * x[i - 1] + b2 * x[i - 2] - a1 * y[k][i - 1] - a2* y[k][i - 2]
        else:
            for j in range(NPTS-2):
                i = j + 2
                y[k][i] = b0 * y[k - 1][i] + b1 * y[k - 1][i - 1] + b2 * y[k - 1][i - 2] - a1 * y[k][i - 1] - a2 * y[k][i - 2]

    return zip(t, y[1])

# Function to create XYData for the input XYData
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createFilteredXYData(inpxyname, outxyname, fc):
    "This function creates an XYData with the filtered output"

    import uti

    inpdata = highPass(inpxyname, fc)

    # Check to see if the XYData was overwritten
    overwriteFlag = False
    if outxyname in session.xyDataObjects.keys():
        overwriteFlag = True

    inpxydata = session.xyDataObjects[inpxyname]
    outxydata = session.XYData(name=outxyname,
                               data=inpdata,
                               sourceDescription = "Output from high pass filter",
                               contentDescription = inpxydata.contentDescription,
                               positionDescription = inpxydata.positionDescription,
                               legendLabel = outxyname,
                               xValuesLabel = inpxydata.xValuesLabel,
                               yValuesLabel = inpxydata.yValuesLabel,
                               )

    # Version 6.6 does not support axisQuantityType
    if not (uti.getVersion()).startswith('6.6'):
        outxydata.setValues(axis1QuantityType = inpxydata.axis1QuantityType)
        outxydata.setValues(axis2QuantityType = inpxydata.axis2QuantityType)

    if overwriteFlag:
        print "The XYData '" + outxyname + "' was overwritten"
    else:
        print "The XYData '" + outxyname + "' has been created by the plug-in."

    return outxydata

# This function plots the curves of XYData
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def plotXYData(inpxyname, outxyname):
    "This function plots the XYData"

    import uti

    xyPlotNames = session.xyPlots.keys()
    vpName = session.currentViewportName
    vp = session.viewports[vpName]

    # If no xyplots exist then create one
    if len(xyPlotNames) == 0:
        xyp = session.XYPlot('XYPlot-1')
    else:
        # If an XYPlot is on the current viewport then replace it
        # Otherwise the last available plot
        if str(type(vp.displayedObject)) == "<type 'XYPlot'>":
            xyp = vp.displayedObject
        else:
            xyp = session.xyPlots[xyPlotNames[-1]]

    # Differences between 6.6 and 6.7
    if (uti.getVersion()).startswith('6.6'):
        xy1 = session.xyDataObjects[inpxyname]
        xyp.Curve(inpxyname, xy1)
        xy2 = session.xyDataObjects[outxyname]
        xyp.Curve(outxyname, xy2)
        xyp.setValues(curvesToPlot=(inpxyname, outxyname, ), )
    else:
        chartName = xyp.charts.keys()[0]
        chart = xyp.charts[chartName]
        xy1 = session.xyDataObjects[inpxyname]
        c1 = session.Curve(xyData=xy1)
        xy2 = session.xyDataObjects[outxyname]
        c2 = session.Curve(xyData=xy2)
        chart.setValues(curvesToPlot=(c1, c2, ), )

    vp.setValues(displayedObject=xyp)
