"""
calculate mac values based on complex criteria only
"""
#

#
# abaqus imports
from abaqus import *
from abaqusConstants import *

try:
    import numpy
except:
    import Numeric as numpy
#
import math
import os,sys

from time import clock

#from abq_CampbellDiagram.Exec import createResultsForViewer

# be verbose if
BLK1 = ' '*4
BLK2 = ' '*8
NLK3 = ' '*12

_verbose = False
_namePrefix = __name__.split('.')[-1]
_verboseBase = BLK2+_namePrefix+':'

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def extractNodeList(stepA, stepB, nodeSet=None):
    """
    extract the list of nodes to access from stepA

    @param stepA     ODB Step object the first step to read data from
    @param stepB     ODB Step object the second step to read data from
    @param nodeSet   ODB_nset if not None, the values are only taken for this set.

    @return swap, takeList
    """
    # Open odb
    #
    if _verbose:
        startTime = clock()
        print '\n',BLK1,'Start Node list extraction: time = %6.3f'%(startTime)
    #
    # Read values from odb for both steps
    valA = stepA.frames[-1].fieldOutputs['U']
    valB = stepB.frames[-1].fieldOutputs['U']

    if nodeSet:
        valA = valA.getSubset(region=nodeSet).values
        valB = valB.getSubset(region=nodeSet).values
    else:
        valA = valA.values
        valB = valB.values
    #
    # Check that val A is bigger than B, swap if needed
    #
    swap = False
    if len(valA) < len(valB):
        valA,valB = valB,valA
        swap = True
    #
    # Check which B node labels exist in A and add it to takeList
    #
    if _verbose:
        endTime = clock()
        print BLK2,'Start check takelist: time = %6.3f'%(endTime)
    #
    if nodeSet:
        takeList = range(len(valA))
    else:
        i=0
        j=0
        takeList=[]
        for a in valA:
            if i >= len(valB): break
            nLabel = a.nodeLabel
            if nLabel == valB[i].nodeLabel:
                takeList.append(j)
                i+=1
            j+=1
    #
    if _verbose:
        endTime = clock()
        print BLK2,'->end check takelist: time = %6.3f'%(endTime)
    #
    return swap, takeList, valA, valB


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def accessNodeData(takeList, stepA, stepB, nodeSet=None,
                   numberOfModes=None):
    """
    read displacements form given steps.
    assume both steps are complex, because checking was done before

    @param takeList      LIST of node indices to use.
    @param stepA         ODB Step object the first step to read data from
    @param stepB         ODB Step object the second step to read data from
    @param nodeSet       ODB_nset if not None, the values are only taken for this set.
    @param numberOfModes INT defines how many modes to take from one frequency
    """
    # 
    #
    if _verbose:
        startTime = clock()
        print '\n',BLK1,'Start Access Node data: time = %6.3f'%(startTime)
    #
    # Check for complex or not:
    if stepA.procedure.find('COMPLEX') > -1:
        isStepAComplex = True
    else:
        isStepAComplex = False
    #
    if stepB.procedure.find('COMPLEX') > -1:
        isStepBComplex = True
    else:
        isStepBComplex = False
    #
    isStepComplex = isStepAComplex or isStepBComplex
    #
    #
    #
    # Retrieve values and take only the necessary node labels
    # Note we filter only frames (modes) that contain 'Increment'
    # in the description.
    if nodeSet:
        if isStepAComplex:
            arrayA = numpy.array([[(v.data,v.conjugateData)
                               for v in f.fieldOutputs['U'].getSubset(region=nodeSet).values]
                               for f in stepA.frames if f.description[:9] != 'Increment'])
        else:
            arrayA = numpy.array([[(v.data,len(v.data)*[0])
                               for v in f.fieldOutputs['U'].getSubset(region=nodeSet).values]
                               for f in stepA.frames if f.description[:9] != 'Increment'])
        #
        if isStepBComplex:
            arrayB = numpy.array([[(v.data,v.conjugateData)
                               for v in f.fieldOutputs['U'].getSubset(region=nodeSet).values] 
                               for f in stepB.frames if f.description[:9] != 'Increment'])
        else:
            arrayB = numpy.array([[(v.data,len(v.data)*[0])
                               for v in f.fieldOutputs['U'].getSubset(region=nodeSet).values] 
                               for f in stepB.frames if f.description[:9] != 'Increment'])
   
    else:
        if isStepAComplex:
            arrayA = numpy.array([[(v.data,v.conjugateData)
                               for v in f.fieldOutputs['U'].values]
                               for f in stepA.frames if f.description[:9] != 'Increment'])
                     
        else:
            
            
            arrayA = numpy.array([[(v.data,len(v.data)*[0])
                               for v in f.fieldOutputs['U'].values]
                               for f in stepA.frames if f.description[:9] != 'Increment'])
	    	
        if isStepBComplex:
            arrayB = numpy.array([[(v.data,v.conjugateData)
                               for v in f.fieldOutputs['U'].values] 
                               for f in stepB.frames if f.description[:9] != 'Increment'])
        else:
            arrayB = numpy.array([[(v.data,len(v.data)*[0])
                               for v in f.fieldOutputs['U'].values] 
                               for f in stepB.frames if f.description[:9] != 'Increment'])
        #
    arrayA = numpy.take(arrayA,takeList,1)
    #
    # Take number of modes from the odb that has the fewest
    # and take only those modes from each array
    #
    matrixLen = min(len(arrayA),len(arrayB))
  
    arrayA = arrayA[:matrixLen]
    arrayB = arrayB[:matrixLen]


    #
    if _verbose:
        endTime = clock()
        print BLK1,'->End Access Node data: time = %6.3f\n'%(endTime)

    return arrayA, arrayB, isStepComplex


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def rearrangeNodeMass(nMass, takeList, valA, nodeSet=None):
    """
    Convert nMass dictionary into a list of mass by node.
    assume both steps are complex, because checking was done before

    @param nMass     DICT with mass by node label.
    @param takeList  LIST of indices to take from valA
    @param valA      ODB_FieldOutputValues of  first step to read data from
    @param nodeSet   ODB_nset if not None, the values are only taken for this set.
    """
    # 
    #
    if _verbose:
        print BLK2,'in rearrangeNodeMass()'
    #
    #
    if nodeSet:
        nodeMassList = [nMass[n.label] for n in nodeSet.nodes]
    else:
        nodeMassList = []
        for i in takeList:
            nLabel = valA[i].nodeLabel
            nodeMassList += [nMass[nLabel]]
    #
    return nodeMassList

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def rescaleMacMatrix(macMatrix, odbStepNorms, nameAStep, nameBStep):
    """
    'Normalize' Mac Matrix based on odbStepNorms

    @param macMatrix    LIST of LIST inlcuding MAC values.
    @param odbStepNorms DICT of LISt including float values as rescaling factors.
    @param nameAStep    STRING name of the first step to read data from
    @param nameBStep    STRING name of the second step to read data from
    """
    # 
    #
    if _verbose:
        print BLK2,'in rescaleMacMatrix()'
    #
    matrixLen = len(macMatrix)
    #
    # Rescale macMatrix:
    for iRow in range(matrixLen):
        stepANorm = odbStepNorms[nameAStep][iRow]
        for iCol in range(matrixLen):
            stepBNorm = odbStepNorms[nameBStep][iCol]
            macMatrix[iRow][iCol] *= stepANorm*stepBNorm

    return macMatrix

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def _normalize(arrayA):
    """
    calculate the norm ||n||2 of the given array by doing for all entries.
    assume to have the dimensions: (numberOfFrames),(numberOFValues)(real+jcomp)

    @param arrayA ARRAY of ARRAY of TUPLE [(displ, complexDispl),] for first step

    @return normalized 
    """
    lenValues = len(arrayA[0])
    # 
    # loop through the frames:
    fullNorm = []
    for row in arrayA:
        oneNorm = 0.0
        for column in row:
            oneNorm += vNorm3(column)
        fullNorm += [math.sqrt(oneNorm)/lenValues]
    #
    return fullNorm


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def vNorm3(complecVect1):
    """
    return vector1 / the norm ||vector1||2
    """
    _norm = 0.0
    vReal,vComplex = complecVect1
    for i in (0,1,2):
        _norm += vReal[i]**2+vComplex[i]**2
    #
    return _norm
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def _calcMacWithMass(a, b, i, massList):
    """
    calculate the mac value for displacements for local vectors

    @param a TUPLE [(displ, complexDispl),] for first step
    @param b TUPLE [(displ, complexDispl),] for second step
    @param massList  LIST of  float with actual mass

    @return numA,numB,denA,denB
    """
    aRe,aIm = a[i]
    bRe,bIm = b[i]
    mass = massList[i]
    # Find dot product of  components
    #
    #
    numAB  = _calcMacAbqtwoValues(aRe, aIm, bRe, bIm)
    numA = numAB[0]*mass
    numB = numAB[1]*mass
    denA = _calcMacAbqoneValue(aRe, aIm)*mass
    denB = _calcMacAbqoneValue(bRe, bIm)*mass
    return (numA,numB,denA,denB)
# end: _calcMacWithMass


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def _calcMacWithoutMass(a, b, i, massList):
    """
    calculate the mac value for displacements for local vectors

    @param a TUPLE [(displ, complexDispl),] for first step
    @param b TUPLE [(displ, complexDispl),] for second step
    @param massList    always ununsed

    @return numA,numB,denA,denB
    """
    aRe,aIm = a[i]
    bRe,bIm = b[i]
    # Find dot product of  components
    #
    #
    numAB  = _calcMacAbqtwoValues(aRe, aIm, bRe, bIm)
    numA = numAB[0]
    numB = numAB[1]
    denA = _calcMacAbqoneValue(aRe, aIm)
    denB = _calcMacAbqoneValue(bRe, bIm)
    return (numA,numB,denA,denB)
# end: _calcMacWithoutMass


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def _calcMacRealOnlyWithoutMass(a, b, i, massList):
    """
    calculate the mac value for displacements for local vectors

    @param a TUPLE [(displ, complexDispl),] for first step
    @param b TUPLE [(displ, complexDispl),] for second step
    @param massList    always ununsed

    @return numA,numB,denA,denB
    """
    aRe,aIm = a[i]
    bRe,bIm = b[i]
    # Find dot product of  components
    #
    numA = numpy.dot(aRe, bRe)
    numB = 0.0
    denA = numpy.dot(aRe, aRe)
    denB = numpy.dot(bRe, bRe)
    return (numA,numB,denA,denB)
# end: _calcMacRealOnlyWithoutMass


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def _calcMacAbqtwoValues( aRe,aIm,bRe,bIm):
    """
    Calculate macx for individual entry

    Use formula:
    (mu1* times mu2)  = A+jB
    (mu1  times mu2*) = A-jB
    
    with: A,B real and imaginary part of result
    A = mu1[re] mu2[re] + mu1[im] mu2[im]
    B = mu1[re] mu2[im] - mu1[im] mu2[re]
    
    @param aRe ARRAY real part of vector a
    @param aIm ARRAY imaginary part of vector a
    @param bRe ARRAY real part of vector b
    @param bIm ARRAY imaginary part of vector b

    @return v1Real,v1Comp, real and imaginary part of result
    """
    #
    re1re2 = numpy.dot(aRe, bRe)
    im1im2 = numpy.dot(aIm, bIm)
    re1im2 = numpy.dot(aRe, bIm)
    im1re2 = numpy.dot(aIm, bRe)
    #
    v1Real = (re1re2+im1im2)
    v1Comp = (re1im2-im1re2)
    #
    return (v1Real,v1Comp)


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def _calcMacAbqoneValue( aRe,aIm):
    """
    Calculate macx for individual entry

    Use formula:
    (mu1* times mu1)  =  (mu1  times mu1*)  =  mu[re]**2 + mu[im]**2
    
    @param aRe ARRAY real part of vector a
    @param aIm ARRAY imaginary part of vector a

    @return v1ConjuV1
    """
    #
    re1re1 = numpy.dot(aRe, aRe)
    im1im1 = numpy.dot(aIm, aIm)
    #
    #v1ConjuV1 = re1re1*re1re1+im1im1*im1im1
    v1ConjuV1 = re1re1+im1im1
    #
    return v1ConjuV1


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def _extractMassValues(odb, nameAStep):
    """
    Calculate Mass matrix, if possible
    Extract the element volume from the very first frame only!
    If element volume is available, try to get the density
      by section assignement region.
    if any of the above fails, return a False as status
    #
    
    @param odb       ABQ ODB obejct
    @param nameAStep STRING name of the first step for comparison.

    @return status, nodeMassDict
    """

    status = False

    if not odb.rootAssembly.instances.has_key("PART-1-1") :
        return False, {}
    #
    stepA = odb.steps[nameAStep]
    if not stepA.frames[0].fieldOutputs.has_key("EVOL"):
        return False, {}
    #
    evol = stepA.frames[0].fieldOutputs['EVOL']
    inst1 = odb.rootAssembly.instances['PART-1-1']
    #
    try:
        elConnec = [(el.label,el.connectivity) for el in inst1.elements]
        elConnec = dict([(_label,(1./len(conn),)+conn) for _label,conn in elConnec])
        nodeDict = dict([(no.label,0.0) for no in inst1.nodes])
        #
        # extract section assignments:
        for sa in inst1.sectionAssignments:
            sectName = sa.sectionName
            sectRegion = sa.region
            sectMaterial = odb.sections[sectName].material
            sectDensity = odb.materials[sectMaterial].density.table[0][0]

            evolBySection = evol.getSubset(region=sectRegion)
            #
            getNodeAveragedData(evolBySection, elConnec, nodeDict, sectDensity)
    except:
        # something gone wrong during evol or density extaction
        return False, {}
    #
    status = True
    return status, nodeDict


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getNodeAveragedData(fieldOutput, elConnec, nodeDict, density):
    """
    Extract data averaged at nodes.
    Note: data is at whole element!

    @param fieldOutput  odb_FieldOutput
    @param elConnec     DICT of element label, quotient, connectivity.
                        quotient is based on number of nodes by element
    @param nodeDict     DICt of node label and mass at nodes. is successively filled.
    @param density      FLOAT value of current density
    
    @return nodeDict DICT of nodeLabel and nodal mass.
    """
    #
    for value in fieldOutput.values:
        elLabel = value.elementLabel
        evol = value.data
        weight = elConnec[elLabel][0]*evol*density
        for nodeLabel in elConnec[elLabel][1:]:
            nodeDict[nodeLabel] += weight
        

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def calculateMacMatrix(arrayA, arrayB, matrixLen, nodeMassList=None, isComplex=True,
                           genTextOutput=False):
    """
    calculate the mac value for displacements given in arrayA and arrayB
    use complex calculation.

    @param arrayA ARRAY of TUPLE [(displ, complexDispl),] for first step
    @param arrayB ARRAY of TUPLE [(displ, complexDispl),] for second step
    @param matrixLen INT
    @param nodeMassList    LIST of mass by node
    @param isComplex       BOOL if true, complex calcualtion done. If false, real only (faster)
    @param genTextOutput   BOOL if true, write results to text file.

    @return macMatrix LIST of LIST
    """
    # Calculate macs and write to file
    #
    if _verbose:
        startTime = clock()
        print BLK1,'Start MacCalc (complexAbq): time = %6.3f'%(startTime)
    #
    macMatrixS = ''
    macMatrix = [range(matrixLen) for a in range(matrixLen)]
    if genTextOutput:
        outfileText = open('MAC_AB1.txt','w')
    #
    m = 0
    nComp = len(arrayA[0])
    rComp = range(nComp)
    #
    # if no mass matrix, provide a valid list to be used:
    if not(isComplex) and not(nodeMassList):
        useRealOnly = True
    else:
        useRealOnly = False
    #
    # To find MAC:
    # Nnumerator (num): the sum of the dot product of components.
    # Denominator (denA and denB): the sum of the square
    # of the magnitude of components
    #
    if nodeMassList:
        massArray = numpy.array(nodeMassList)
    else:
        massArray = 1.0
    #
    try:
        for a in arrayA:
            for b in arrayB:
                #
                # Calc Mac for real only without mass!
                if useRealOnly:
                    aRe = a[:,0]
                    bRe = b[:,0]
                    #
                    numA = sum(aRe*bRe) 
                    numB = 0.0
                    denA = sum(aRe*aRe)
                    denB = sum(bRe*bRe)
                else:
                    # Calc MAC with real and complex and potentially with mass.
                    aRe = a[:,0]
                    aIm = a[:,1]
                    bRe = b[:,0]
                    bIm = b[:,1]
                    re1re2 = numpy.sum(aRe*bRe, axis=1)
                    im1im2 = numpy.sum(aIm*bIm, axis=1)
                    re1im2 = numpy.sum(aRe*bIm, axis=1)
                    im1re2 = numpy.sum(aIm*bRe, axis=1)
                    #
                    numA = numpy.sum( (re1re2+im1im2)*massArray ) 
                    numB = numpy.sum( (re1im2-im1re2)*massArray )
                    #
                    re1re1 = numpy.sum(aRe*aRe, axis=1)
                    im1im1 = numpy.sum(aIm*aIm, axis=1)
                    denA = numpy.sum( (re1re1+im1im1)*massArray )
                    #
                    re1re1 = numpy.sum(bRe*bRe, axis=1)
                    im1im1 = numpy.sum(bIm*bIm, axis=1)
                    denB = numpy.sum( (re1re1+im1im1)*massArray )
                #
                # If any of the row or column is zero, we will
                # filter them out
                # 
                if denA == 0 or denB == 0:
                    MAC = 0.0
                    if denA == 0 and denB == 0: # Take only both are 0
                        matrixLen-=1
                        print "Changed Matrix len: ",matrixLen
                else:
                    MAC = (numA*numA+numB*numB)/(denA*denB)
                    macMatrixS += '%8.5f, ' % MAC
                    macMatrix[m/matrixLen][m%matrixLen] = MAC
                    m += 1
            if genTextOutput:
                outfileText.write(macMatrixS +'\n')
            macMatrixS = ''
        if genTextOutput:
            outfileText.close()

    except ZeroDivisionError:
        msg = 'Division by zero has occurred.'\
              'A possible reason is mismatching node numbers.'\
              'Please ensure that node numbers match between output databases.'
        raise ValueError, msg
    except:
        msg= 'A general error has occurred.\nPlease contact customer service.'
        raise ValueError, msg

    if _verbose:
        endTime = clock()
        print BLK1,'End MacCalc: time = %6.3f, (%6.3f)'%(endTime,endTime-startTime)
    #
    return macMatrix
# end: calculateMacMatrix



