"""
Create contour and bar plot results for MAC calculations
"""
#
#
# abaqus imports
from abaqus import *
from abaqusConstants import *
from odbAccess import Odb

import math
import os,sys
import uti


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
class ResultsOdb:

    """
    This class will create an ODB with shell elements.
    The elements will be used to view the MAC matrix in a contour plot
    """
    def __init__(self, matrixLen, swap, stepNames):
        """
        Creates basic ODB.
        Data will be filled later on.
        """
        self.matrixLen = matrixLen
        self.swap = swap

        self.createOdb()
        self.addNodesAndElements()
        self.addStepAndFrame(stepNames)

        modelA = 'Model A - '
        modelB = 'Model B - '
        if swap:
            modelA,modelB = modelB,modelA
        self.textA = modelA+'Step: '+stepNames[0]
        self.textB = modelB+'Step: '+stepNames[1]

        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def createOdb(self):
        """
        Create an ODB (which also creates the rootAssembly)
        """
        index = 1
        macOdbFileName = 'contourMAC' + str(index)
        while os.path.isfile(macOdbFileName + '.odb'):
            index += 1
            macOdbFileName = 'contourMAC' + str(index)

        self.macOdbFileName = macOdbFileName+'.odb'
        self.odb = Odb(name=macOdbFileName,
                  analysisTitle='ODB created with NVH MAC plug-in',
                  description='shell elements for contour plot ',
                  path=self.macOdbFileName)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def addNodesAndElements(self):
        """
        Set up the section categories.
        """
        sCat = self.odb.SectionCategory(name='M', description='MAC')

        #  Create shell model,
        #
        matrixLenE = self.matrixLen-1
        nodeData = [(n+1,n%self.matrixLen,n/self.matrixLen)
                       for n in range(self.matrixLen*self.matrixLen)]
        part1 = self.odb.Part(name='part-1',
                              embeddedSpace=TWO_D_PLANAR,
                              type=DEFORMABLE_BODY)
        part1.addNodes(nodeData=nodeData, nodeSetName='nset-1')

        elementData = [(n+1, (n/matrixLenE)+n+1,
                             (n/matrixLenE)+n+2,
                             (n/matrixLenE)+n+3+matrixLenE,
                             (n/matrixLenE)+n+2+matrixLenE)
                             for n in range(matrixLenE*matrixLenE)]

        part1.addElements(elementData=elementData, type='S4')

        # Instance the part.
        #
        self.instance = self.odb.rootAssembly.Instance(name='part-1-1', object=part1)

        # Create instance level sets for section assignment
        #
        elLabels = [n+1 for n in range(matrixLenE*matrixLenE)]
        elset_1 = self.instance.ElementSetFromElementLabels(name='MAC',
                                                             elementLabels=elLabels)
        #instance1.assignSection(region=elset_1,section=section_1)
        self.nodeLabelData = [n+1 for n in range(self.matrixLen*self.matrixLen)]

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def addStepAndFrame(self, stepNames):
        """
        Create a step and a frame.
        """
        step1 = self.odb.Step(name='Mac_1',
                              description='MAC step, based on %s <-> %s'%tuple(stepNames),
                              domain=TIME,
                              timePeriod=1.0)
        frame1 = step1.Frame(incrementNumber=1,
                             frameValue=1,
                             description='MAC')
        self.frame = frame1
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def addFieldOutput(self, macMatrix, fieldName, fieldDescription):
        """
        Field data:
        Write nodal displacements.
        """
        uField = self.frame.FieldOutput(name=fieldName,
                                        description=fieldDescription,
                                        type=SCALAR)
        dispDataN=[]
        for i in range(self.matrixLen):
            for j in range(self.matrixLen):
                dispDataN.append((macMatrix[self.matrixLen-i-1][j],))
        #
        uField.addData(position=NODAL,
                       instance=self.instance,
                       labels=self.nodeLabelData, data=dispDataN)

    
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def closeOdb(self):
        """
        Save and close odb
        """
        self.odb.save()
        self.odb.close()


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def displayOdb(self):
        """
        Display Odb in viewport
        Create annotations for mode labels.
        """

        o1 = session.openOdb(name=self.macOdbFileName)
        vp = session.viewports[session.currentViewportName]
        vp.setValues(displayedObject=o1)
        vp.odbDisplay.display.setValues(plotState=(CONTOURS_ON_DEF, ))
        vp.view.fitView()
        vp.view.setValues(session.views['Front'])
        vp.viewportAnnotationOptions.setValues(state=OFF, title=OFF, annotations=ON)

        # Create annotations for mode labels
        #
        c=0
        temp=self.matrixLen/10
        while temp:
            c+=1
            temp/=10

        # Swap names if odbs were swapped
        #
        for k in sorted(o1.userData.annotations.keys()):
            del o1.userData.annotations[k]

        for l in range(self.matrixLen):
            ls = str(l+1)
            t = o1.userData.Text(name=self.textA + ' -'+ls,
                                 text=ls,
                                 offset=(-7-c*2, -5),
                                 anchor=(0, self.matrixLen-l-1, 0))
            vp.plotAnnotation(annotation=t)
            t = o1.userData.Text(name=self.textB + ' -'+ls,
                                 text=ls,
                                 offset=(-2-c, -6),
                                 anchor=(l, 0, 0))
            vp.plotAnnotation(annotation=t)

        t = o1.userData.Text(name=self.textA,
                             text=self.textA,
                             offset=(-10-c*2, -10),
                             anchor=(0, self.matrixLen/4, 0),
                             rotationAngle=90)
        vp.plotAnnotation(annotation=t)
        t = o1.userData.Text(name=self.textB,
                             text=self.textB,
                             offset=( -12, -16),
                             anchor=(self.matrixLen/4, 0, 0))
        vp.plotAnnotation(annotation=t)


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def _makeMacBarGraph(mac_diag, matrixLen):

    """
    Method to create the bar graphs for the specified macs.
    The bar graphs are plotted in the current viewport.    
    """ 

    # Get current viewport
    #
    vp = session.viewports[session.currentViewportName]
    bgName = vp.name

    # Create XYPlot
    #
    try:
        xyp = session.XYPlot(name =bgName)
    except:
        xyp = session.xyPlots[bgName]

    # Now create the appropriate curves               
    #
    start = (0.1,0.0)
    width = .8
    inc = .2    
    curvesToPlot = []
    name = 'Mode '

    i=1
    for mag in mac_diag:
        name = 'mode '+str(i)
        i+=1
        xyData = []
        xyData.append(start)
        first = start[0],mag
        xyData.append(first)
        xyData.append(first)
        second = start[0]+width, mag
        xyData.append(second)
        xyData.append(second)
        third = start[0]+width,start[1]
        xyData.append(third)

        # So that the xy data does not interfere with viewports
        #
        xyDataName = '%s-%s' % (bgName, name)
        xyData = session.XYData(name = xyDataName,data = xyData)

        if (uti.getVersion()).startswith('6.6'):
            curve = xyp.Curve(name, xyData)
            curve.curveOptions.setValues(lineThickness=MEDIUM,
                                     legendTextSource=USER_DEFINED,
                                     legendText=name)
        else:
            curve = session.Curve(name, xyData)

        curvesToPlot.append(name)
        start = third[0]+inc,start[1]


    # Differences between 6.6 and 6.7
    if (uti.getVersion()).startswith('6.6'):
        xyp.setValues(curvesToPlot=curvesToPlot)
        xyp.xyPlotOptions.setValues(xAxisMaxAutoCompute=OFF,xAxisMaxValue=matrixLen)
        xyp.xyPlotOptions.setValues(xAxisMinAutoCompute=OFF,xAxisMinValue=0)
        xyp.xyPlotOptions.setValues(yAxisMinAutoCompute=OFF,yAxisMaxAutoCompute=OFF)
        xyp.xyPlotOptions.setValues(xAxisMajorTickMode=TOTAL_NUMBER,
                                    numXAxisMajorTicks=matrixLen-1)
        xyp.xyPlotOptions.setValues(xAxisLabelDecimalPlaces=0,
                                    xAxisLabelFrequency=0,
                                    yAxisMajorTickMode=TOTAL_NUMBER,
                                    yAxisLabelDecimalPlaces=0,
                                    yAxisLabelFrequency=1)
        xyp.xyPlotOptions.setValues(numYAxisMajorTicks=0)
        xyp.xyPlotOptions.setValues(numYAxisMinorTicks=0)
        xyp.xyPlotOptions.setValues(numXAxisMinorTicks=0)
        xyp.setValues(xAxisTitleSource=USER_DEFINED,
                      yAxisTitleSource=USER_DEFINED, xAxisTitle='Modes', 
                      yAxisTitle='MAC diagonal')

    else:
        chartName = xyp.charts.keys()[0]
        chart = xyp.charts[chartName]
        chart.setValues(curvesToPlot=(curvesToPlot), )
        chart.gridArea.style.setValues(color='#FFFFFF')

        chart.axes1[0].axisData.setValues(maxAutoCompute=False,maxValue=matrixLen)
        chart.axes1[0].axisData.setValues(minAutoCompute=False,minValue=0)
        chart.axes1[0].setValues(labelFrequency=0)
        chart.axes1[0].axisData.setValues(tickMode=TOTAL_NUMBER,
                                    tickCount=matrixLen-1)
        chart.axes1[0].axisData.setValues(minorTickCount=0)
        chart.axes1[0].axisData.setValues(useSystemTitle=False, title='Modes')

        chart.axes2[0].axisData.setValues(maxAutoCompute=False, maxValue=1)
        chart.axes2[0].setValues(labelFrequency=1)
        chart.axes2[0].axisData.setValues(tickMode=TOTAL_NUMBER)
        chart.axes2[0].axisData.setValues(tickCount=0)
        chart.axes2[0].axisData.setValues(minorTickCount=0)
        chart.axes2[0].axisData.setValues(useSystemTitle=False, title='MAC Diagonal')
        chart.axes2[0].axisData.setValues(labelNumDigits=0)

    vp.setValues(displayedObject=xyp)
    vp.viewportAnnotationOptions.setValues(state=OFF, title=OFF, annotations=OFF)


#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def plotResults(matrixLen, macMatrix, nameAStep, nameBStep, swap,
                name, description, plotType):
    """
    Create bar graph of diagonal
    Create Field output
    
    """
    if plotType==BANDED:
        # Get diagonal of matrix
        #
        macDiag=[macMatrix[a][a] for a in range(matrixLen)]

        _makeMacBarGraph(macDiag, matrixLen)
    #
    elif plotType == CONTOURS_ON_UNDEF:
        resultsOdb = ResultsOdb(matrixLen,
                                swap,
                                stepNames=(nameAStep, nameBStep))
        resultsOdb.addFieldOutput(macMatrix,
                                  fieldName=name,
                                  fieldDescription=description)
        resultsOdb.closeOdb()
        resultsOdb.displayOdb()

