from abaqus import *
from abaqusConstants import *
import visualization
from octaveBandConstants import *
import octaveData
import uti
try:
    import Numeric
except:
    import numpy.oldnumeric as Numeric

debug = 0

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
class OctaveKernel:
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self):
        """Constructor method"""
        pass

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def calculateOctaveBands(self, numPoints=30, xyName='',
                             octaveType=THIRDOCTAVE, outputType=BOTH):
        """Method to calculate octave band curves from existing curve data"""

        self.curvesToPlot = []
        self.curvesToPlot.append((xyName, None))

        self.inputName = xyName
        
        if xyName == None:
            raise TypeError, 'Please choose an STL curve to work from..'
            
        # get current odb
        vp = session.viewports[session.currentViewportName]

        # get correct band data
        if octaveType == THIRDOCTAVE:
            bandData = octaveData.thirdOctaveData
            self.bandDict = octaveData.thirdOctaveDict
        else:
            bandData = octaveData.oneOctaveData
            self.bandDict = octaveData.oneOctaveDict

        tolerance = 1./numPoints
        # calculate band limits for calculations
        self.newBandData = {}
        for lowerFreq, centerFreq, upperFreq in bandData:
            lowerTol = lowerFreq*tolerance
            upperTol = upperFreq*tolerance
            self.newBandData[centerFreq] = {}
            self.newBandData[centerFreq]['xyPoints'] = []
            self.newBandData[centerFreq]['bandData'] = [lowerFreq-lowerTol,
                                                        lowerFreq+lowerTol,
                                                        upperFreq-upperTol,
                                                        upperFreq+upperTol]

        # loop over xyData and put in correct center band 'buckets'
        # some data may fall into two bands (because of band tolerance)
        centerBands = self.newBandData.keys()
        centerBands.sort()
        for f, stl in session.xyDataObjects[xyName]:
            for centerFreq in centerBands:
                minF, a, b, maxF = self.newBandData[centerFreq]['bandData']
                if f >= minF and f <= maxF:
                    self.newBandData[centerFreq]['xyPoints'].append((f, stl))
                    
        # now remove bands that don't have data for whole range
        for centerFreq in centerBands:
            xyPoints = self.newBandData[centerFreq]['xyPoints']
            if xyPoints == []:
                del self.newBandData[centerFreq]
                continue
            a, minF, maxF, b = self.newBandData[centerFreq]['bandData']
            if xyPoints[0][0] > minF or xyPoints[-1][0] < maxF:
                del self.newBandData[centerFreq]
                continue
            
        centerBands = self.newBandData.keys()
        centerBands.sort()

        # strip out low and high frequencies so that only 1 point is to left or
        # right of band. Also interpolate/extrapolate
        fToRemove = []
        for centerFreq in centerBands:
            xMin, a, xMax = self.bandDict[centerFreq]
            xyPoints = self.newBandData[centerFreq]['xyPoints']
            startVal = []
            middleVals = []
            endVal = []
            for x, y in xyPoints:
                if x < xMin:
                    startVal = [(x,y)]
                elif x == xMin:
                    startVal = []
                    middleVals.append((x,y))
                elif x == xMax:
                    middleVals.append((x,y))
                    break
                elif x > xMax:
                    endVal = [(x,y)]
                    break
                else:
                    middleVals.append((x,y))
            if middleVals == []:
                fToRemove.append(centerFreq) # takes care of very loose tolerance around band ends
                continue
            # tmp
            if debug:
                newXy = startVal + middleVals + endVal
                session.XYData(name='%s-before' % centerFreq, data=newXy, sourceDescription='tmp')
            
            # interpolate/extrapolate
            # lower end
            if middleVals[0][0] == xMin:
                pass
            elif startVal == []:
                # extrapolate
                newPair = self.extrapolate(point1=middleVals[0], point2=middleVals[1], f=xMin,
                                           extrapolateType='before')
                # insert this point
                middleVals.insert(0,newPair[:])
            else:
                # interpolate
                newPair = self.interpolate(startPoint=startVal[0], endPoint=middleVals[0],
                                           f=xMin)
                # insert this point
                middleVals.insert(0,newPair[:])

            # upper end
            if middleVals[-1][0] == xMax:
                pass
            elif endVal == []:
                # extrapolate
                newPair = self.extrapolate(point1=middleVals[-2], point2=middleVals[-1], f=xMax,
                                           extrapolateType='after')
                # append this point
                middleVals.append(newPair[:])
            else:
                # interpolate
                newPair = self.interpolate(startPoint=middleVals[-1], endPoint=endVal[0],
                                           f=xMax)
                # append this point
                middleVals.append(newPair[:])

            # update points
            self.newBandData[centerFreq]['xyPoints'] = middleVals[:]
            
            # tmp
            if debug:
                session.XYData(name='%s-after' % centerFreq, data=middleVals, sourceDescription='tmp')

        for f in fToRemove:
            del self.newBandData[f]
        centerBands = self.newBandData.keys()
        centerBands.sort()

        # now run calculation
        log10 = Numeric.log10
        for centerFreq in centerBands:
            fBand = self.bandDict[centerFreq][2] - self.bandDict[centerFreq][0]
            xyPoints = self.newBandData[centerFreq]['xyPoints']
            total = 0.
            for i in range(len(xyPoints)-1):
                fStart, stlStart = xyPoints[i]
                fEnd, stlEnd = xyPoints[i+1]
                total += (fEnd-fStart) * (10**(stlEnd/10) + 10**(stlStart/10))
            
            total = 10*log10(total/(2.* fBand))
            self.newBandData[centerFreq]['xyOutput'] = total

        if outputType == BOTH:
            self.createBarChart()
            self.createLineCurve()
        elif outputType == LINE:
            self.createLineCurve()
        else:
            self.createBarChart()

        self.plotCurves()
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def interpolate(self, startPoint, endPoint, f):
        """Metyhod to interpolate between two values"""

        startX, startY = startPoint
        endX, endY = endPoint

        slope = (endY-startY)/(endX-startX)

        newY = startY + (f-startX)*slope

        return (f, newY)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def extrapolate(self, point1, point2, f, extrapolateType):
        """Method to extrapolate"""

        startX, startY = point1
        endX, endY = point2
        
        slope = (endY-startY)/(endX-startX)

        if extrapolateType == 'after':
            newY = endY + (f-endX)*slope
        else:
            newY = startY - (startX-f)*slope
            
        return (f, newY)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def createLineCurve(self):
        """Method to create the line curve"""

        centerBands = self.newBandData.keys()
        centerBands.sort()
        xyData = []
        for centerFreq in centerBands:
            xyData.append((centerFreq, self.newBandData[centerFreq]['xyOutput']))
        name = 'Line - %s' % self.inputName
        session.XYData(name=name, data=xyData, sourceDescription=name)
        self.curvesToPlot.append((name, HOLLOW_CIRCLE))
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def createBarChart(self):
        """Method to create the bar chart"""

        centerBands = self.newBandData.keys()
        centerBands.sort()
        xyData = []
        for centerFreq in centerBands:
            xMin, a, xMax = self.bandDict[centerFreq]
            xyData.append((xMin, 0.))
            xyData.append((xMin, self.newBandData[centerFreq]['xyOutput']))
            xyData.append((xMax, self.newBandData[centerFreq]['xyOutput']))
            xyData.append((xMax, 0.))
            
        name = 'Bar - %s' % self.inputName
        session.XYData(name=name, data=xyData, sourceDescription=name)
        self.curvesToPlot.append((name, None))
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def plotCurves(self):
        """Method to plot the curves"""

        try:
            xyp = session.xyPlots['XYPlot-1']
        except:
            xyp = session.XYPlot('XYPlot-1')

        curvesToPlot = []
        
        for curveName, symbolMarker in self.curvesToPlot:
            curvesToPlot.append(curveName)
            if uti.getVersion().split('-')[0].split('.')[1] < 7:
                curve = xyp.Curve(curveName, session.xyDataObjects[curveName])
            else:
            	curve = session.Curve(xyData=session.xyDataObjects[curveName])
            if symbolMarker:
                curve.curveOptions.setValues(showSymbol=ON,
                                             symbolMarker=symbolMarker)
        
        chartName = xyp.charts.keys()[0]
        xyp.charts[chartName].setValues(curvesToPlot=curvesToPlot)

