from abaqus import *
import copy
from abq_SlideLine.slnUtils import parseKeyword
from exceptions import *

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def defineSlideLine(elsetName, masterSurfaceName, slaveSurfaceName,
              interactionPropertyName, numModes=1, elemInc=0, nodeInc=0):
    modelName = session.sessionState[session.currentViewportName]['modelName']
    model = mdb.models[modelName]

    masterInstanceName = getInstanceNameFromSurfaceName(model, masterSurfaceName)
    slaveInstanceName = getInstanceNameFromSurfaceName(model, slaveSurfaceName)
    
    instBlock, assemBlock = getKeywordBlocks(model, slaveInstanceName, masterInstanceName,
                                                   slaveSurfaceName, masterSurfaceName, elsetName,
                                                   interactionPropertyName, numModes,
                                                   elemInc, nodeInc)
    writeToKeywordEditor(model, elsetName, slaveInstanceName, instBlock, assemBlock)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getInstanceNameFromSurfaceName(model, surfaceName):
    a = model.rootAssembly
    surface = a.surfaces[surfaceName]
    return a.surfaces[surfaceName].elements[0].instanceName

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getNodeListFromSurface(model, surfaceName):
    a = model.rootAssembly
    surface = a.surfaces[surfaceName]
    attachedNodeLabels = {}
    for node in surface.nodes:
        attachedNodeLabels[node.label] = []
    lastCornerNodeIndexMap = {3:2, 4:3, 6:2, 8:3, 9:3}
    maxNumExteriorNodesMap = {3:2, 4:2, 6:4, 8:4, 9:4}
    midSideNodeLabels = []
    for element in surface.elements:
        numNodesPerElement = len(element.connectivity)
        if numNodesPerElement not in lastCornerNodeIndexMap.keys():
            msg = "Only elements with 3, 4, 6, 8, or 9 elements \n" \
                  "are supported for use with this feature."
            raise AbaqusException, msg
        lastCornerNodeIndex = lastCornerNodeIndexMap[numNodesPerElement]
        maxNumExteriorNodes = maxNumExteriorNodesMap[numNodesPerElement]
        conn = element.connectivity
        instanceName = element.instanceName
        i = a.instances[instanceName]
        validNodeLabels = []
        for kIndex in range(numNodesPerElement):
            nodeIndex = conn[kIndex]
            nodeLabel = i.nodes[nodeIndex].label
            if attachedNodeLabels.has_key(nodeLabel):
                if kIndex > lastCornerNodeIndex:
                    midSideNodeLabels.append(nodeLabel)
                validNodeLabels.append(nodeLabel)
        if len(validNodeLabels) > 1:          
            # Ignore elements with only one node (no edges) contributing
            #
            for validNodeLabel in validNodeLabels:
                for label in validNodeLabels:
                    if label != validNodeLabel:
                        attachedNodeLabels[validNodeLabel].append(label)
    if midSideNodeLabels:
        isSecondOrder = TRUE
    else:
        isSecondOrder = FALSE

    endNodes = []
    for nodeLabel, attachedLabels in attachedNodeLabels.items():
        if nodeLabel in midSideNodeLabels: continue
        if isSecondOrder:            
            if len(attachedLabels) == 2:
                endNodes.append(nodeLabel)
        else:
            if len(attachedLabels) == 1:
                endNodes.append(nodeLabel)
    if len(endNodes) != 2:        
        msg = "The face along surface %s was not continuous. \n" \
              "The slideline/ISL elements could not be created." % surfaceName
        raise AbaqusException, msg

    lastCornerNode = endNodes[0]
    lastMidNode = None
    nodeList = []
    nodeList.append(lastCornerNode)
    for kNode in range(0,len(attachedNodeLabels.keys())-2):
        if lastCornerNode:
            attachedNodeList = attachedNodeLabels[lastCornerNode]
        else:
            attachedNodeList = attachedNodeLabels[lastMidNode]

        # When encountering an element that has more than 2 nodes
        # attached, make sure the last node worked on is the other end
        # of this element. So we sort the nodes in terms of # of
        # attached nodes
        #
        for node in [b[1] for b in sorted([(len(attachedNodeLabels[n]), n) for n in attachedNodeList])]:
            if node in nodeList: continue
            if lastCornerNode:
                if midSideNodeLabels:
                    if node in midSideNodeLabels:
                        nodeList.append(node)
                        lastMidNode = node
                        lastCornerNode = None
                        break
                else:
                    nodeList.append(node)
                    lastCornerNode = node
                    break
            else:
                if node not in midSideNodeLabels:
                    nodeList.append(node)
                    lastMidNode = None
                    lastCornerNode = node
                    break
    nodeList.append(endNodes[1])
    if not isNodeOrderCorrect(model, surfaceName, nodeList, isSecondOrder):
        nodeList.reverse()
    return nodeList, isSecondOrder

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def isNodeOrderCorrect(model, surfaceName, nodeList, isSecondOrder):

    # If nodes go along connectivity in CCW manner, the order is incorrect
    #
    a = model.rootAssembly
    surf = a.surfaces[surfaceName]
    iName = surf.elements[0].instanceName
    i = a.instances[iName] 
    
    # Grab node and node index
    #
    node1 = i.nodes.getFromLabel(nodeList[0])
    nodeIndex1 = i.nodes.index(node1)
    if isSecondOrder:
        node2 = i.nodes.getFromLabel(nodeList[2])
    else:
        node2 = i.nodes.getFromLabel(nodeList[1])
    nodeIndex2 = i.nodes.index(node2)
    
    # Loop through elements to find element that contains both nodes given
    # in list above
    #
    for elem in surf.elements:
        if nodeIndex1 in elem.connectivity and nodeIndex2 in elem.connectivity: break
    connIndex1 = list(elem.connectivity).index(nodeIndex1)
    connIndex2 = list(elem.connectivity).index(nodeIndex2)
    pair = (connIndex1,connIndex2)

    # Check if the order of connectivity of node indexes is acceptable
    #
    # Triangle
    #
    if len(elem.connectivity) in (3,6):
        if pair in ( (0,1), (1,2), (2,0) ): return False

    # Quads
    #
    elif len(elem.connectivity) in (4,8):
        if pair in ( (0,1), (1,2), (2,3), (3,0) ): return False

    return True

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def writeToKeywordEditor(model, elsetName, slaveInstanceName, instBlock, assemBlock):
    modelName = session.sessionState[session.currentViewportName]['modelName']

    # Comment out deletion of previous keyword block edit
    #
    #model.keywordBlock.setValues(edited = 0)
    model.keywordBlock.synchVersions(storeNodesAndElements=False)
    instFileName = "ISL-%s.inc" % elsetName
    assemFileName = "SlideLine-%s.inc" % elsetName
    instFile = open(instFileName, 'w')
    assemFile = open(assemFileName, 'w')
    instFile.write(instBlock)
    instFile.close()
    assemFile.write(assemBlock)
    assemFile.close()
    instString = "*Include, input=%s\n" % instFileName
    assemString = "*Include, input=%s\n" % assemFileName

    foundPartInclude = FALSE
    foundInstanceInclude = FALSE
    foundAssemInclude = FALSE

    isDependent = model.rootAssembly.instances[slaveInstanceName].dependent
    slavePartName = model.rootAssembly.instances[slaveInstanceName].part.name

    scopedElsetName = "%s.%s" % (slaveInstanceName,elsetName)
    firstElsetName = "%s_0" % elsetName
    blocks =  model.keywordBlock.sieBlocks
    numBlocks = len(blocks)
    inInstance = FALSE
    inAssembly = FALSE
    inPart = FALSE
    for kBlock in range(0,numBlocks):
        block = blocks[kBlock]
        lines = block.split("\n")
        keyword, parameters = parseKeyword(lines)
        if keyword == "*assembly":
            inAssembly = TRUE       
        elif keyword == "*part":
            if isDependent:
                if parameters["name"] == slavePartName.lower():
                    inPart = TRUE
                    #model.keywordBlock.insert(kBlock, """%s""" % instString)
        elif keyword == "*endpart":
            if isDependent and inPart:
                #print "inserting blocK"
                model.keywordBlock.insert(kBlock-1, """%s""" % instString)
                inPart = FALSE
        elif keyword == "*instance":
            if not isDependent:
                if parameters["name"] == slaveInstanceName.lower():
                    inInstance = TRUE
            inAssembly = FALSE
        elif keyword == "*endinstance":
            if not isDependent:
                if inInstance and not foundInstanceInclude:
                    model.keywordBlock.insert(kBlock-1, """%s""" % instString)
            inInstance = FALSE
            inAssembly = TRUE
        elif keyword == "*include":
            if inPart:
                if parameters["input"] == instFileName.lower():
                    foundPartInclude = TRUE
                    model.keywordBlock.replace(kBlock, """%s""" % instString)
            elif inInstance:
                if parameters["input"] == instFileName.lower():
                    foundInstanceInclude = TRUE
                    model.keywordBlock.replace(kBlock, """%s""" % instString)
            elif inAssembly:
                if parameters["input"] == assemFileName.lower():
                    foundAssemInclude = TRUE
                    model.keywordBlock.replace(kBlock, """%s""" % assemString)
        elif keyword == "*endassembly":
            if not foundAssemInclude:
                model.keywordBlock.insert(kBlock, """%s""" % assemString)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getMaxElemLabel(model, surfaceName):
    a = model.rootAssembly
    surf = model.rootAssembly.surfaces[surfaceName]
    iName = surf.elements[0].instanceName
    instance = a.instances[iName]
    if instance.dependent:
        p = instance.part
        maxNum = (len(p.elements)/1000)*1000 + 1000
    else:
        maxNum = (len(instance.elements)/1000)*1000 + 1000
    return maxNum

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getKeywordBlocks(model, slaveInstanceName, masterInstanceName, slaveSurfaceName,
                     masterSurfaceName, elsetName,
                     interactionPropertyName, numModes, elemInc, nodeInc):

    maxElemNum = getMaxElemLabel(model, slaveSurfaceName)
    masterNodes, isSlaveSecondOrder = getNodeListFromSurface(model, masterSurfaceName)
    slaveNodes, isMasterSecondOrder = getNodeListFromSurface(model, slaveSurfaceName)
    if isSlaveSecondOrder != isMasterSecondOrder:
        msg = "Elements making up the slave and master surfaces must be of the same order."
        raise AbaqusException, msg
    modeAngles = {}
    modeAngles[1] = [0,180]
    modeAngles[2] = [0,90,180]
    modeAngles[3] = [0,60,120,180]
    modeAngles[4] = [0,45,90,135,180]    


    # Initialize String
    #
    assemString = ""
    instString = ""
    propString = getInteractionString(model, interactionPropertyName)
    
    for kMode in range(len(modeAngles[numModes])):
        setName = "%s_%s" % (elsetName,modeAngles[numModes][kMode])
        elemOffset = maxElemNum + elemInc*kMode
        nodeOffset = nodeInc*kMode
        
        # Write ISL elements
        #
        if isSlaveSecondOrder:
            line = "*Element, Type=ISL22A, Elset=%s\n" % setName
            skip = 2
            numElems = (len(slaveNodes)-1)/2
        else:
            line = "*Element, Type=ISL21A, Elset=%s\n" % setName
            skip = 1
            numElems = len(slaveNodes)-1
        instString = instString + line
        elemNum = elemOffset + 1
        if bool(len(slaveNodes) & 1):
            s=2
        else:
            s=1
        for kNode in range(0,len(slaveNodes)-s,skip):
            label1 = slaveNodes[kNode]
            label2 = slaveNodes[kNode+1]
            label1 = label1 + nodeOffset
            label2 = label2 + nodeOffset
            if isSlaveSecondOrder:
                label3 = slaveNodes[kNode+2]
                label3 = label3 + nodeOffset
                line = "%s, %s, %s, %s\n" % (elemNum, label1, label2, label3)
            else:
                line = "%s, %s, %s\n" % (elemNum, label1, label2)
            elemNum = elemNum + 1
            instString = instString + line

        # Write Slideline
        #
        assemElsetName = "%s.%s" % (slaveInstanceName, setName)
        if isSlaveSecondOrder:
            line = "*Slideline, Elset=%s, Type=Parabolic\n" % assemElsetName
        else:
            line = "*Slideline, Elset=%s, Type=Linear\n" % assemElsetName
        assemString = assemString + line
        curNode = 0
        for knode in range(len(masterNodes)):
            label = masterNodes[knode]
            offsetLabel = label + nodeOffset
            nodeNum = "%s.%s" % (masterInstanceName,offsetLabel)
            if curNode == 0:
                line = "%s" % nodeNum
            else:
                line = line + ", %s" % nodeNum
            curNode = curNode+1
            if curNode == 4:  # Only write four nodes per line (should check width)
                line = line + "\n"
                assemString = assemString + line
                curNode = 0
        if curNode != 0:
            line = line + "\n"
            assemString = assemString + line

        # Write *Interface
        #
        interfaceString = getInterfaceString(slaveInstanceName, setName, propString, numModes, modeAngles[numModes][kMode])
        assemString = assemString + interfaceString

    return instString, assemString

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getInteractionString(model, interactionPropertyName):
   intBlock = ""
   blocks =  model.keywordBlock.sieBlocks
   numBlocks = len(blocks)
   isInBlock = 0
   for kBlock in range(0,numBlocks):
       block = blocks[kBlock]
       lines = block.split("\n")
       keyword, parameters = parseKeyword(lines)
       if keyword == "*surfaceinteraction" and parameters["name"] == interactionPropertyName.lower():
           isInBlock = 1
       elif keyword in ('*bond', '*cohesivebehavior', '*contactcontrols', '*surfacebehavior', '*contactdamping',
                        '*friction', '*gapconductance', '*gapheatgeneration', '*gapradiation',
                        '*gap', '*gapelectricalconductance'):
           if isInBlock:
               intBlock = intBlock + block + "\n"
       else:
           isInBlock = 0
   return intBlock

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getInterfaceString(slaveInstanceName, elsetName, propString, numModes, angle):
   intBlock = "*Interface, Elset=%s.%s\n" \
              "*Asymmetric-axisymmetric, Angle=%s, Mode=%s\n" % (slaveInstanceName,elsetName,angle,numModes)
   return intBlock + propString
