from abaqusGui import *
from abaqusConstants import *
from abq_SlideLine.slnSlideLineDB import SlnSlideLineDB 
import os.path

###########################################################################
# Class definition
###########################################################################

class SlnSlideLineForm(AFXForm):

    [
        ID_OVERWRITE,
        ID_SHOWWARNING,
    ] = range(AFXForm.ID_LAST, AFXForm.ID_LAST+2)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
        
        AFXForm.__init__(self, owner)

        FXMAPFUNC(self, SEL_COMMAND, self.ID_OVERWRITE, SlnSlideLineForm.onCmdOverwrite)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_SHOWWARNING, SlnSlideLineForm.onShowWarning)

        self.cmd = AFXGuiCommand(self, 'defineSlideLine', 'abq_SlideLine.slnModule')
        self.elsetNameKw = AFXStringKeyword(self.cmd, 'elsetName', TRUE)
        self.masterSurfaceNameKw = AFXStringKeyword(self.cmd, 'masterSurfaceName', TRUE)
        self.slaveSurfaceNameKw = AFXStringKeyword(self.cmd, 'slaveSurfaceName', TRUE)
        self.interactionPropertyNameKw = AFXStringKeyword(self.cmd, 'interactionPropertyName', TRUE)
        self.numModesKw = AFXIntKeyword(self.cmd, 'numModes', TRUE, 1)
        self.elemIncKw = AFXIntKeyword(self.cmd, 'elemInc', TRUE, 0)
        self.nodeIncKw = AFXIntKeyword(self.cmd, 'nodeInc', TRUE, 0)
        self.showWarning = TRUE
        from slnSlideLine_plugin import verAbaqus
        self.verAbaqus = str(verAbaqus)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def activate(self):
        modelName = session.sessionState[session.currentViewportName]['modelName']
        model = mdb.models[modelName]
        msg = "The model must contain at least 1 surface interaction definition \n" \
              "to use this tool."
        if not hasattr(model, 'interactionProperties'):
            showAFXErrorDialog(getAFXApp().getAFXMainWindow(), msg)
            return
        if len(model.interactionProperties.keys()) == 0:
            showAFXErrorDialog(getAFXApp().getAFXMainWindow(), msg)
            return
        if len(model.rootAssembly.surfaces.keys()) < 2:
            msg = "The model must contain at least two assembly-level surfaces \n" \
                  "(master and slave) to use this tool."
            showAFXErrorDialog(getAFXApp().getAFXMainWindow(), msg)
            return

        if getDisplayedObjectType() != ASSEMBLY: # abaqusConstants now includes ASSEMBLY (symbolicConstant)

            showAFXErrorDialog(getAFXApp().getAFXMainWindow(), 
                'The assembly must be shown in the current\n' + \
                'viewport before you may use this function.')
            return
        if len(model.parts.keys()) == 0:
            showAFXErrorDialog(getAFXApp().getAFXMainWindow(), 
                'No part instances are available in the model.')
            return
        flag = FALSE
        for part in model.parts.keys():
            if model.parts[part].space != TWO_D_PLANAR and model.parts[part].space != AXISYMMETRIC:
                flag = TRUE
                break
        if flag:
            showAFXInformationDialog(getAFXApp().getAFXMainWindow(), 
                'Only axisymmetric or 2D models are supported.  Current model has parts with other dimensionality.')
        AFXForm.activate(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        modelName = session.sessionState[session.currentViewportName]['modelName']
        model = mdb.models[modelName]
        fileName = self.elsetNameKw.getValue().strip()
        master = self.masterSurfaceNameKw.getValue()
        slave = self.slaveSurfaceNameKw.getValue()
        db = self.getCurrentDialog()
        if master == slave:
            msg = "Master and slave surface can not be the same."
            showAFXErrorDialog(db, msg)
            return FALSE
        if len(model.rootAssembly.surfaces[slave].elements) == 0:
            msg = "No elements were found in slave surface '%s'." % slave
            showAFXErrorDialog(db, msg)
            return FALSE
        if len(model.rootAssembly.surfaces[master].elements) == 0:
            msg = "No elements were found in master surface '%s'." % master
            showAFXErrorDialog(db, msg)
            return FALSE
        if self.elemIncKw.getValue() < 0:
            showAFXErrorDialog(db, 'A non-negative value for element number offset must be specified.')
            db.onKeywordError(self.elemIncKw)
            return FALSE
        if self.nodeIncKw.getValue() < 0:
            showAFXErrorDialog(db, 'A non-negative value for node number offset must be specified.')
            db.onKeywordError(self.nodeIncKw)
            return FALSE
        if not fileName:
            showAFXErrorDialog(db, 'An elset file name must be specified.')
            db.elsetText.setFocusToTextField()
            return FALSE

        self.doCustomChecks()

        if os.path.exists("ISL-" + fileName + ".inc") or os.path.exists("SlideLine-" + fileName + ".inc"):
            db = self.getCurrentDialog()
            msg = "File already exists: ISL-%s" % fileName + ".inc or SlideLine-%s" % fileName + ".inc\n\n" + "OK to overwrite?"
            showAFXWarningDialog(db, msg, AFXDialog.YES|AFXDialog.NO, self, self.ID_OVERWRITE)
            return FALSE

        return TRUE

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):

        msg = "Plug-in defines a slide line and ISL21A/ISL22A elements from" \
              " the master and slave surfaces given.\nFiles created and written" \
              " to the keyword editor are:\n" \
              "1)'ISL-elset.inc' - Part or Instance level, *ELEMENT, TYPE=ISL\n" \
              "2)'SlideLine-elset.inc' - Assembly level, *SLIDELINE and *INTERFACE\n\n" \
              "The following items in the input file will need to be edited:\n" \
              "1) Modify *ELEMENT to change to CAXA type and add OFFSET parameter\n" \
              "2) Remove Master and Slave surface definitions"
        if self.verAbaqus >= '6.6':
            if self.showWarning:
                showAFXDismissableWarningDialog(getAFXApp().getAFXMainWindow(), msg, self, self.ID_SHOWWARNING)
        else:
            showAFXInformationDialog(getAFXApp().getAFXMainWindow(), msg)

        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdOverwrite(self, sender, sel, ptr):

        if sender.getPressedButtonId() == AFXDialog.ID_CLICKED_YES:
            AFXForm.issueCommands(self)
        return TRUE

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onShowWarning(self, sender, sel, ptr):

        if self.verAbaqus >= '6.6':
            self.showWarning = sender.getCheckButtonState()
        return TRUE


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        db = SlnSlideLineDB(self)
        modelName = session.sessionState[session.currentViewportName]['modelName']
        model = mdb.models[modelName]
        for propName in model.interactionProperties.keys():
            db.propertyBox.appendItem(propName)        
        db.propertyBox.setCurrentItem(0)
        self.interactionPropertyNameKw.setValue(model.interactionProperties.keys()[0])
        for surfName in model.rootAssembly.surfaces.keys():
            db.masterBox.appendItem(surfName)
            db.slaveBox.appendItem(surfName)
        self.masterSurfaceNameKw.setValue(model.rootAssembly.surfaces.keys()[0])
        self.slaveSurfaceNameKw.setValue(model.rootAssembly.surfaces.keys()[1])
        return db