"""
Kernel module for the CAX-CCL Plugin
"""
from abaqus import *
from abaqusConstants import *
from customKernel import *
import testUtils,osutils
from symbolicConstants import *
from types import *
from job import JobType
import methodCallback

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

class Model(RepositorySupport):
    """This class creates a Cax-Ccl model with the current model name as its key"""
    
    def __init__(self,name):
        RepositorySupport.__init__(self)
        self.name = name

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
class caxCCL(RepositorySupport):
    """This abstract class serves as a name space container for
    Cax Ccl  specific models data"""

    def __init__(self):

       RepositorySupport.__init__(self)
       self.Repository('models', Model)


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def errorMessageFunc(message):
    """
    Method to print error message based on the ABAQUS Version.
    Attributes:
        message - message to be printed
    Calling Methods: updateInputFile
    """
    print message
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

def checkModel(callingMethod,args,keywordArgs,userData):
    
    """
    Method checks if the model has any unhandled features.

    """

    
    if not hasattr(callingMethod.caller, 'model'):
        return
    try:
        modelName = callingMethod.caller.model
        jobName   = callingMethod.caller.name
    except:
        errorMessageFunc(
            'Error: The callback procedure did not return a model name.\n')

    if hasattr(session.customData, 'CaxCcl'):
        if session.customData.CaxCcl.models.has_key(modelName):
            if mdb.jobs[jobName].model in sorted(session.customData.CaxCcl.models.keys()):
                if mdb.jobs[jobName].type != RESTART:
                    raise ValueError, 'The job for Symmetric Model Generation should be of type Restart\n Toggle on Restart option before submitting the job.'


    if not modelName in sorted(mdb.models.keys()):
        errorMessage =  ':Model Name "%s" sent to callback method ' \
                        'is not available in the mdb.\n' %(modelName)
        errorMessageFunc(errorMessage)
        
## Check if New sets were created in the model and give error if so.
    if hasattr(session.customData, 'CaxCcl'):
        if session.customData.CaxCcl.models.has_key(modelName):
            caxmodelname = session.customData.CaxCcl.models[modelName].CaxModelName
            for cclpart in mdb.models[modelName].parts.keys():
                if cclpart in mdb.models[caxmodelname].parts.keys():
                    for cclModelSet in mdb.models[modelName].parts[cclpart].sets.keys():
                        if not cclModelSet in mdb.models[caxmodelname].parts[cclpart].sets.keys():
                                error1 = "The set " + '"' + cclModelSet+ '"' + " is not available in the CAX Model " + '"' + caxmodelname + '".\n'
                                error2 = "The Symmetric Model Generation plug-in only allows reuse of sets available in CAX Model."
                                error = error1 + error2
                                raise ValueError, error
                else:
                    for cclModelSet in mdb.models[modelName].parts[cclpart].sets.keys():
                        for caxpart in mdb.models[caxmodelname].parts.keys():
                            checkIndex=0
                            if cclModelSet in mdb.models[caxmodelname].parts[caxpart].sets.keys():
                                checkIndex = 1
                        if checkIndex == 0:        
                                error1 = "The set " + '"' + cclModelSet + '"' + " is not available in the CAX Model " + '"' + caxmodelname + '".\n'
                                error2 = "The Symmetric Model Generation plug-in only allows reuse of sets available in CAX Model."
                                error = error1 + error2
                                raise ValueError, error
            for cclAssemblySet in mdb.models[modelName].rootAssembly.sets.keys():

                if not cclAssemblySet in mdb.models[caxmodelname].rootAssembly.sets.keys():

                    error1 = "The set " + '"' + cclAssemblySet + '"' + " is not available in the CAX Model " + '"' + caxmodelname + '".\n'
                    error2 = "The Symmetric Model Generation plug-in only allows reuse of sets available in CAX Model."
                    error = error1 + error2
                    raise ValueError, error

    if hasattr(session.customData, 'CaxCcl'):
        if  session.customData.CaxCcl.models.has_key(modelName):
            for part in mdb.models[modelName].parts.keys():
                if mdb.models[modelName].parts[part].queryGeometry(printResults=0)['category'] == 'OrphanMesh':
                    raise ValueError, 'Orphan Meshes are not allowed in the Cylindrical model.'
            if mdb.models[modelName].materials.keys() != []:
                raise ValueError, 'Material definitions are not allowed in the Cylindrical model.'
    
            if mdb.models[modelName].sections.keys() != []:
                raise ValueError, 'Section definitions are not allowed in the Cylindrical model.'
                    
            if mdb.models[modelName].interactionProperties.keys() != []:
                raise ValueError, 'Interaction Properties definitions are not allowed in the Cylindrical model.'
                        
            if mdb.models[modelName].amplitudes.keys() != []:
                raise ValueError, 'Amiplitude definitions are not allowed in the Cylindrical model.'
                            
            if mdb.models[modelName].profiles.keys() != []:
                raise ValueError, 'Profile definitions are not allowed in the Cylindrical model.'
            if mdb.models[modelName].constraints.keys() != []:
                for constraintName in mdb.models[modelName].constraints.keys():
                    if mdb.models[modelName].constraints[constraintName].controlPoint[0].startswith('_PickedSet'):
                        raise ValueError, 'Constraint "%s" defined using pickedsets.\n\nSymmetric Model generation plug-in requires the use of named set for Refpt definition.' %constraintName
                    if mdb.models[modelName].constraints[constraintName].surface[0].startswith('_PickedSet'):
                        raise ValueError, 'Constraint "%s" defined using pickedsets.\n\nSymmetric Model generation plug-in requires the use of named set for Surface definition.' %constraintName
            if mdb.models[modelName].boundaryConditions.keys() != []:
                for bcName in mdb.models[modelName].boundaryConditions.keys():
                    if mdb.models[modelName].boundaryConditions[bcName].region[0].startswith('_PickedSet'):
                        raise ValueError, 'Boundary Condition "%s" defined using pickedsets.\n\nSymmetric Model generation plug-in requires the use of named set for region definition.' %bcName
            if mdb.models[modelName].loads.keys() != []:
                for loadName in mdb.models[modelName].loads.keys():
                    if mdb.models[modelName].loads[loadName].region[0].startswith('_PickedSet'):
                        raise ValueError, 'Load "%s" defined using pickedsets.\n\nSymmetric Model generation plug-in requires the use of named set for region definition.' %loadName


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

def cleanInputFile(Block):
    keyword=Block
    if Block.lower().startswith('*node'):
        if Block.lower().find('output') == -1:
            keyword=''

    if Block.lower().startswith('*element'):
        if Block.lower().find('output') == -1:
            keyword=''

    if Block.lower().startswith('*part'):
        keyword=''

    if Block.lower().startswith('*assembly'):
        keyword=''
        
    if Block.lower().startswith('*restart'):
        keyword=''
        
    if Block.lower().startswith('*nset'):
        keyword=''

    if Block.lower().startswith('*elset'):
        keyword=''

    if Block.lower().startswith('*surface'):
        keyword=''

    if not Block.lower().find('part') == -1:
        if Block.lower().startswith('**'):
            keyword=''

    if not Block.lower().find('assembly') == -1:
        if Block.lower().startswith('**'):
            keyword=''

    if Block.lower() == '*end part':
        keyword=''

    if Block.lower().startswith('*instance'):
        keyword=''

    if Block.lower() == '*end instance':
        keyword=''

    if Block.lower() == '*end assembly':
        keyword=''

    return keyword
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

def writeSymmModelGenParameters(cclModelName,caxModelName, resultTransferStepName, oldJobName, meshTypeDefinition = 'Uniform', numEleminHoopDir = 2 ,elemBiasDefinition = ()):
    

    """
    Method stores the dialog data in session custom data and
    converts the model into Restart model.
    Arguements
        cclModelName - CCL model Name
        meshTypeDefinition - Radio Button (Uniform or Biased)
        numEleminHoopDir - Element info for meshTypeDefinition=Uniform
        elemBiasDefinition  - Element Bias info for meshTypeDefinition=Biased
        caxModelName - CAX model Name
        resultTransferStepName  - Step to copy results from for *Symmetric Results Transfer
        oldJob - Jobname for *Symmetry Model Generation

    """
    if meshTypeDefinition == UNIFORM:
        if not numEleminHoopDir > 0:
            raise ValueError, 'For Uniform mesh definition, No. of Elements should be a positive integer.'

    if meshTypeDefinition == BIASED:
        if elemBiasDefinition == ():
            raise ValueError, 'Bias definition not defined.'
        else:
            i=1
            for row in elemBiasDefinition:
                j=1
                for item in row:
                    if j != 1:
                        try:
                            value = float(item)
                            if value <= 0.0:
                                if j == 2:
                                    raise ValueError,'Bias mesh definition is incomplete or incorrect.\n "%s" in "Elements" column should be corrected.' % value
                                if j == 3:
                                    raise ValueError,'Bias mesh definition is incomplete or incorrect.\n "%s" in "Bias" column should be corrected.' % value
                        except:
                            if j == 2:
                                raise TypeError,'Bias mesh definition is of incorrect type.\n "%s" in "Elements" column should be corrected.' % value
                            if j == 3:
                                raise TypeError,'Bias mesh definition is of incorrect type.\n "%s" in "Bias" column should be corrected.' % value

                    else:
                        try:
                            value = int(item)
                            if value <= 0:
                                if j == 2:
                                    raise ValueError,'Bias mesh definition is incomplete or incorrect. \n "%s" in "Elements" column should be corrected' % value
                                if j == 3:
                                    raise ValueError,'Bias mesh definition is incomplete or incorrect. \n "%s" in "Bias" column should be corrected' % value
                        except:
                            if j == 2:
                                raise TypeError, 'Bias mesh definition is of incorrect type.\n "%s" in "Elements" column should be corrected.' % item
                            if j == 3:
                                raise TypeError, 'Bias mesh definition is of incorrect type.\n "%s" in "Bias" column should be corrected.' % item
                    j+=1
                i+=1
            totBiasAngle = 0
            for row in elemBiasDefinition:

                for item in row:
                    if float(item) > 180.0:
                        raise ValueError,'Bias Angle cannot be greater than 180 degrees'

                    totBiasAngle += item
                    break
            if not (totBiasAngle == 180.0):
                raise ValueError,'Bias Angle should add up to 180 degrees'



    if not hasattr(session.customData, 'CaxCcl'):
        session.customData.CaxCcl = caxCCL()

    if not session.customData.CaxCcl.models.has_key(cclModelName):

        session.customData.CaxCcl.Model(cclModelName)
    restartStepnames = mdb.models[cclModelName].steps.keys()
    if restartStepnames[-1] == "Initial":
        raise ValueError,'No Steps have been defined in the model "%s".\nDefine Step before redefining the Symmetric Model Generation parameters.' % cclModelName
    
    session.customData.CaxCcl.models[cclModelName].steptoWriteResFrom=resultTransferStepName
    session.customData.CaxCcl.models[cclModelName].RestartJobfrom=oldJobName
    session.customData.CaxCcl.models[cclModelName].Bias=meshTypeDefinition
    session.customData.CaxCcl.models[cclModelName].BiasInfoforUniform=numEleminHoopDir
    session.customData.CaxCcl.models[cclModelName].BiasInfoforBiased=elemBiasDefinition
    session.customData.CaxCcl.models[cclModelName].CaxModelName=caxModelName
    


    mdb.models[cclModelName].setValues(restartJob=oldJobName, restartStep=restartStepnames[-1])
    if mdb.models[caxModelName].noPartsInputFile:
        mdb.models[cclModelName].setValues(noPartsInputFile=ON)
    print '                                              '
    print 'Symmetric Model parameters were written for "%s" model.' % cclModelName

    
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~            
def writeInputFile(callingMethod,args,keywordArgs,userData):

    """
    Method writes the input file from keyword block

    """
    if not hasattr(callingMethod.caller, 'model'):
        return
    try:
        modelName = callingMethod.caller.model
        jobName   = callingMethod.caller.name
    except:
        errorMessageFunc(
            'Error: The callback procedure did notr return a model name.\n')


    if not modelName in sorted(mdb.models.keys()):
        errorMessage =  ':Model Name "%s" sent to callback method ' \
                        'is not available in the mdb.\n' %(modelName)
        errorMessageFunc(errorMessage)

##   Checking in the job description to see if the job was run using the plugin in the previous session
##                 Checking for a string in the job description to determine if the job input file was re-written using plugin.
##


    jobDescriptionAppendTxt = "--Cylindrical Model using Symmetric Model Generation Plugin"
    if hasattr(session.customData, 'CaxCcl'):
        if session.customData.CaxCcl.models.has_key(modelName):
            pass
        else:
            jobDescription = callingMethod.caller.description
            if jobDescription.find(jobDescriptionAppendTxt) != -1:
                errorMessage1= "Symmetric Model generation parameters need to be redefined for %s.\n" %modelName
                errorMessage2= "Re-run %s after defining the parameters" % jobName
                errorMessage = errorMessage1 + errorMessage2
                errorMessageFunc(errorMessage)
                raise ValueError, errorMessage
    else:
        jobDescription = callingMethod.caller.description
        if jobDescription.find(jobDescriptionAppendTxt) != -1:
            errorMessage1= "Symmetric Model generation parameters need to be redefined for %s.\n" % modelName
            errorMessage2= "Re-run %s after defining the parameters" % jobName
            errorMessage = errorMessage1 + errorMessage2
            errorMessageFunc(errorMessage)
            raise ValueError, errorMessage



    if hasattr(session.customData, 'CaxCcl'):
        if  session.customData.CaxCcl.models.has_key(modelName):

##   Appending string to the job description to tag the job.
            jobDescription = callingMethod.caller.description
            if jobDescription.find(jobDescriptionAppendTxt) == -1:
                jobDescription = callingMethod.caller.description + jobDescriptionAppendTxt
                mdb.jobs[callingMethod.caller.name].setValues(description=jobDescription)

                
            model = mdb.models[modelName]
            caxModel=mdb.models[session.customData.CaxCcl.models[modelName].CaxModelName]
            i=0
            AvailablestepName=caxModel.steps.keys()
            stepNum=len(AvailablestepName)
            if session.customData.CaxCcl.models[modelName].steptoWriteResFrom in caxModel.steps.keys():
                while not i == len(AvailablestepName):
                    if session.customData.CaxCcl.models[modelName].steptoWriteResFrom == AvailablestepName[i]:
                        stepNum=i
                    i+=1
            
            SymmModtext = '*Symmetric Model Generation, Revolve, Node Offset=100000, Element Offset=100000 \n' +\
                          '0.0,0.0,0.0,0.0,1.0,0.0\n' +\
                          '1.0,0.0,0.0\n'
            
            septext = '**-----------------------------------------------------------------------------\n'
            preprinttext = '*Preprint, echo=NO, model=NO, history=NO, contact=NO\n' +\
                           '**---------------------------------------------------------------------\n'
            headingtext = '*Heading\n' + model.description + '\n'+\
                          '**---------------------------------------------------------------------\n'
            SymmResText = '*Symmetric Results Transfer, Step=' + str(stepNum) + '\n' +\
                          '**---------------------------------------------------------------------\n'

            if session.customData.CaxCcl.models[modelName].Bias == BIASED:

                numEleminBias=0
                for line in session.customData.CaxCcl.models[modelName].BiasInfoforBiased:
                    SymmModtext += str(line[0]) +','+ str(line[1]) + ',' + str(line[2]) + ',CYLINDRICAL\n'
                    numEleminBias +=line[1]
                symmBCNsetEnd = (numEleminBias * 200000) + 1
                Nsettxt = '*Nset, nset=SymmPlane,gen\n' +\
                '1,99999, 1\n' +\
                str(symmBCNsetEnd) +','+ str(symmBCNsetEnd +99998) + ',1\n'
                
            else:
                SymmModtext += '180,' + str(session.customData.CaxCcl.models[modelName].BiasInfoforUniform) + ',1.0,CYLINDRICAL\n'
                symmBCNsetEnd = (session.customData.CaxCcl.models[modelName].BiasInfoforUniform * 200000) + 1
                Nsettxt = '*Nset, nset=SymmPlane,gen\n' +\
                '1,99999, 1\n' +\
                str(symmBCNsetEnd) +','+ str(symmBCNsetEnd +99998) + ',1\n'

            keywordBlock = model.keywordBlock
            keywordBlock.synchVersions(storeNodesAndElements=FALSE)
            sieB = keywordBlock.sieBlocks    
            BCregion=[]
            previousline=''
            inputfilename = str(callingMethod.caller.name) + ".inp"
            FILE=open(inputfilename,"w")
            FILE.write(headingtext)
            FILE.write(preprinttext)
            FILE.write(SymmModtext)
            FILE.write(septext)
            FILE.write(SymmResText)
            FILE.write(septext)
            FILE.write(Nsettxt)
            FILE.write(septext)
            writebc=0
            for keyword in sieB:
                inputline=cleanInputFile(keyword)
                if not inputline=='':
                    FILE.write(inputline +'\n')

                    if inputline.lower().startswith('*step'):

                        writebc=1
                    if inputline.lower().startswith('*boundary'):
                        BCkeyword=inputline.split('\n')
                        # below try except are used for inactive BCs for a particular step
                        # where there would not be any region mentioned in the keyword block, 
                        # thus it is not to be written to the input file.
                        #
                        try :
                            region=BCkeyword[1].split(',')
                        except:
                            pass

                        try:
                            if not region[0] in BCregion:
    
                                BCregion.append(region[0])
                                
                                bcstr = str(region[0]) + ', 3, 5\n'
                                
                                FILE.write(bcstr)
                                previousline=inputline
                        except:

                            pass
                    if inputline.lower().startswith('*boundary'):
                
                        previousline=inputline
                        if writebc == 1:

                            FILE.write('SymmPlane,ZSYMM\n')
                            writebc=0
            FILE.close()
            print '                                              '
            print 'The input file "%s" was re-written with Symmetric Model generation parameters' % callingMethod.caller.name


methodCallback.addCallback(JobType, 'writeInput', checkModel, callAfter=FALSE)
methodCallback.addCallback(JobType, 'writeInput', writeInputFile, callAfter=TRUE)

