"""This module is called by the GUI plugin of VCCT to execute the kernel commands
This is not a stand-alone kernel script."""

from abaqus import *
from customKernel import *
from abaqusConstants import NONE
from symbolicConstants import *
from types import *
from abq_Vcct.vcctConstants import *
import methodCallback

ValueError = '\nArgumentError'

#===============================
# Class to create VCCT Interface
#===============================
class Interface(CommandRegister):
    """The class creates a Interface in VCCT model
    Usage: Interface(name,contactPairName,bondNset,debondStepName)"""
    
    def __init__(self, name, contactPairName, analysisType, interfaceProperty, 
                 bondNset, debondStep, 
                 linearScaling=None, crackOutputFile=DAT,crackOutputFrequency=1,):

        self.verifyArguments(name, contactPairName,analysisType,
                             interfaceProperty,bondNset,debondStep,
                             linearScaling, crackOutputFile,
                             crackOutputFrequency,'__init__')
        CommandRegister.__init__(self)
        self.name = name
        
        # Since linear scaling may be None without being sent, need to 
        # set its value here
        #
        self.linearScaling=None
        self.setValues(contactPairName,analysisType,
                       interfaceProperty,bondNset,debondStep,
                       linearScaling,crackOutputFile,
                       crackOutputFrequency,'__init__')
        print 'VCCT plug-in interface "%s" has been created.' % self.name
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyArguments(self, name, contactPairName,analysisType,
                        interfaceProperty,bondNset,debondStep,
                        linearScaling,crackOutputFile,crackOutputFrequency,flag):
        """This function checks the type of the arguments of Interface class"""

        if flag == '__init__':
            if type(name) != StringType:
                raise ValueError,'name;expecting string, found %s'\
                      %type(name)
            # Additional check to impose case insenitivity of name string
            #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            if type(contactPairName) != StringType:
                raise ValueError,  'contactPairName; expecting string, found %s'\
                      %type(contactPairName)
            if type(interfaceProperty) != StringType:
                raise ValueError,  'interfaceProperty; expecting string, found %s'\
                      %type(interfaceProperty)
            if type(bondNset) != StringType:
                raise ValueError,  'bondNset; expecting string, found %s'\
                      %type(bondNset)
            if type(debondStep) != StringType:
                raise ValueError,  'debondStep; expecting string, found %s'\
                  %type(debondStep)
            if type(linearScaling) != FloatType and type(linearScaling) != IntType and \
                type(linearScaling) != NoneType:
                raise ValueError,  'linearScaling; expecting Float, found %s'\
                  %type(linearScaling)
            # Crack Output
            #-------------
            if type(crackOutputFile) != SymbolicConstantType and \
               type(crackOutputFile) != NoneType:
                raise ValueError,  'crackOutputFile; expecting SymbolicConstant, '\
                  'found %s' %type(crackOutputFile)
            if type(crackOutputFrequency) != IntType and crackOutputFrequency != None:
                raise ValueError,  'crackOutputFrequency; expecting Float, found %s'\
                  %type(crackOutputFrequency)
            
        elif flag == 'setValues':
            if type(contactPairName) != StringType and contactPairName != None and \
               type(contactPairName) != NoneType:
                raise ValueError,  'contactPairName; expecting string, found %s'\
                      %type(contactPairName)
            if type(interfaceProperty) != StringType and interfaceProperty != None and \
               type(interfaceProperty) != NoneType:
                raise ValueError,  'interfaceProperty; expecting string, found %s'\
                      %type(interfaceProperty)
            if type(bondNset) != StringType and bondNset != None and \
               type(bondNset) != NoneType:
                raise ValueError,  'bondNset; expecting string, found %s'\
                      %type(bondNset)
            if type(debondStep) != StringType and debondStep != None and \
               type(debondStep) != NoneType:
                raise ValueError,  'debondStep; expecting string, found %s'\
                  %type(debondStep)
            if type(linearScaling) != FloatType and type(linearScaling) != NoneType and \
               type(linearScaling) != IntType :
                raise ValueError,  'linearScaling; expecting Float, found %s'\
                  %type(linearScaling)
            # Crack Output
            #-------------
            if type(crackOutputFile) != SymbolicConstantType and \
               type(crackOutputFile) != NoneType and \
               crackOutputFile != None:
                raise ValueError,  'crackOutputFile; expecting SymbolicConstant, '\
                      'found %s' %type(crackOutputFile)
            if type(crackOutputFrequency) != IntType and type(crackOutputFrequency) != NoneType \
               and type(crackOutputFrequency) != FloatType and crackOutputFrequency != None:
                raise ValueError,  'crackOutputFrequency; expecting Int, found %s'\
                      %type(crackOutputFrequency)
            if type(crackOutputFrequency) == FloatType and \
               crackOutputFrequency-int(crackOutputFrequency) != 0:
                raise ValueError,  'crackOutputFrequency; expecting Int, found %s'\
                      %type(crackOutputFrequency)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def setValues(self, contactPairName=None, analysisType=None, 
                  interfaceProperty=None, bondNset=None,
                  debondStep=None,linearScaling=None,
                  crackOutputFile=None, crackOutputFrequency=None,
                  flagSetValues=None):
        """This function resets the values of the members of Interface class"""

        if flagSetValues != '__init__':
            self.verifyArguments(None,contactPairName,analysisType,interfaceProperty,
                                 bondNset,debondStep,
                                 linearScaling,crackOutputFile,
                                 crackOutputFrequency,'setValues')
        if contactPairName:
            self.contactPairName = contactPairName
        if analysisType == 1 or analysisType == 0:
            self.analysisType = analysisType
        if debondStep:
            self.debondStep = debondStep
        if interfaceProperty:
            self.interfaceProperty = interfaceProperty
        if bondNset:
            self.bondNset = bondNset
        if linearScaling or linearScaling == 0:
            self.linearScaling = linearScaling
        #-----------------------------------------------------    
        if crackOutputFile:
            self.crackOutputFile = crackOutputFile
            
        if crackOutputFrequency or crackOutputFrequency==0:
            self.crackOutputFrequency = crackOutputFrequency

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~       
    def copy(self,newName,newModel=None):
        """This method copies vcct interface data to a new name"""
        
        if newModel == None:
            vp = session.currentViewportName
            modelName = session.sessionState[vp]['modelName']
            curModel = mdb.customData.vcct.models[modelName]
            interfaceKeys = curModel.interfaces.keys()
        elif type(newModel) == StringType:
            if newModel in mdb.customData.vcct.models.keys():
                curModel = mdb.customData.vcct.models[newModel]
                interfaceKeys = curModel.interfaces.keys()
            else:
                raise AttributeError,'No model found with name: "%s" in vcct models'\
                      %newName
        else:
            raise ValueError,'expected string, found %s'%type(newName)
        if type(newName) == StringType and newName not in interfaceKeys:
            curModel.Interface(newName,self.contactPairName,
                               self.analysisType,
                               self.interfaceProperty,
                               self.bondNset,
                               self.debondStep,
                               self.linearScaling,
                               self.crackOutputFile,
                               self.crackOutputFrequency,)
        elif newName in interfaceKeys:
            raise NameError,'name "%s" already exists'%newName
        else:
            raise ValueError,'Expected string, found %s'%type(newName)

#================================================       
# Class to create old VCCT Interaction Property
# This is a legacy object, to allow upgrades
#================================================
#
class InteractionProperty(CommandRegister):
    """This is a dummy class used for upgrade purposes"""
    
    pass

#================================================       
# Class to create VCCT type interfaceProperty
#================================================
#
class InterfaceProperty(CommandRegister):
    """The class creates a interfaceProperty in VCCT model
    Usage: InterfaceProperty(name,G_IC,G_IIC,G_IIIC,fractureCriterion)"""

    def __init__(self, name, G_IC, G_IIC, G_IIIC,
                 fractureCriterion, eta=0, m=0, n=0, o=0,
                 damv=0.0, tol=0.2, criticalEnergySets=NONE):

        self.verifyArguments(name, G_IC, G_IIC, G_IIIC,
                             fractureCriterion, eta, m, n, o,
                             damv,tol, criticalEnergySets,'__init__')
        CommandRegister.__init__(self)
        self.name = name
        self.setValues(G_IC, G_IIC, G_IIIC,
                       fractureCriterion, eta, m, n, o, 
                       damv, tol, criticalEnergySets,'__init__')
        print 'VCCT plug-in interface property "%s" has been created.' % self.name

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyArguments(self,name, G_IC, G_IIC, G_IIIC,
                        fractureCriterion, eta, m, n, o,
                        damv, tol, criticalEnergySets, flag):
        """This function checks the type of the arguments of
        InterfaceProperty class"""

        if flag == '__init__':
            if type(name) != StringType:
                raise ValueError,'name;expecting string, found %s' \
                      %type(name)
            # Additional check to impose case insensitivity on name string
            #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            vp = session.currentViewportName
            modelName = session.sessionState[vp]['modelName']
            existingKeys = mdb.customData.vcct.models[modelName].interfaceProperties.keys()

            if type(G_IC) != FloatType and \
               type(G_IC) != IntType:
                raise ValueError, 'G_IC; expecting Float, found %s'\
                  %type(G_IC)
            if type(G_IIC) != FloatType and \
               type(G_IIC) != IntType:
                raise ValueError, 'G_IIC; expecting Float, found %s'\
                  %type(G_IIC)
            if type(G_IIIC) != FloatType and \
               type(G_IIIC) != IntType:
                raise ValueError, 'G_IIIC; expecting Float, found %s'\
                  %type(G_IIIC)
            if type(fractureCriterion) !=  SymbolicConstantType:
                raise ValueError, 'fractureCriterion; expecting  SymbolicConstant, '\
                  'found %s'%type(fractureCriterion)
            if type(eta) != FloatType and eta != None and \
               type(eta) != IntType:
                raise ValueError, 'eta; expecting Float, found %s'\
                      %type(eta)
            if type(m) != FloatType and m != None and \
               type(m) != IntType:
                raise ValueError, 'm; expecting Float, found %s'\
                  %type(m)
            if type(n) != FloatType and n != None and \
               type(n) != IntType:
                raise ValueError, 'n; expecting Float, found %s'\
                  %type(n)
            if type(o) != FloatType and o != None and \
               type(o) != IntType:
                raise ArgumeotError, 'o; expecting Float, found %s'\
                  %type(o)
            if type(damv) != FloatType and \
               type(damv) != IntType and damv != None:
                raise ValueError, 'damv; expecting Float, found %s'\
                  %type(damv)
            if type(tol) != FloatType and \
               type(tol) != IntType and tol!= None:
                raise ValueError, 'tol; expecting Float, found %s'\
                  %type(tol)
            if type(criticalEnergySets) != TupleType and \
               criticalEnergySets != NONE:
                raise ValueError, 'criticalEnergySets; expecting Tuple, found %s'\
                  %type(criticalEnergySets)
            if (fractureCriterion == BK_LAW or fractureCriterion == REEDER_LAW) and \
               eta == None:
                raise ValueError, 'eta should be specified with BK_LAW and REEDER_LAW'
            if fractureCriterion == POWER_LAW and m == None:
                raise ValueError, 'm should  be specified with POWER_LAW'
            if fractureCriterion == POWER_LAW and n == None:
                raise ValueError, 'n should be specified with POWER_LAW'
            if fractureCriterion == POWER_LAW and o == None:
                raise ValueError, 'o should be specified with POWER_LAW'

        elif flag == 'setValues':
            if type(G_IC) != FloatType and \
               type(G_IC) != IntType and type(G_IC) != NoneType:
                raise ValueError, 'G_IC; expecting Float, found %s'\
                  %type(G_IC)
            if type(G_IIC) != FloatType and \
               type(G_IIC) != IntType and type(G_IIC) != NoneType:
                raise ValueError, 'G_IIC; expecting Float, found %s'\
                  %type(G_IIC)
            if type(G_IIIC) != FloatType and \
               type(G_IIIC) != IntType and type(G_IIIC) != NoneType:
                raise ValueError, 'G_IIIC; expecting Float, found %s'\
                  %type(G_IIIC)
            if type(fractureCriterion) !=  SymbolicConstantType and \
               type(fractureCriterion) != NoneType:
                raise ValueError, 'fractureCriterion; expecting  SymbolicConstant,'\
                  ' found %s'%type(fractureCriterion)
            if type(eta) != FloatType and type(eta) != NoneType and \
               type(eta) != IntType  and eta != None:
                raise ValueError, 'eta; expecting Float, found %s'\
                  %type(eta)
            if type(m) != FloatType and type(m) != NoneType and \
               type(m) != IntType and m != None:
                raise ValueError, 'm; expecting Float, found %s'\
                  %type(m)
            if type(n) != FloatType and type(n) != NoneType and \
               type(n) != IntType  and n != None:
                raise ValueError, 'n; expecting Float, found %s'\
                  %type(n)
            if type(o) != FloatType and type(o) != NoneType and \
               type(o) != IntType  and o != None:
                raise ArgumeotError, 'o; expecting Float, found %s'\
                      %type(o)
            #---------------------------------
            if type(damv) != FloatType and \
               type(damv) != IntType and type(damv) != NoneType and\
               damv != None:
                raise ValueError, 'damv; expecting Float, found %s'\
                  %type(damv)
            if type(tol) != FloatType and \
               type(tol) != IntType and type(tol)!= NoneType and \
               tol != None:
                raise ValueError, 'tol; expecting Float, found %s'\
                  %type(tol)
            if type(criticalEnergySets) != TupleType and \
               type(criticalEnergySets) != NoneType and \
               criticalEnergySets != NONE:
                raise ValueError, 'criticalEnergySets; expecting Tuple, found %s'\
                      %type(criticalEnergySets)
            #
            if (fractureCriterion == BK_LAW or fractureCriterion == REEDER_LAW) and (not \
               eta or eta ==  None):
                raise ValueError, 'eta should be specified with BK_LAW and REEDER_LAW'
            if fractureCriterion == POWER_LAW and (m == None or not m):
                raise ValueError, 'm should  be specified with POWER_LAW'
            if fractureCriterion == POWER_LAW and (n == None or not n):
                raise ValueError, 'n should be specified with POWER_LAW'
            if fractureCriterion == POWER_LAW and (o == None or not o):
                raise ValueError, 'o should be specified with POWER_LAW'

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    def setValues(self, G_IC=None, G_IIC=None,G_IIIC=None,fractureCriterion=None, 
                  eta=None,m=None,n=None,o=None,damv=None,tol=None,
                  criticalEnergySets=None,flagSetValues=None):
        """This function resets the values of the members of InterfaceProperty class"""

        # Check the type of the arguments before assigning them
        if flagSetValues != '__init__':
            self.verifyArguments(None,G_IC,G_IIC,G_IIIC,
                                 fractureCriterion,eta,m,n,o,damv,tol,
                                 criticalEnergySets,'__setValues__')

        if G_IC or G_IC == 0:
            self.G_IC = G_IC
          
        if G_IIC or G_IIC == 0:
            self.G_IIC = G_IIC
           
        if G_IIIC or G_IIIC == 0:
            self.G_IIIC = G_IIIC
          
        if fractureCriterion:
            self.fractureCriterion = fractureCriterion

        if (fractureCriterion == BK_LAW or fractureCriterion == REEDER_LAW) and eta == None:
            raise ValueError, 'eta should be specified with BK_LAW or REEDER_LAW'
        else:
            if eta or eta == 0:
                self.eta = eta
                
        if m == None and fractureCriterion == POWER_LAW:
            raise ValueError, 'm should be specified with POWER_LAW'
        else:
            if m or m == 0:
                self.m = m
            
        if n == None and fractureCriterion == POWER_LAW :
            raise ValueError, 'n should be specified with POWER_LAW'
        else:
            if n or n == 0:
                self.n = n
        
        if o == None and fractureCriterion == POWER_LAW :
            raise ValueError, 'o should be specified with POWER_LAW'
        else:
            if o or o == 0:
                self.o = o
         
        if damv or damv == 0:
           self.damv = damv
            
        if tol or tol == 0:
            self.tol = tol
            
        if criticalEnergySets:
            self.criticalEnergySets = criticalEnergySets
        
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def copy(self,newName,newModel=None):
        """This method copies the vcct interfaceProperty to a new name"""

        if newModel == None:
            vp = session.currentViewportName
            modelName = session.sessionState[vp]['modelName']
            curModel = mdb.customData.vcct.models[modelName]
            propertyKeys = curModel.interfaceProperties.keys()
        elif type(newModel) == StringType:
            if newModel in mdb.customData.vcct.models.keys():
                curModel = mdb.customData.vcct.models[newModel]
                propertyKeys = curModel.interfaceProperties.keys()
            else:
                raise AttributeError,'No model found with name "%s"'%newName
        else:
            raise ValueError,'Expected string, found %s'%type(newModel)
        if type(newName) == StringType:
            if newName not in propertyKeys:
                curModel.InterfaceProperty(newName, self.G_IC, self.G_IIC, self.G_IIIC,
                                self.fractureCriterion,
                                self.eta, self.m, self.n, self.o,
                                self.damv, self.tol, self.criticalEnergySets)
            elif newName in propertyKeys:
                raise KeyError,'name "%s" already exists'%newName
        else:
            raise ValueError,'Expected string, found %s'%type(newName)

#================================================       
# Class to create VCCT contact clearance
#================================================
#
class ContactClearance(CommandRegister):
    """The class creates a contactClearance in VCCT model
    Usage: ContactClearance(name,adjust,clearance,searchNSet)"""

    def __init__(self, name, searchNSet, adjust=True, clearance=0.0,):

        self.verifyArguments(name, searchNSet, adjust, clearance, '__init__')
        CommandRegister.__init__(self)
        self.name = name
        self.setValues(searchNSet, adjust, clearance, '__init__')
        print 'VCCT plug-in contact clearance "%s" has been created.' % self.name

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyArguments(self,name, searchNSet, adjust, clearance, flag):
        """This function checks the type of the arguments of
        ContactClearance class"""

        if flag == '__init__':
            if type(name) != StringType:
                raise ValueError,'name;expecting string, found %s' \
                      %type(name)
            # Additional check to impose case insensitivity on name string
            #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            vp = session.currentViewportName
            modelName = session.sessionState[vp]['modelName']
            existingKeys = mdb.customData.vcct.models[modelName].contactClearances.keys()

            if type(searchNSet) != StringType: 
                raise ValueError, 'searchNSet; expecting String, found %s'\
                  %type(searchNSet)
            if type(adjust) != BooleanType:
                raise ValueError, 'adjust; expecting Boolean, found %s'\
                  %type(adjust)
            if type(clearance) != FloatType and \
               type(clearance) != IntType:
                raise ValueError, 'clearance; expecting Float, found %s'\
                  %type(clearance)

        elif flag == 'setValues':
            if type(searchNSet) != StringType: 
                raise ValueError, 'searchNSet; expecting String, found %s'\
                  %type(searchNSet)
            if type(adjust) != BooleanType:
                raise ValueError, 'adjust; expecting Boolean, found %s'\
                  %type(adjust)
            if type(clearance) != FloatType and \
               type(clearance) != IntType:
                raise ValueError, 'clearance; expecting Float, found %s'\
                  %type(clearance)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    def setValues(self, searchNSet=None, adjust=None, clearance=None,flagSetValues=None):
        """This function resets the values of the members of ContactClearance class"""

        # Check the type of the arguments before assigning them
        if flagSetValues != '__init__':
            self.verifyArguments(None,adjust,clearance,searchNSet,'__setValues__')

        if searchNSet:
            self.searchNSet = searchNSet
        if adjust or adjust == 0:
            self.adjust = adjust
        if clearance or clearance == 0:
            self.clearance = clearance
         
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def copy(self,newName,newModel=None):
        """This method copies the vcct contactClearance to a new name"""

        if newModel == None:
            vp = session.currentViewportName
            modelName = session.sessionState[vp]['modelName']
            curModel = mdb.customData.vcct.models[modelName]
            keys = curModel.contactClearances.keys()
        elif type(newModel) == StringType:
            if newModel in mdb.customData.vcct.models.keys():
                curModel = mdb.customData.vcct.models[newModel]
                keys = curModel.contactClearances.keys()
            else:
                raise AttributeError,'No model found with name "%s"'%newName
        else:
            raise ValueError,'Expected string, found %s'%type(newModel)
        if type(newName) == StringType:
            if newName not in keys:
                curModel.ContactClearance(newName, self.searchNSet,self.adjust, self.clearance, )
            elif newName in keys:
                raise KeyError,'name "%s" already exists'%newName
        else:
            raise ValueError,'Expected string, found %s'%type(newName)


#==================================
# Class to create models repository
#==================================
#
class Model(RepositorySupport):
    """This class creates a VCCT model with the current model name as its key"""
    
    def __init__(self,name):
        RepositorySupport.__init__(self)
        self.name = name
        if not hasattr(self, 'interface'):
            self.Repository('interfaces',Interface)
        if not hasattr(self, 'interfaceProperties'):
            self.Repository('interfaceProperties',InterfaceProperty)
        if not hasattr(self, 'interactionProperties'):
            self.Repository('interactionProperties',InteractionProperty)
        if not hasattr(self, 'contactClearances'):
            self.Repository('contactClearances',ContactClearance)

#==================================================================
# Class to create abstract class vcct to hold the vcct repositories
#==================================================================
#
class vcct(RepositorySupport):
    """This abstract class serves as a name space container for
    vcct specific models data"""

    def __init__(self):

       RepositorySupport.__init__(self)
       self.Repository('models', Model)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def iPToInterfacePropertyUpgrade():

    # Called from initializeMdb()
    # If here, then mdb object should have old interaction properties object
    # copy interface and interaction properties objects to temp variable,
    # call vcct() to wipe out old objects and install new ones, then
    # call constructor of new objects
    #
    from abaqus import mdb
    if not mdb.pathName == '<unnamed>':
        if 'vcct' in mdb.customData._getAttributeNames():
            if 'models' in mdb.customData.vcct._getAttributeNames():
                for modelName in mdb.customData.vcct.models:
                    m = mdb.customData.vcct.models[modelName]
                    objs = {'IntProp':[],'Interface':[]}
                    objs['IntProp'] = [m.interactionProperties[key] for key in m.interactionProperties.keys()]
                    objs['Interface'] = [m.interfaces[key] for key in m.interfaces.keys()]
                    mdb.customData.vcct = vcct()
                    mdb.customData.vcct.Model(modelName)
                    for iP in objs['IntProp']:
                        if iP.eta == NONE: iP.eta = None
                        if iP.m == NONE: iP.m = None
                        if iP.n == NONE: iP.n = None
                        if iP.o == NONE: iP.o = None
                        mdb.customData.vcct.models[modelName].InterfaceProperty(
                            iP.name, iP.G_IC, iP.G_IIC, iP.G_IIIC,
                            iP.fractureCriterion, iP.eta, iP.m, iP.n, iP.o,
                            iP.damv, iP.tol, iP.criticalEnergySets)
                        print 'Interface property object created from existent interaction property "%s".' % iP.name
                    for it in objs['Interface']:
                        if len(objs['IntProp']):
                            interfaceProperty = objs['IntProp'][0].name # use the first one
                            msgTxt = 'Interface property "%s" was added to interface "%s"' % (interfaceProperty, it.name)
                        else:
                            interfaceProperty = 'InterfaceProperty-1' # give a default name
                            msgTxt = 'Dummy interface property "InterfaceProperty-1" was added to interface "%s"\n'\
                                     'Ensure that interface properties are correct' % (interfaceProperty, it.name)
                        mdb.customData.vcct.models[modelName].Interface(it.name, it.contactPairName,
                               interfaceProperty, it.bondNset, it.debondStep, 0.9, 
                               it.crackOutputFile, it.crackOutputFrequency,)
                        print msgTxt


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def onModelCreate(mdbModel,posArgs,editModelData,userData):
    """This method creates/copies a model object in the vcct.models repository
    when a new model in mdb is created"""

    try:
        toModelName = editModelData['name']
        mdb.customData.vcct.Model(toModelName) # failing here
        fromModel = editModelData['objectToCopy']
        fromModelName = fromModel.name
        oldModel = mdb.customData.vcct.models[fromModelName]

        # Interfaces
        #
        for obj in [oldModel.interfaces,
                    oldModel.interfaceProperties,
                    oldModel.contactProperties,]:
            for key in obj.keys:
                obj[key].copy(key,toModelName)
    except:
        pass

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def onModelRename(callingMethod,posArgs,keywordArgs,userData):
    """This method renames the current model name in vcct"""

    callingObject = str(callingMethod)
    start = callingObject.find(' ')
    end = callingObject.find('changeKey')
    callingObject = callingObject[start+2:end-1]
    if callingObject == 'mdb.models':

        # Interactively, keywordArgs will be populated
        # Note, this will fail ugly if arguments change
        if keywordArgs.keys():
            oldName = keywordArgs['fromName']
            newName = keywordArgs['toName']

        # Through API, use posArgs
        #
        else:
            oldName = posArgs[0]
            newName = posArgs[1]
        verifyMdbForApp(None)
        if oldName in mdb.models.keys() and callingObject == 'mdb.models':
            mdb.customData.vcct.models.changeKey(fromName=oldName,toName=newName)

# The following callbacks are called for creating, renaming, copying a mdb model
# and make the corresponding changes to the vcct model.
#
methodCallback.addCallback(mdb, 'Model', onModelCreate,callAfter=True)
methodCallback.addCallback(mdb.models, 'changeKey', onModelRename)

