from abaqus import *
from customKernel import *
from job import JobType
from abaqusConstants import SMALL, BOTH, FILE
from abq_Vcct.vcctConstants import *
from abq_Vcct.vcctClassModule import *
import methodCallback, uti


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def errorMessageFunc(message):
    """
    Method to print error message based on the ABAQUS Version.
    Attributes:
        message - message to be printed
    Calling Methods: updateInputFile
    """
    print ValueError, message

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def keywordPosUsingExactString(modelName, searchString):
    """
    
    Method to find the index of a keyword search string in the
    sieBlocks object.
    Attributes:
        modelName - Name of the model.
        searchString - Exact string representing one line of a 
                       keyword block.
    Calling Methods: updateInputFile
    
    """
    searchIndex = []
    lcSearchString = searchString.lower()
    keyIndex = -1
    sieBlocksObj = mdb.models[modelName].keywordBlock.sieBlocks
    for keyBlocks in sieBlocksObj:
        keyIndex = keyIndex + 1
        blocks = keyBlocks.split('\n')
        for block in blocks:
            subBlocks = block.split(',')
            for subBlock in subBlocks:
                subBlock = subBlock.strip('')
                lcSubBlock = subBlock.lower()
                if lcSubBlock == lcSearchString:
                    searchIndex.append(keyIndex)
    return searchIndex

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def keywordPosUsingParameter(modelName, searchString):
    """

    Method to find the index of a case sensitive parameter
    in the sieBlocks object.
    Attributes:
        modelName - Name of the model.
        searchString - String whose occurence is to be found
                       after the equal ('=') sign.
    Calling Methods: updateInputFile
    
    """
    searchIndex = []
    keyIndex = -1
    fVal = -1
    sieBlocksObj = mdb.models[modelName].keywordBlock.sieBlocks
    for keyBlocks in sieBlocksObj:
        keyIndex = keyIndex + 1
        blocks = keyBlocks.split('\n')
        for block in blocks:
            subBlocks = block.split(',')
            for subBlock in subBlocks:
                fVal = subBlock.find(searchString)
                #Need for additional checks to get exact match
                if (fVal != -1):
                    try:
                        subList = subBlock.split('=')
                        subString = subList[1].strip('')
                        subString = subString.strip('"')
                        subString = subString.strip('')
                        if subString.lower() == searchString.lower():
                            searchIndex.append(keyIndex)
                    except:
                        pass
    return searchIndex

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def modifyKeywordBlock(kb, indexes, text ):

    """

    Method to modify specific keyword blocks using insert and replace
    methods.  It uses replace() if the keyword begins with the
    first 5 characters of string passed, otherwise it uses insert().
    Attributes:
        kb              - keyword block
        indexes         - indexes of blocks
        text            - string to insert or replace in block
    Calling Methods: updateInputFile
    
    """
    for index in indexes:
        j=-1
        for t in text:
            if t.startswith('**'):
                kb.insert(position=index+j, text=t)
            else:
                if t[:5] != kb.sieBlocks[index+j+1][:5]:
                    kb.insert(position=index+j, text=t)
                else:
                    kb.replace(position=index+j+1, text=t)
            j+=1

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getContClearAssignObjs(string):

    s = string.split('\n')
    ret = ','
    if len(s) == 2:
        if len(s[1].split(',')) == 2:
            ret = s[1]
    return ret

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getSurfName(surfObj):

    if surfObj[1] == 'Assembly':
        surfName = surfObj[0]
    else:
        surfName = surfObj[2] + '.' + surfObj[0]
    return surfName    
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def modStrWSpace(string):

    if string.__contains__(' '): string = '"' + string + '"'
    return string

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def updateInputFile(callingMethod, args, keywordArgs, userData):
    """
    Method to update the keywordBlock object with the VCCT keywords.
    """
    # Additional check for material evaluation jobs
    #
    if not hasattr(callingMethod.caller, 'model'):
        return
    try:
        modelName = callingMethod.caller.model
    except:
        errorMessageFunc(
            'Error: The callback procedure did not return a model name.\n')

    if not modelName in sorted(mdb.models.keys()):
        errorMessage =  ':Model Name "%s" sent to callback method ' \
                        'is not available in the mdb.\n' %(modelName)
        errorMessageFunc(errorMessage)

    if not hasattr(mdb.customData,'vcct'):
        errorMessage =  ':Custom data "vcct" does not exist.\n' \
                        'Please make sure that custom data object was \n' \
                        'properly initialized.'
        errorMessageFunc(errorMessage)

    # Get model and vcct model information
    #
    model = mdb.models[modelName]
    stepKeys = model.steps.keys()
    stepKeys.remove('Initial')
    if modelName not in mdb.customData.vcct.models.keys():
        print 'The model %s does not have a corresponding VCCT model.\n' \
              'This can occur if models were copied without the plug-in loaded. ' \
              'No VCCT information written to input file.' % modelName
    else:
        vcctModel = mdb.customData.vcct.models[modelName]
        vcctInterfaceProperties = vcctModel.interfaceProperties
        vcctInterfaces = vcctModel.interfaces
        vcctContactClearances = vcctModel.contactClearances
        keywordBlock = model.keywordBlock
        printWarnExp = 0

        # Create list of interface properties to check
        # if interface property name in interfaces are valid
        #
        interfacePropertyNameList = [vcctInterfaceProperties[d].name for d in vcctInterfaceProperties]

        # Create a dictionary with available steps as keys
        # Dictionary will hold interface property and output used in each step
        #
        debondStepDict = {}
        for step in stepKeys:
            debondStepDict[step] = []

        # Flag for flattened input file
        #
        noPartsInputFileFlag = 0
        if model.noPartsInputFile == ON:
            noPartsInputFileFlag = 1

        """

        Need to iterate through the Interface object under VCCT
        customData to add:
        
            To model level:

                # STANDARD
                *Contact Pair
                *Initial Condition

                # EXPLICIT
                *Contact Clearance
                *Nodal Energy Rate
                *Fracture Criteria

            To step level:

                # STANDARD
                *Debond
                *Controls
                *Nodal Energy Rate
                *Fracture Criteria

                # EXPLICIT

                *Contact Clearance Assignment


        Need to check which contact (pair) interaction has been
        definied in Initial step and collect all interface properties
        and their corresponding starting step

        """
        rmvdKeywords=[]
        try:

            # Write only if at least one interface and one interface
            # property exists
            #
            i=0
            for sb in keywordBlock.sieBlocks:
                if sb.lower().startswith('*debond'):
                    if keywordBlock.sieBlocks[i+1].lower().startswith('*fracture criterion'):
                        keywordBlock.replace(i+1,'')
                        rmvdKeywords.append("*Fracture criterion")
                                                       
                        if keywordBlock.sieBlocks[i+1] == '**':
                            keywordBlock.replace(i+2,'')
                    keywordBlock.replace(i,'')
                if sb.lower().startswith('*controls, type=vcct'):
                    keywordBlock.replace(i,'')
                    rmvdKeywords.append("*Controls, type=vcct")
                                 
                if sb.lower().startswith('*contact clearance,'):
                    keywordBlock.replace(i,'')
                    rmvdKeywords.append("*Contact clearance")

                    
                if sb.lower().startswith('*contact clearance assignment'):
                    keywordBlock.replace(i,'')
                    rmvdKeywords.append("*Contact clearance assignment")

                if sb.lower().startswith('*fracture criterion,'):
                    keywordBlock.replace(i,'')
                    rmvdKeywords.append("*Fracture criterion")
               
                if sb.lower().startswith('*nodal energy rate'):
                    keywordBlock.replace(i,'')
                    rmvdKeywords.append("*Nodal energy rate")
          
                   
                if sb.lower().startswith('*initial conditions, type=contact'):
                    keywordBlock.replace(i,'')
                    rmvdKeywords.append("*Initial conditions, type=contact")
       
                
                i+=1
            keywordBlock.synchVersions()            
            rmvdkeywordList=' '.join(rmvdKeywords)
            print "The following keywords are removed by the VCCT plug-in: ",  rmvdkeywordList,\
            "\nYou may add the removed keywords manually to the input file."
            if len(vcctInterfaceProperties) and len(vcctInterfaces):

                # This module checks if *Contact Pair or *Initial Conditions exist.
                # Do not worry about duplicates
                #
                # But need to remove *Debond, *Fracture Criteria, 
                # *Contact Clearance Assignment, *Fracture criterion,
                # *Nodal energy rate and *Controls.
                # before writing new ones
                #

                # Iterate through VCCT interface objects
                #
                contactPairList = []
                for interface in sorted(vcctInterfaces.keys()):

                    # Contact Pair name
                    #
                    contactPairName = vcctInterfaces[interface].contactPairName

                    # 1 for EXPLICIT
                    # 0 for STANDARD
                    #
                    explicitType = vcctInterfaces[interface].analysisType

                    # Additional vars
                    #
                    linearScaling = vcctInterfaces[interface].linearScaling
                    interfacePropertyName = vcctInterfaces[interface].interfaceProperty
                    crackOutputFile = vcctInterfaces[interface].crackOutputFile
                    crackOutputFrequency = vcctInterfaces[interface].crackOutputFrequency

                    # Bond node set for STANDARD
                    # This variables refers to Contact Clearance name for EXPLICIT analysis
                    #
                    if explicitType:
                        bondNset = vcctInterfaces[interface].bondNset
                        if bondNset not in vcctModel.contactClearances.keys():
                            errorMessage =  ':VCCT interface "%s" references missing '\
                                            'contact clearance named "%s".' % (interface, bondNset)
                            errorMessageFunc(errorMessage)
                        else:
                            contactClearance = vcctContactClearances[bondNset]
                    else:
                        # Slave and Master
                        #
                        bondNset = modStrWSpace(vcctInterfaces[interface].bondNset)
                        slave = getSurfName(model.interactions[contactPairName].slave)
                        master = getSurfName(model.interactions[contactPairName].master)

                    # Check if interface property is in interface property name list
                    #
                    if interfacePropertyName not in interfacePropertyNameList:
                        errorMessage =  ':VCCT interface "%s" references missing '\
                                        'interface property named "%s".' % (interface, interfacePropertyName)
                        errorMessageFunc(errorMessage)

                    # Interface property name
                    #
                    debondStep = vcctInterfaces[interface].debondStep
                    interfaceProperty = vcctModel.interfaceProperties[interfacePropertyName]

                    # For EXPLICIT 
                    #
                    if explicitType:

                        # Get indexes for Surface Interaction in keyword blocks.
                        # Create text string with all of the *Contact Clearance before the first
                        #
                        searchString = '*Surface Interaction'
                        keywordBlockIndexes = keywordPosUsingExactString(modelName, searchString)
                        if keywordBlockIndexes == []:
                            errorMessageFunc(':No Surface Interaction found in keyword block.')
                        else:
                            if len(vcctContactClearances.keys()) > 1:
                                printWarnExp += 1
                            keywordBlockIndex = keywordBlockIndexes[0]
                            for key in vcctContactClearances.keys():
                                adjust, clearance = '', ''
                                if not vcctContactClearances[key].adjust:
                                    adjust = ', ADJUST=NO'
                                if vcctContactClearances[key].clearance:
                                    clearance = ', CLEARANCE=%s' % vcctContactClearances[key].clearance
                                text = '*contact clearance, name="%s", search nset=%s%s%s\n' % \
                                       (vcctContactClearances[key].name,
                                        vcctContactClearances[key].searchNSet,
                                        adjust,
                                        clearance)
                            keywordBlock.insert(position=keywordBlockIndex-1, text=text)

                        # Get indexes for Cohesive behavior in keyword blocks.
                        # Create text string with *Fracture criterionore after it
                        # Add Fracture Criterion (EXPLICIT is model level)
                        # and nodal energy rate if needed
                        #
                        searchString = '*Cohesive behavior'
                        keywordBlockIndexes = keywordPosUsingExactString(modelName, searchString)
                        if keywordBlockIndexes == []:
                            errorMessageFunc(':No Cohesive Behavior found in keyword block.')
                        else:
                            keywordBlockIndex = keywordBlockIndexes[0]

                            # Nodal Energy Rate
                            # It goes after Fracture Criteria, but need to check if to add to
                            # fracture criteria
                            #
                            if interfaceProperty.criticalEnergySets != NONE:
                                cESText = 'NODAL ENERGY RATE, ' # Fracture Criterion keyword
                                text = '*Nodal Energy Rate\n'
                                for setInfo in interfaceProperty.criticalEnergySets:
                                    text += str(setInfo[0]) + ', ' + str(setInfo[1]) + ', ' + \
                                            str(setInfo[2]) + ', ' + str(setInfo[3]) + ' \n'
                                keywordBlock.insert(position=keywordBlockIndex, text=text)
                            else:
                                cESText = ''

                            # Fracture Criterion
                            #
                            G_Values = ' %s, %s, %s, ' % (str(interfaceProperty.G_IC),
                                                          str(interfaceProperty.G_IIC),
                                                          str(interfaceProperty.G_IIIC))
                            text = ''
                            text += '*Fracture Criterion, tolerance=' + str(interfaceProperty.tol) + \
                                   ', type=vcct, ' + cESText + 'MIXED MODE BEHAVIOR=' 

                            if interfaceProperty.fractureCriterion == POWER_LAW:
                                text += 'POWER\n' + G_Values + \
                                        str(interfaceProperty.m) + ', ' + str(interfaceProperty.n) + ', ' + \
                                        str(interfaceProperty.o) + '\n'
                            else:
                                if interfaceProperty.fractureCriterion == REEDER_LAW:
                                    text += 'REEDER\n'
                                else:
                                    text += 'BK\n'
                                text += G_Values + str(interfaceProperty.eta) + '\n'
                            keywordBlock.insert(position=keywordBlockIndex, text=text)

                    # For STANDARD
                    #
                    else:


                        # Get indexes Contact Pairs in keyword blocks given interaction.
                        # Create text string with Initial Conditions for each Contact Pair
                        #
                        text = []
                        searchString = '** Interaction: ' + contactPairName
                        keywordBlockIndexes = keywordPosUsingExactString(modelName, searchString)
                        if keywordBlockIndexes == []:
                            errorMessageFunc(':No %s found in keyword block.' % contactPairName)
                        else:
                            for keywordBlockIndex in keywordBlockIndexes:
                                i = 0
                                if '*Contact Pair' in keywordBlock.sieBlocks[keywordBlockIndex+1+i]:
                                    text.append('*initial conditions, type=contact\n' + slave + ', ' + master +
                                                ', ' + bondNset)
                                    modifyKeywordBlock(keywordBlock, [keywordBlockIndex+2+i], text)
                                    i+=1
                                else:
                                    errorMessageFunc(':Contact pair "%s" not found' % contactPairName)
                        
                        # Collect interfaceProperty information used in step 
                        # (and every step hereafter) of interaction
                        # Slave and Master name need are also added
                        #
                        # Iterate through steps
                        #
                        for step in stepKeys[stepKeys.index(debondStep):]:
                            
                            if debondStepDict[step] == []:
                                
                                debondStepDict[step].append((interfacePropertyName, slave, master, linearScaling,
                                                             crackOutputFile, crackOutputFrequency))
                            
                            else:

                                if interfacePropertyName not in [l[0] for l in debondStepDict[step]]:
                                    debondStepDict[step].append((interfacePropertyName, slave, master, linearScaling,
                                                                 crackOutputFile, crackOutputFrequency))
                                      
                                else:
                                    #pass
                                    debondStepDict[step].append((interfacePropertyName, slave, master, linearScaling,
                                                                 crackOutputFile, crackOutputFrequency))                    

                # For EXPLICIT 
                #
                if explicitType:

                    # We do not have access to general contact object through scripting.
                    # Thus, searching for *Contact clearance assignment is done 
                    # through finding a *Contact Property Assignment in the keyword block
                    # and copying the master and slave values
                    # If more than one contact property assignment is found, throw an error
                    #
                    searchString = '*Contact Inclusions'
                    keywordBlockIndexes = keywordPosUsingExactString(modelName, searchString)
                    if keywordBlockIndexes == []:
                        # No entries found
                        errorMessageFunc(':No Contact Inclusions found in keyword block.')
                    else:
                        if len(keywordBlockIndexes) > 1:
                            printWarnExp += 1
                        keywordBlockIndex = keywordBlockIndexes[0]
                        mastSlav = getContClearAssignObjs(keywordBlock.sieBlocks[keywordBlockIndex])
                        for key in vcctContactClearances.keys():
                            text = '*Contact Clearance Assignment\n' + \
                                   mastSlav + ', "' + key + '"'
                            keywordBlock.insert(position=keywordBlockIndex, text=text)

                # For STANDARD
                #
                else:
                    # For each Step, append each interfaceProperty necessary
                    #
                    addedControlsKw = 0
                    for step in sorted(debondStepDict):

                        text = ''
                        
                        for interfaceInfoList in debondStepDict[step]:
                            
                            interfaceProperty = vcctModel.interfaceProperties[interfaceInfoList[0]]

                            # Check if nodal energy rate will be added
                            #
                            if interfaceProperty.criticalEnergySets != NONE:
                                cESText = 'NODAL ENERGY RATE, ' # Fracture Criterion keyword
                            else:
                                cESText = ''

                            # Fracture criterion g values
                            #
                            G_Values = ' %s, %s, %s, ' % (str(interfaceProperty.G_IC),
                                                          str(interfaceProperty.G_IIC),
                                                          str(interfaceProperty.G_IIIC))

                            # Output
                            #
                            cOFile = ''
                            cOFrequency = ''
                            if interfaceInfoList[5] != 0:
                                cOFrequency = 'FREQUENCY=' + str(interfaceInfoList[5]) + ', '
                                if interfaceInfoList[4] == FILE:
                                    cOFile = 'OUTPUT=FILE, '
                                elif interfaceInfoList[4] == BOTH:
                                    cOFile = 'OUTPUT=BOTH, '

                            # Debond, fracture criterion, nodal energy rate
                            #
                            text += '*Debond, slave=' + interfaceInfoList[1] + \
                                   ', master=' + interfaceInfoList[2] + ', ' + cOFrequency + \
                                   cOFile + 'viscosity=' + str(interfaceProperty.damv) + '\n'
                            text += '*Fracture Criterion, tolerance=' + str(interfaceProperty.tol) + \
                                   ', type=vcct, ' + cESText + 'MIXED MODE BEHAVIOR=' 
                            if interfaceProperty.fractureCriterion == POWER_LAW:
                                text += 'POWER\n' + G_Values + \
                                        str(interfaceProperty.m) + ', ' + str(interfaceProperty.n) + ', ' + \
                                        str(interfaceProperty.o) + '\n'
                            else:
                                if interfaceProperty.fractureCriterion == REEDER_LAW:
                                    text += 'REEDER\n'
                                else:
                                    text += 'BK\n'
                                text += G_Values + str(interfaceProperty.eta) + '\n'

                            # Now add nodal energy rate
                            #
                            if interfaceProperty.criticalEnergySets != NONE:
                                text += '*Nodal Energy Rate\n'
                                for setInfo in interfaceProperty.criticalEnergySets:
                                    text += str(setInfo[0]) + ', ' + str(setInfo[1]) + ', ' + \
                                            str(setInfo[2]) + ', ' + str(setInfo[3]) + ' \n'

                            if (not addedControlsKw) and interfaceInfoList[3]:
                                text += '*Controls, type=VCCT LINEAR SCALING\n' + \
                                         str(interfaceInfoList[3]) + ',\n'
                                addedControlsKw = 1

                        # For each step, place the keywords before each *Boundary
                        # If not available, place at an arbitrary two indexes down,which should exist
                        #
                        stepIndex = keywordPosUsingParameter(modelName, step)[0]
                        boundaryIndexes = keywordPosUsingExactString(modelName, '** BOUNDARY CONDITIONS')
                        if boundaryIndexes:
                            boundaryIndex = [bi for bi in boundaryIndexes if bi > stepIndex]
                            if boundaryIndex:
                                keywordBlock.insert(position=boundaryIndex[0]-1, text=text)
                            else:
                                keywordBlock.insert(position=stepIndex+1, text=text)
                        else:
                            keywordBlock.insert(position=stepIndex+1, text=text)
                        
                # End
                keywordBlock.synchVersions()
                if printWarnExp:
                    print 'Warning: More than one surface interaction and/or contact definitions found.\n'\
                          'Please check input file for duplicate or incorrect assignments.'

        except:
            print ValueError, 'An error occurred inserting keywords to the keyword block.\n'\
                'This error is due to the VCCT plug-in.\n'\
                'Please contact support for details.'

methodCallback.addCallback(JobType, 'writeInput', updateInputFile)
methodCallback.addCallback(JobType, 'submit', updateInputFile)

