# This module includes the derived classes for Manager and Copy forms

from abaqusGui import *
from abq_ViewerSetSupport.createSetProc import CreateSetProc
from abq_CaeManagerTemplate.managerForm import ManagerForm
from abq_CaeManagerTemplate.copyForm import CopyForm

###########################################################################
# Viewer Set Manager Class definition
###########################################################################

class ViewerSetManForm(ManagerForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
    
        CreateForm = CreateSetProc(owner, 'CREATE')
        EditForm = CreateSetProc(owner, 'EDIT')

        # Need to set mode names since infrastructure may default to
        # the first object when activating or calling getFirstStep/dialog
        #
        CreateForm.setModeName('CreateForm')

        self.repository = 'session.odbs[%s].rootAssembly'
        self.baseName = 'Set'

        ManagerForm.__init__(self,
                             owner,
                             CreateForm,
                             EditForm,
                             self.repository,
                             self.baseName,
                             'Set Manager')
        
        self.managerLabels = 'Name\tType\tInstance'
        # Set values on the create and edit forms.
        #
        self.createForm.setValues(self.repository, self.baseName)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def activate(self):
        """
        Reimplemented to check if an ODB is displayed in the viewport or not
        """
        if not getDisplayedObjectType() == ODB:
            errMsg = 'An output database must be displayed in the '\
                     'current viewport.'
            showAFXErrorDialog(getAFXApp().getAFXMainWindow(), errMsg)
            return None

        # Create a dummy command to use the expandObjectName functionality
        #
        self.cmd = AFXGuiCommand(
            self, 'NodeSet', 'session.odbs[%s].rootAssembly')

        AFXForm.activate(self)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        if not self.db:
            from abq_ViewerSetSupport.viewerSetManDB import ViewerSetManDB
            self.db = ViewerSetManDB(self, self.repository, self.managerTitle, 
                self.managerLabels)
        return self.db      

###########################################################################
# Viewer Set Copy Class definition
###########################################################################

class ViewerSetCopyForm(CopyForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, formOwner, dbOwner, repository):

        CopyForm.__init__(self, formOwner, dbOwner, repository)
        self.defaultObj = repository

        self.manDB = dbOwner
        self.type = 'Node'
        self.instance = 'rootAssembly'

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):

        # Overwrite the keywords for copy
        #

        self.newNameKw.deactivate()
        if self.cmd.getNumKeywords() < 2:
            self.cmd = self.manDB.form.copyForm.cmd
            self.nameKw = AFXStringKeyword(self.cmd, 'name', True)
            self.nodesKw = AFXObjectKeyword(self.cmd, 'nodes', True, 'objects')
            self.elementsKw = AFXObjectKeyword(self.cmd, 'elements', True, 'objects')

        row = self.getCurrentRow(self.manDB.table)

        repository = self.repository
        if self.instance != 'rootAssembly':
            repository += '.instances["%s"]' % self.instance

        if self.type == 'Node':
            repository += '.nodeSets'
            self.nodesKw.activate()
            self.elementsKw.deactivate()
            cmd = 'objects=%s["%s"].nodes' % (repository, self.oldName)
        else:
            repository += '.elementSets'
            self.nodesKw.deactivate()
            self.elementsKw.activate()
            cmd = 'objects=%s["%s"].elements' % (repository, self.oldName)

        exec('existingNames = %s.keys()' % repository )
        newName = self.newNameKw.getValue()
        if newName in existingNames:
            showAFXErrorDialog(self.getCurrentDialog(),
                '"%s" already exists.\n\n' 
                'Delete that object first, \n'
                'or specify a different name.' % newName
            )
            return False
        sendCommand(cmd)
        self.nameKw.setValue(newName)

        return True

    # This method returns the selected (highlighted) row. 
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getCurrentRow(self, table):

        # Get number of rows
        #
        return [r for r in range(1,table.getNumRows()) if table.isRowSelected(r)][0]

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def setValues(self, name): 

        row = self.getCurrentRow(self.manDB.table)
        self.oldName = self.manDB.table.getItemText(row,0)
        self.type = self.manDB.table.getItemText(row,1)
        self.instance = self.manDB.table.getItemText(row,2)

        self.repository = self.cmd.getExpandedObjectName()
       
        # Check type of set
        #
        if self.type == 'Node':
            self.cmd.setMethod('NodeSet')
        else:
            self.cmd.setMethod('ElementSet')

        # Check instance
        #
        if self.instance == 'rootAssembly':
            self.cmd.setObjectName(self.repository)
        else:
            self.cmd.setObjectName(
                self.repository + '.instances["%s"]' % self.instance)

        self.newNameKw.setValue(name)