from abaqusGui import *
from abq_hic.crashAttrSymbolicConstants import *

###########################################################################
# Class definition
###########################################################################

class HICResultantForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):

        self.owner = owner
        AFXForm.__init__(self, owner=owner)
        self.getHicCmd   = AFXGuiCommand(mode=self,
                                         method='hic_resul',
                                         objectName='abq_hic.hicUtils',
                                         registerQuery=FALSE)
        self.resultPlotKw = AFXTupleKeyword(command=self.getHicCmd,
                                            name='resultPlot',
                                            isRequired=TRUE,
                                            opts=AFXTUPLE_TYPE_STRING)
        self.resultTypeKw = AFXSymConstKeyword(command=self.getHicCmd,
                                               name='resultType',
                                               isRequired=TRUE,
                                               defaultValue=COMP_TYPE.getId())
        self.timeScaleFactorKw = AFXSymConstKeyword(command=self.getHicCmd,
                                                    name='timeScaleFactor',
                                                    isRequired=TRUE,
                                                    defaultValue=TSF_SECOND.getId())
        self.lengthScaleFactorKw = AFXSymConstKeyword(command=self.getHicCmd,
                                                    name='lengthScaleFactor',
                                                    isRequired=TRUE,
                                                    defaultValue=LSF_MILLIMETER.getId())
        self.hicWindowKw = AFXSymConstKeyword(command=self.getHicCmd,
                                              name='hicWindow',
                                              isRequired=TRUE,
                                              defaultValue=WIN_36.getId())
        self.saeFilterKw = AFXStringKeyword(command=self.getHicCmd,
                                              name='saeFilter',
                                            isRequired=TRUE,
                                            defaultValue='NONE')

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def activate(self):
        """
        Reimplemented to check if XY-Plot exists.
        """
        from kernelAccess import session

        plotsPresent = 0

        if session.xyDataObjects.keys():
            plotsPresent = 1

        for odbName in session.odbs.keys():
            odb = session.odbs[odbName]
            try:
                if odb.userData.xyDataObjects.keys():
                    plotsPresent = 1
                    break
            except:
                pass

        if not plotsPresent:

            errMsg = 'There is no XY Data available in the session.\n '\
                     'At least one XY Data is required.'

            showAFXInformationDialog(getAFXApp().getAFXMainWindow(), errMsg)

            return None

        return AFXForm.activate(self)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        """
        Return the first dialog to be posted
        """
        
        from abq_hic.hicResDB import HicResDB
        self.hicResDB = HicResDB(self)

        return self.hicResDB

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def handleException(self, exception):
        """
        Overload the exception handling method
        """
        
        errorType  = exception[0]
        errorValue = exception[1]
        db         = self.getCurrentDialog()

        showAFXErrorDialog(db, str(errorValue))

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):
        """
        Checks for invalid entries.
        """

        db = self.getCurrentDialog()

        if db.checkButtonSAE.getCheck():
            self.saeFilterKw.activate()
        else:
            self.saeFilterKw.deactivate()
        
        if not self.resultPlotKw.getValue(0):
            errMsg = 'A valid XY Data must be selected to compute HIC.'
            showAFXErrorDialog(getAFXApp().getAFXMainWindow(), errMsg)
            return 0
        
        inputType = self.resultTypeKw.getValue()
        if inputType == RESULT_TYPE.getId():
            self.resultPlotKw.removeElements(1,2)
        elif inputType == COMP_TYPE.getId():
            firstComp = self.resultPlotKw.getValue(0)
            secondComp = self.resultPlotKw.getValue(1)
            thirdComp = self.resultPlotKw.getValue(2)
            if not firstComp or not secondComp or not thirdComp:
                errMsg = "XY Data has not been selected for one of the " \
                         "acceleration components."
                showAFXErrorDialog(getAFXApp().getAFXMainWindow(), errMsg)
                return 0
            if firstComp in [secondComp, thirdComp] or \
               secondComp in [firstComp, thirdComp]:
                errMsg = "The same XY Data cannot be used for more than " \
                         "one component."
                showAFXErrorDialog(getAFXApp().getAFXMainWindow(), errMsg)
                return 0

        return 1    
