"""acousticVis.py -- Acoustic Pressure Visualization script.

An ODB API script that adds elements to a new odb upon which acoustic
pressure output can be visualized.

The odb being displayed in the active viewport will be used.
A new odb will be created with the name currentOdbName_acvis.odb.
If this odb already exists data will be appended to it.

Operations that are carried out on new odb:

   Membrane elements corresponding to the named set of infinite elements
   will be created at the correct radius.
   For each analysis step and frequency requested by the user output will be
   generated.

   The created odb will have the same number of steps and frames as the 
   original odb. Only the steps and frequencies
   requested will have output.

   For reference purposes the original infinite elements will be
   recreated in the new odb. 


User Input:

   o  Infinite elements set name:  If left blank all infinite elements
                                   in the model will be located and used
                                   to create the visualization surface.

   o  Radius of visualization sphere: REQUIRED

   o  Center point for sphere: If left blank will use (0,0,0)

   o  Steps: An integer, a list of integers (comma separated),
             a range (1:20), or a combination of the two e.g. 4,5,10:20, 30.
             If left blank all steps will be used.
             Any steps that are not Steady State or have no results
             will be ignored.

   o  Frequencies: A float, a list of floats or a range.
                   If left blank all frequencies will be used.

   o  Decibel reference value: REQUIRED

   o  Created instance name:  REQUIRED - this name will be appended
                              to the instance name (in a flattened input
                              deck this will always be PART-1-1) of the
                              infinite elements being used.

   o  Speed of sound: REQUIRED - to be given in units consistent with
                             those used in the analysis.

   o  Write to original ODB. The default is NO. 
                                


Disclaimer
----------

This script is provided as an example of the scripting interface and
demonstrates how to visualize results from Acoustic Infinite elements.

This script is not an official product of Abaqus INC. and has not been
through any formal Quality Assurance procedure.

Abaqus INC. cannot accept liability for any consequences or damages
which may result from the use of this script.

Features
--------
    o  Abaqus/Standard V6.4-PRE6 or later required.
    o  Applicable to *STEADY STATE DYNAMICS, DIRECT analysis steps only
    o  Supported 3D elements: ACIN3D3, ACIN3D4, ACIN3D6, ACIN3D8 only
    o  2D elements not supported.
    o  Output created: POR, PFAR, PORdB, PFARdB, INTEN_FAR
    


How To
------

The script can be installed anywhere and accessed through the File->Run script
menu item in Abaqus/CAE or Abaqus/Viewer. An odb of the required type and of
the required vintage should be being viewed in the current viewport.
The script operates on the odb that is being displayed in the currently
active viewport.

Any error messages, warning messages and information messages are printed
to the message area of the CLI.

"""

from abaqus import *
from abaqusConstants import *
from visualization import *

import math
import sys, os
import numpy as Numeric
import profile
from abaqusExceptions import excTyValStr

HUGE=1000000000.0

C33 = 15./4.0
C43 = 63.0/4.0
C73 = 675.0/4.0
C83 = 1155.0/4.0

C74 = 825./2.0
C84 = 1925./2.0

C55 = 105./8.0
C65 = 825./8.0
C75 = 7425./16.0
C85 = 25025./16.0
C95 = 35035./8.0

C66 = 99./4.0
C76 = 3861./16.0
C86 = 21021./16.0
C96 = 21021./4.0

C77 = 3003./4.0/16.0
C87 = 35035./4.0/16.0
C97 = 7007./2.0

C88 = 715./8.0
C98 = 2431./2.0

C99 = 21879./128.

CM=Numeric.zeros((10,))
CM[0] = 1.0

for i in xrange(1,10):
    CM[i] = 2*math.sqrt((2*i+1.)/4.0)

COEFF=Numeric.zeros((9,9),Numeric.float)

for i in xrange(0,9):
    for j in xrange(0,9):
        COEFF[i][j]=0.

COEFF[0][0]=1.

COEFF[1][0]=1.
COEFF[1][1]=2.

COEFF[2][0]=1.
COEFF[2][1]=5.
COEFF[2][2]=C33

COEFF[3][0]=1.
COEFF[3][1]=9.
COEFF[3][2]=C43
COEFF[3][3]=7.

COEFF[4][0]=1.
COEFF[4][1]=14.
COEFF[4][2]=42.
COEFF[4][3]=42.
COEFF[4][4]=C55

COEFF[5][0]=1.
COEFF[5][1]=20.
COEFF[5][2]=90.
COEFF[5][3]=150.
COEFF[5][4]=C65
COEFF[5][5]=C66

COEFF[6][0]=1.
COEFF[6][1]=27.
COEFF[6][2]=C73
COEFF[6][3]=C74
COEFF[6][4]=C75
COEFF[6][5]=C76
COEFF[6][6]=C77

COEFF[7][0]=1.
COEFF[7][1]=35.
COEFF[7][2]=C83
COEFF[7][3]=C84
COEFF[7][4]=C85
COEFF[7][5]=C86
COEFF[7][6]=C87
COEFF[7][7]=C88

COEFF[8][0]=1.
COEFF[8][1]=44.
COEFF[8][2]=462.
COEFF[8][3]=2002.
COEFF[8][4]=C95
COEFF[8][5]=C96
COEFF[8][6]=C97
COEFF[8][7]=C98
COEFF[8][8]=C99

# Python-2.6 does not allow string exceptions
# Change to class exceptions - April 2010
class AcousticVisException(Exception):
    pass

class NO_ODB(AcousticVisException): pass
class WRONG_ANALYSIS_CODE(AcousticVisException): pass
class WRONG_ODB_VERSION(AcousticVisException): pass
class NO_SSD_STEPS(AcousticVisException): pass
class NO_SETS(AcousticVisException): pass
class NO_INF_ELS(AcousticVisException): pass
class STEPS_MISMATCH(AcousticVisException): pass
class FRAMES_MISMATCH(AcousticVisException): pass
class INTERSECTION_ERROR(AcousticVisException): pass
class NAME_ERROR(AcousticVisException): pass
class OUTPUT_ERROR(AcousticVisException): pass
class ODB_ALREADY_OPEN(AcousticVisException): pass
class CANCELLING(AcousticVisException): pass


raiseMsg={}
raiseMsg[NO_ODB] = 'There is no output database (ODB) associated with'\
                     ' the current viewport.\n'\
                'Please select an ODB from the context bar or use "File-Open"\n'\
                'to open an ODB.'

raiseMsg[WRONG_ANALYSIS_CODE] = 'The output database (ODB) associated with'\
                                  ' the current viewport\n'\
                             'was created using %s. For this script to work\n'\
                             'an original analysis odb using Abaqus/Standard must be used.'

raiseMsg[WRONG_ODB_VERSION] = 'The output database (ODB) associated'\
                                ' with the current viewport\n'\
                           'was created using %s. The earliest version that\n'\
                           'can be used is 6.4-PRE6. '

raiseMsg[NO_SSD_STEPS] = 'Exterior acoustic pressure visualization'\
                           ' can only be performed for\n'\
                      '"Steady State Dynamics, Direct" steps. '\
                      'No Steady State Dynamics steps were found\n'\
                      'in the output database (ODB) associated with'\
                      ' the current viewport.'
 
raiseMsg[NO_INF_ELS] = 'No infinite elements of type ACIN3D3, ACIN3D4, '\
                         'ACIN3D6, ACIN3D8\n'\
                         'found for element set %s.' 
raiseMsg[STEPS_MISMATCH] = 'Visualization ODB exists but has different '\
                             'step definitions to analysis ODB. This visualization\n'\
                             'ODB cannot be used with this analysis ODB.\n'\
                             'To proceed please delete or rename the visualization ODB.'
raiseMsg[FRAMES_MISMATCH] = 'Visualization ODB exists but has different '\
                              'frame definitions to analysis ODB. This visualization\n'\
                             'ODB cannot be used with this analysis ODB.\n'\
                             'To proceed please delete or rename the visualization ODB.'
raiseMsg[INTERSECTION_ERROR] = 'Visualization surface must lie completely within '\
                                 'the volume of the selected infinite elements.\n'\
                                 'Please check your visualization radius '\
                                 'and centre point definitions.'
raiseMsg[NAME_ERROR] = 'Results already exist on a part named %s '\
                         'and cannot be overwritten.'\
                         'Please change the set name to be created.'
raiseMsg[OUTPUT_ERROR] = 'Output %s not available for frames requested.'\
                           'Please check results on ODB.'
raiseMsg[CANCELLING] = 'Cancelling'

INFINITE_ELEMENTS=['ACIN3D3', 'ACIN3D4', 'ACIN3D6', 'ACIN3D8']
TINY=1.0e-10
conversionTable={}
conversionTable['steps'] = int         # Python int conversion
conversionTable['frequencies'] = float # Python float conversion
#
# Various vector operations
#
def scalarMultiply(scalar, vector):
    return [vector[0]*scalar, vector[1]*scalar, vector[2]*scalar]

def normalise(vector):
    length = math.sqrt(vector[0]**2 + vector[1]**2 + vector[2]**2)
    if length > 0:
        vector = [vector[0]/length,vector[1]/length,vector[2]/length]
    else:
        vector = [0,0,0]
    return vector

def vectorDifference(first, second):
    vector = [first[0]-second[0], first[1]-second[1], first[2]-second[2]]
    return vector

def intersect(coord, unitVec, centre, radius):
    a = unitVec[0]**2 + unitVec[1]**2 + unitVec[2]**2
    b = 2.*(unitVec[0]*(coord[0] - centre[0]) + 
            unitVec[1]*(coord[1] - centre[1]) + 
            unitVec[2]*(coord[2] - centre[2]))
    c = ((coord[0] - centre[0])**2 + 
        (coord[1] - centre[1])**2 + 
        (coord[2] - centre[2])**2) - radius*radius

    lam1 = (-b + math.sqrt(b*b-4*a*c))/(2*a)
    lam2 = (-b - math.sqrt(b*b-4*a*c))/(2*a)
    int1 = (coord[0] + lam1*unitVec[0], 
            coord[1] + lam1*unitVec[1], 
            coord[2] + lam1*unitVec[2]) 
    int2 = (coord[0] + lam2*unitVec[0], 
            coord[1] + lam2*unitVec[1], 
            coord[2] + lam2*unitVec[2]) 

    rc = normalise(vectorDifference(int1, coord))
    rc2 = normalise(vectorDifference(int2, coord))
    if Numeric.dot(rc,unitVec) > 0:
        return int1
    #elif Numeric.dot(rc, unitVec) < 0:
    #    return int1
    raise INTERSECTION_ERROR

    
def getOdbs(doName=None):
    if doName == None:
        vp=session.viewports[session.currentViewportName]
        do=vp.displayedObject
        if type(do) is not OdbType:
            raise NO_ODB 
        doName=do.name
        odb = session.odbs[doName]
        odb.close()
    odb=openOdb(doName)
    newOdb = None
    newOdbName='%s_acvis.odb' % doName[:-4]
    try:
        newOdb=session.odbs[newOdbName]
    except KeyError:
        pass
    return odb, newOdb, newOdbName, doName

    
def parseRange(values, conversion='steps'):
    valueNumbers=[]
    error = 0
    if values != '':
        args = values.split(',')
        for arg in args:
            args2 = arg.split(':')
            if len(args2) == 2:
                try:
                    r1 = conversionTable[conversion](args2[0])
                    r2 = conversionTable[conversion](args2[1])
                    if conversion == 'steps':
                        valueNumber=range(r1, r2+1)
                    else:
                        valueNumber=((r1,r2),)
                    for value in valueNumber:
                        valueNumbers.append(value)
                except:
                    error = 1
            elif len(args2) == 1:
                r1 = conversionTable[conversion](args2[0])
                if conversion == 'steps':
                    valueNumbers.append(r1)
                else:
                    valueNumbers.append((r1,r1))
            else:
                error = 1
    return valueNumbers, error

def parseInputs(inputs, numSteps, elementSets, doName):
    if None in inputs:
        print 'Cancelling'
        odb = session.odbs[doName]
        vp=session.viewports[session.currentViewportName]
        vp.setValues(displayedObject=odb)
        return None
    elementSetName=inputs[0]
    error = 0
    if elementSetName != '':
        if elementSetName not in elementSets:
            print 'Element set name %s not present in ODB.'\
                  % elementSetName
            print 'Please re-enter element set name or cancel.'
            print 'Available element sets are:'
            print elementSets
            error = 1
    else:
        elementSetName=' ALL ELEMENTS'
        
    radius = inputs[1]
    try:
        radius = float(radius)
    except ValueError:
        print 'Radius not recognised. Please re-enter radius.'
        error = 1
        
    centrePoint = inputs[2]
    if centrePoint != '':
        args = centrePoint.split(',')
        if len(args) != 3:
            print 'Centre point not recognised. Please re-enter.'
            error=1
        else:
            centrePoint=[]
            for arg in args:
                try:
                    centrePoint.append(float(arg))
                except ValueError:
                    error=1
                    print 'Centre point not recognised. Please re-enter.'
    else:
        centrePoint = (0,0,0)
        
    steps=inputs[3]
    stepNumbers, locError=parseRange(steps)
    if locError == 1:
        print 'Step number(s) not recognised. Please re-enter.'
        error = 1
    for step in stepNumbers:
        if step > numSteps or step < 1:
            error = 1
            print 'Step number not in valid range. Number of steps in current odb '\
                  'is %s.\nPlease re-enter.' % numSteps
    frequencyMinMax = [] 
    frequencies = inputs[4]
    frequencyMinMax, locError=parseRange(frequencies, conversion='frequencies')
    if locError == 1:
        error = 1
        print 'Frequency not recognised. Please re-enter.'
    if frequencyMinMax == []:
        frequencyMinMax = ((0,HUGE),)
    dbRefValue = inputs[5]
    try:
        dbRefValue = float(dbRefValue)
    except ValueError:
        print 'Decibel reference value not recognised. '\
              'Please re-enter value.'
        error = 1
        
    outputSetName = inputs[6]
    if outputSetName == '':
        print 'Output instance name is required.'
        print 'Please re-enter output instance name or cancel.'
        error = 1
    else:
        pat = re.compile(r'(^[\d*_])|(.*_$)|(.*["])')
        if pat.match(outputSetName) != None:
            error = 1
            print 'Names must be 1-38 characters long,'
            print 'may not begin with a number, may not'
            print 'begin or end with a space or an'
            print 'underscrore, and may not contain double'
            print 'quotes, periods, or backward slashes.'
            print 'Please re-enter output set name or cancel.'

    sos = inputs[7]
    try:
        sos = float(sos)
    except ValueError:
        print 'Speed of sound not recognised. Please re-enter speed of sound.'
        error = 1

    rho = inputs[8]
    try:
        rho = float(rho)
    except ValueError:
        print 'Density not recognised. Please re-enter density.'
        error = 1

    inp = inputs[9]
    writeToOrigOdb = False
    try:
        inp = inp.lower()
        if inp[0] == 'y':
            writeToOrigOdb = True
    except (ValueError, IndexError):
        writeToOrigOdb = False
                   
    if error:
        text=inputs
        userInputs = askForInputs(elementSets,numSteps, doName, text)
        try:
            elementSetName, radius, centrePoint, stepNumbers,frequencyMinMax,\
            dbRefValue, outputSetName, sos, rho, writeToOrigOdb = userInputs
        except TypeError:
            raise CANCELLING
    
    return (elementSetName, radius, centrePoint, stepNumbers,
            frequencyMinMax, dbRefValue, outputSetName, sos,
            rho, writeToOrigOdb)

def askForInputs(elementSets, numSteps,doName, text=['','','','','','','','','','']):
    
    fieldTitles = [
        'Infinite elements set name (leave blank for all):',
        'Radius of visualization sphere:',
        'Center point (leave blank for 0,0,0):',
        'Steps (leave blank for all):',
        'Frequencies (leave blank for all)',
        'Decibel reference value:',
        'Created instance name:',
        'Speed of sound:',
        'Density:',
        'Write to original ODB? (default is NO):',
        ]
    fields=[]
    for i in xrange(len(fieldTitles)):
        fields.append((fieldTitles[i], text[i]))
    inputs=getInputs(fields, 'Acoustic Pressure Calculation Tool')
    userInputs=parseInputs(inputs, numSteps, elementSets, doName)
    return userInputs

def createFrames(odb, newOdb):
    for stepName in odb.steps.keys():
        step=odb.steps[stepName]
        frames = step.frames
        newStep=newOdb.steps[step.name]
        numFrames=len(newStep.frames)
        for frame in frames:
            if numFrames != 0:
                if len(newStep.frames) != len(frames):
                    raise FRAMES_MISMATCH
            else:
                newStep.Frame(incrementNumber=frame.incrementNumber,
                              frameValue=frame.frameValue,
                              description=frame.description)

def createSteps(odb, newOdb):
    """The new odb will contain the same number of steps as the original.
       Only some of these steps may be Steady state dynamics steps.
       Only some of these steps may have been selected to create output.
    """
    for stepName in odb.steps.keys():
        step=odb.steps[stepName]
        if step.previousStepName == 'Initial':
            newOdb.Step(name=step.name,
                        description=step.description,
                        domain=step.domain,
                        procedure=step.procedure,
                        timePeriod=step.timePeriod,
                        totalTime=step.totalTime)
        else:
            newOdb.Step(name=step.name,
                        description=step.description,
                        domain=step.domain,
                        previousStepName=step.previousStepName,
                        procedure=step.procedure,
                        timePeriod=step.timePeriod,
                        totalTime=step.totalTime)
    newOdb.save()
    createFrames(odb, newOdb)
    newOdb.save()
        

        
        
def checkSteps(odb, newOdb):
    """The new odb will contain the same number of steps as the original.
       If this odb exists check that it is consistent with previous odb.
    """
    steps=odb.steps.keys()
    newSteps=odb.steps.keys()
    if len(newSteps) != len(steps):
        raise STEPS_MISMATCH
    for s in xrange(len(steps)):
        if steps[s] != newSteps[s]:
            raise STEPS_MISMATCH
    #If odb created from a datacheck no frames will exist.
    #Add frames if not already present.
    createFrames(odb, newOdb)
    
def checkInstanceNames(odb, newOdb, newName):
    """If the visualization odb already exists
       will have existing instances with results.
       Check there will not be a name clash.
    """
    for iname in odb.rootAssembly.instances.keys():
        newInstanceName='%s-%s' % (iname,newName)
        if newOdb.rootAssembly.instances.has_key(newInstanceName):
            raise NAME_ERROR, newInstanceName
    steps=odb.steps.keys()
    newSteps=odb.steps.keys()
    if len(newSteps) != len(steps):
        raise STEPS_MISMATCH
    for s in xrange(len(steps)):
        if steps[s] != newSteps[s]:
            raise STEPS_MISMATCH
    createFrames(odb, newOdb)
    
def createInfiniteElements(odb, newOdb, newInstances, elementSetName):
    #
    #  When writing to new odb do not want to create infinite elements
    #  for viewing results. The next time new output is requested, unless
    #  user specifies a set name, all infinite elements will be used.
    #  The visualization elements do not the correct output.
    #  Visualization elements will be membrane elements.
    #
    elTypes={}
    elTypes['ACIN3D3'] = 'M3D3'
    elTypes['ACIN3D4'] = 'M3D4'
    elTypes['ACIN3D6'] = 'M3D6'
    elTypes['ACIN3D8'] = 'M3D8'
    
    newElements={}
    for instanceName in newInstances.keys():
        newElements[instanceName] = {}
        
        for infType in INFINITE_ELEMENTS:
            newElements[instanceName][infType] = []
        elementLabels=newInstances[instanceName]['infEls'].keys()
        instance=odb.rootAssembly.instances[instanceName]
        nodeLabels={}
        for label in elementLabels:
            element=instance.getElementFromLabel(label)
            elType=element.type
            newElements[instanceName][elType].append((label,
                                           element.connectivity))
            conn=element.connectivity
            for node in conn:
                nodeLabels[node] = ''
        labels=[]
        coords=[]
        for nodeLabel in nodeLabels.keys():
            node=instance.getNodeFromLabel(nodeLabel)
            labels.append(nodeLabel)
            coords.append(node.coordinates)
        inIn=0
    for name in newElements.keys():
        totEl=0
        for key in newElements[name].keys():
            totEl = totEl + len(newElements[name][key])
        if totEl == 0:
            del newElements[name]

    for instanceName in newElements.keys():
        newElements[instanceName] = {}
        
        if newOdb.parts.has_key(instanceName):
            newPart=newOdb.parts[instanceName]
        else:
            newPart = newOdb.Part(name=instanceName, 
                                  embeddedSpace=THREE_D,
                                  type=DEFORMABLE_BODY)
        for infType in INFINITE_ELEMENTS:
            newElements[instanceName][infType] = []
        elementLabels=newInstances[instanceName]['infEls'].keys()
        instance=odb.rootAssembly.instances[instanceName]
        nodeLabels={}
        for label in elementLabels:
            element=instance.getElementFromLabel(label)
            elType=element.type
            newElements[instanceName][elType].append((label,
                                           element.connectivity))
            conn=element.connectivity
            for node in conn:
                nodeLabels[node] = ''
        labels=[]
        coords=[]
        for nodeLabel in nodeLabels.keys():
            node=instance.getNodeFromLabel(nodeLabel)
            labels.append(nodeLabel)
            coords.append(node.coordinates)
        inIn=0
        if newOdb.rootAssembly.instances.has_key(instanceName):
            i = newOdb.rootAssembly.instances[instanceName]
            if i.nodeSets.has_key(elementSetName):
                inIn = 1
        if newPart.nodeSets.has_key(elementSetName) or inIn:
            pass
        else:
            newPart.addNodes(labels=labels, coordinates=coords,
                             nodeSetName=elementSetName)
        inIn=False
        if newOdb.rootAssembly.instances.has_key(instanceName):
            i = newOdb.rootAssembly.instances[instanceName]
            if i.elementSets.has_key(elementSetName):
                inIn = True
        if not inIn:
            for elType in newElements[instanceName].keys():
                labels=[]
                conn=[]
                for lc in newElements[instanceName][elType]:
                    labels.append(lc[0])
                    conn.append(lc[1])
                if len(labels) > 0:
                    newPart.addElements(labels=labels,
                                        connectivity=conn,
                                        type=elTypes[elType],
                                        elementSetName=elementSetName)

        if not newOdb.rootAssembly.instances.has_key(instanceName):
            newOdb.rootAssembly.Instance(name=instanceName,
                                         object=newPart)
            
    for name in newElements.keys():
        totEl=0
        for key in newElements[name].keys():
            totEl = totEl + len(newElements[name][key])
        if totEl == 0:
            del newElements[name]
    return newElements
    
def checkOdb():
    odb, newOdb, newOdbName, doName = getOdbs()
    version=odb.jobData.version
    if version[0:15] != 'Abaqus/Standard':
        raise WRONG_ANALYSIS_CODE, version
      
    # Newer versions (2016+) are not in the form major.minor-upd (e.g. 6.14-1)
    if re.search('\d\.\d+-\d+', version):
        prePos = version.find('PRE')
        start = version.find(' ')+1
        end = version.find('-')
        majorMinorStr = version[start:end]
        major = int(majorMinorStr.split('.')[0])
        minor = int(majorMinorStr.split('.')[1])

        if major < 6:
            raise WRONG_ODB_VERSION, version
        elif major == 6 and minor < 4:
            raise WRONG_ODB_VERSION, version
        elif major == 6 and minor == 4 and prePos > -1:
            preVersion = int(version[prePos+3:])
            if preVersion < 6: # 6.4 PRE6
                raise WRONG_ODB_VERSION, version
                    
    stepKeys = odb.steps.keys()
    if len(stepKeys) < 1:
        print 'No available steps in odb.'
        raise NO_SSD_STEPS
    ssd=False
    for key in stepKeys:
        step = odb.steps[key]
        procedure = step.procedure
        domain = step.domain
        if procedure == '*STEADY STATE DYNAMICS' and domain is FREQUENCY:
            ssd=True
            break

    if not ssd:
        raise NO_SSD_STEPS

    return doName

def getUserInput(doName):
    """Pops up getInputs dialog for user input. Requires list of existing
    element sets for comparing with user input.
    """
    odb, newOdb, newOdbName, doName = getOdbs(doName)
    elementSets=[]
    elementSets=odb.rootAssembly.elementSets.keys()
    for instanceName in odb.rootAssembly.instances.keys():
        instance=odb.rootAssembly.instances[instanceName]
        elementSetNames=instance.elementSets.keys()
        for name in elementSetNames:
            if name not in elementSets:
                elementSets.append(name)
                elementSets.append('%s.%s' % (instance.name, name))
    numSteps = len(odb.steps.keys())
    userInputs=askForInputs(elementSets, numSteps, doName)
    return userInputs

def checkForInfiniteElements(userInputs, doName):
    odb, newOdb, newOdbName, doName = getOdbs(doName)
    instanceNames = odb.rootAssembly.instances.keys()
    newInstances={}
    for instance in instanceNames:
        newInstances[instance] = {'infEls':{},}
    elementSetName=userInputs[0]
    elementSets=odb.rootAssembly.elementSets.keys()
    instanceNames = odb.rootAssembly.instances.keys()
    if elementSetName in elementSets:
        set=odb.rootAssembly.elementSets[elementSetName]
        elements=set.elements
        instances=set.instances
        for i in xrange(len(elements)):
            element = elements[i]
            for el in element:
                if el.type in INFINITE_ELEMENTS:
                    newInstances[instances[i].name]['infEls'][el.label] = ''                    
    for instanceName in odb.rootAssembly.instances.keys():
        instance=odb.rootAssembly.instances[instanceName]
        elementSets=instance.elementSets.keys()
        newSets=[]
        for set in elementSets:
            newSets.append('%s.%s' % (instanceName, set))
        args = elementSetName.split('.')
        if len(args) == 2:
            iName, setName = args[0], args[1]
        else:
            iName = None
            setName = args[0]
        if (iName == instanceName) and (setName in elementSets) and (elementSetName in newSets):
            set = instance.elementSets[setName]
            elements=set.elements
            for element in elements:
                    if element.type in INFINITE_ELEMENTS:
                        newInstances[instanceName]['infEls'][element.label] = ''
            
    anyInfEls=0
    for key in newInstances.keys():
        if len(newInstances[key]['infEls']) > 0:
            anyInfEls = 1
    if anyInfEls == 0:
        raise NO_INF_ELS, elementSetName

    return newInstances

def addOutputs(doName, rePDisp, imPDisp, iNames, nodeNums, stepName, fId,
               newInstances, newName, name='NEW',
               description='Pressure',ALL=True,
               writeToOrigOdb=False):
    """For each variable passed in, add to frame of step.
    Variables are passed in as a long array. Need to order
    by instance.
    """
    odb, newOdb, newOdbName, doName = getOdbs(doName)
    if writeToOrigOdb:
        newOdbName = odb.name
        newOdb = openOdb(newOdbName)
        
    step=newOdb.steps[stepName]
    frame=step.frames[fId]
    try:
        fo1=frame.FieldOutput(name=name,description=description,
                              type=SCALAR)
    except OdbError:
        fo1=frame.fieldOutputs[name]
        
    for key in newElements.keys():
        newInstanceName = '%s-%s' % (key, newName)
        instance=newOdb.rootAssembly.instances[newInstanceName]
        
        fieldData1=[]
        fieldData2=[]
        labels=[]
        for j in xrange(len(nodeNums)):
            if iNames[j] == key:
                fieldData1.append((rePDisp[j],))
                if imPDisp is not None:
                    fieldData2.append((imPDisp[j],))
                labels.append(nodeNums[j])
        if ALL:
            if len(fieldData2) > 0:
                fo1.addData(instance=instance, position=NODAL,
                            labels=labels, data=fieldData1, conjugateData=fieldData2)
            else:
                fo1.addData(instance=instance, position=NODAL,
                            labels=labels, data=fieldData1)
        else:
            for l in xrange(len(labels)):
                label = (labels[l],)
                fd1 = (fieldData1[l],)
                fd2 = (fieldData2[l],)
                fo1.addData(instance=instance, position=NODAL,
                            labels=label, data=fd1, conjugateData=fd2)

def createVisualizationOutput(doName, userInputs, newInstances, newElements,
                              stepFrames, phaseDist, subSet, dimension=3):
    """For each step and each frame requested, create nodal output POR,
       PFar, PORdB, PFARdB
       Require to extract the nodal information from the analysis odb
       and manipulate it for each step/frame to create required output.

       stepFrames is a dictionary containing the steps and frames where
       output is required.

       To create new fieldoutput need instance name, position,
       labels and data.

       For models with multiple parts and instances create the field data
       on an instance by instance basis.
    """

    odb, newOdb, newOdbName, doName = getOdbs(doName)
    elementSetName = userInputs[0]
    radius = userInputs[1]
    centrePoint = userInputs[2]
    realCentre = (0,0,0) #Origin of infinite vectors
    steps = userInputs[3]
    frequencyMinMax = userInputs[4]
    dbRefValue = userInputs[5]
    newName = userInputs[6]
    sos = userInputs[7]
    rho = userInputs[8]
    writeToOrigOdb = userInputs[9]
    if writeToOrigOdb:
        newOdbName = odb.name
        newOdb = openOdb(newOdbName)
    
    reqOutputs=['PINF0','PINF1','PINF2','PINF3','PINF4','PINF5','PINF6',
                'PINF7','PINF8','PINF9']

    reqOutputs=['POR',]

    reqOutputs={}
    reqOutputs['PINFS'] = ['PINF0','PINF1','PINF2','PINF3','PINF4','PINF5',
                           'PINF6','PINF7','PINF8','PINF9']
      
    for step in stepFrames.keys():
        frames=odb.steps[step].frames
        for fId in stepFrames[step]:
            pinfs = [] 
            frame = frames[fId]
            fo=frame.fieldOutputs.keys()
            for reqOutput in reqOutputs['PINFS']:
                if reqOutput not in fo:
                    raise OUTPUT_ERROR, reqOutput
                subField=frame.fieldOutputs[reqOutput].getSubset(region=subSet)
                pinfs.append(subField.values)
            num = len(pinfs[0])
            subField=frame.fieldOutputs['INFR'].getSubset(position=NODAL,
                                                          region=subSet)
            rinfs = subField.values

            pinfRArray = Numeric.zeros((num, 10), Numeric.float)
            pinfIArray = Numeric.zeros((num, 10), Numeric.float)
            infrArray = Numeric.zeros((num,), Numeric.float)
            pdArray = Numeric.zeros((num,), Numeric.float)
            iNames = []
            nodeNums = []
            bArray = Numeric.ones((10, num), Numeric.float)
            for v in xrange(num):
                for i in xrange(10):
                    realP = pinfs[i][v].data
                    imagP = pinfs[i][v].conjugateData
                    pinfRArray[v][i] = realP
                    pinfIArray[v][i] = imagP

                infrArray[v] = rinfs[v].data
                try:
                    pdArray[v] = \
                    phaseDist[pinfs[i][v].instance.name][pinfs[i][v].nodeLabel]
                except KeyError:
                    pdArray[v] = 0.0
                iNames.append(pinfs[i][v].instance.name)
                nodeNums.append(pinfs[i][v].nodeLabel)
            
            ec = (pdArray - infrArray)/(pdArray + infrArray)
            xi = ec - 1
            fac1 = 1 + ec
            for m in xrange(2,11):
                tmp = math.sqrt((2*m-1)/4.)*fac1
                sum = 0.0
                for k in xrange(1,10):
                    sum = sum + COEFF[m-2][k-1]*xi**(k-1)
                bArray[m-1] = tmp*sum
            bArray = Numeric.transpose(bArray)

            rSum = Numeric.zeros((num,),Numeric.float)
            iSum = Numeric.zeros((num,),Numeric.float)
            farRSum = Numeric.zeros((num,),Numeric.float)
            farISum = Numeric.zeros((num,),Numeric.float)

            for m in xrange(num):
                rSum[m] = Numeric.dot(bArray[m], pinfRArray[m])
                iSum[m] = Numeric.dot(bArray[m], pinfIArray[m])
                farRSum[m] = Numeric.dot(pinfRArray[m], CM)
                farISum[m] = Numeric.dot(pinfIArray[m], CM)
            rePray = -0.5*xi*rSum
            imPray = -0.5*xi*iSum
            frequency = frame.frameValue
            omega = 2.*math.pi*frequency
            k=omega/sos
            kinfrArray=k*infrArray
            const1=((kinfrArray)**(0.5*(dimension-1)))
            theta=k*pdArray
            cosFar = Numeric.cos(kinfrArray)
            sinFar = Numeric.sin(kinfrArray)
            rPfar = const1*(cosFar*farRSum - sinFar*farISum)
            iPfar = const1*(cosFar*farISum + sinFar*farRSum)
            rePDisp = Numeric.cos(theta)*rePray + Numeric.sin(theta)*imPray
            imPDisp = Numeric.cos(theta)*imPray - Numeric.sin(theta)*rePray

            # Magnitude of POR at a node
            #
            magPDisp = Numeric.sqrt(rePDisp*rePDisp + imPDisp*imPDisp)
            rmsPDisp = magPDisp/sqrt(2)
            
            ok = 1
            if dbRefValue == 0:
                ok = 0
            for i in range(len(rmsPDisp)):
                if rmsPDisp[i] == 0:
                    ok = 0
            if ok:
                try:
                    PORdB = 20.*Numeric.log10(rmsPDisp/(dbRefValue))
                except OverflowError:
                    PORdB = 0.*rmsPDisp
            else:
                PORdB = 0.*rmsPDisp

            # Magnitude of POR at a far-field node
            #
            magPFar = Numeric.sqrt(rPfar*rPfar + iPfar*iPfar)
            rmsPFar = magPFar/sqrt(2)

            ok = 1
            if dbRefValue == 0:
                ok = 0
            for i in range(len(rmsPFar)):
                if rmsPFar[i] == 0:
                    ok = 0
            if ok:
                try:
                    PFARdB = 20.*Numeric.log10(rmsPFar/(dbRefValue))
                except OverflowError:
                    PFARdB = 0*rmsPFar
            else:
                PFARdB = 0*rmsPFar

            # Create Far Field acoustic intensity field variable
            #
            PFarInten = magPFar**2/(2*rho*sos) 

            # Create field outputs
            #
            name='POR'
            description = 'Acoustic pressure'
            addOutputs(doName, rePDisp, imPDisp, iNames, nodeNums, step,
                       fId, newInstances, newName, name, description, True,
                       writeToOrigOdb)

            name='PFAR'
            description = 'Far-field Acoustic pressure'
            addOutputs(doName, rPfar, iPfar, iNames, nodeNums, step, fId,
                       newInstances, newName, name, description, True,
                       writeToOrigOdb)
            name='PORdB'
            description = 'Acoustic pressure in dB relative to the dbRefValue'
            addOutputs(doName, PORdB, None, iNames, nodeNums, step, fId,
                       newInstances, newName, name, description, True,
                       writeToOrigOdb)
            name='PFARdB'
            description = 'Far-field Acoustic pressure in dB relative to the dbRefValue'
            addOutputs(doName,PFARdB, None, iNames, nodeNums, step, fId,
                       newInstances, newName, name, description, True,
                       writeToOrigOdb)

            name='INTEN_FAR'
            description = 'Far-field Acoustic intensity'
            addOutputs(doName,PFarInten, None, iNames, nodeNums, step, fId,
                       newInstances, newName, name, description, True,
                       writeToOrigOdb)

    newOdb.save()
    newOdbName = newOdb.name
    newOdb.close()
    openOdb(newOdbName)
    
def createVisualizationElements(userInputs, newInstances, newElements, doName):
    odb, newOdb, newOdbName, doName = getOdbs(doName)
    #
    #  When writing to new odb do not want to create infinite elements
    #  for viewing results. The next time new output is requested, unless
    #  user specifies a set name, all infinite elements will be used.
    #  The viaulaization elements do not the correct output.
    #  Visualization elements will be membrane elements.
    #
    elTypes={}
    elTypes['ACIN3D3'] = 'M3D3'
    elTypes['ACIN3D4'] = 'M3D4'
    elTypes['ACIN3D6'] = 'M3D6'
    elTypes['ACIN3D8'] = 'M3D8'
    
    elementSetName=userInputs[0]
    radius = userInputs[1]
    centrePoint = userInputs[2]
    realCentre = (0,0,0)
    steps = userInputs[3]
    frequencyMinMax = userInputs[4]
    newName = userInputs[6]
    sos = userInputs[7]
    rho = userInputs[8]
    writeToOrigOdb = userInputs[9]
    if writeToOrigOdb:
        newOdbName = odb.name
        newOdb = openOdb(newOdbName)
                   
    values = None
    infVecs={}
    for name in newInstances.keys():
        infVecs[name] = {}
    for key in odb.steps.keys():
        step = odb.steps[key]
        frames = step.frames
        for frame in frames:
            if frame.fieldOutputs.has_key('INFN'):
                infn=frame.fieldOutputs['INFN']
                values = infn.values
                break
    
    for value in values:
        infVecs[value.instance.name][value.nodeLabel] = value.data
        length=math.sqrt(value.data[0]**2 +
                         value.data[1]**2 +
                         value.data[2]**2)
    phaseDist={}
    nodeSeq=[]
    for name in newElements.keys():
        usedNodes={}
        phaseDist[name] = {}
        newPartName = '%s-%s' % (name, newName)
        
        newPart = newOdb.Part(name=newPartName,
                              embeddedSpace=THREE_D,
                              type=DEFORMABLE_BODY)
        
        instanceData = newInstances[name]
        elements=instanceData['infEls']
        instance=odb.rootAssembly.instances[name]
        for label in elements.keys():
            element=instance.getElementFromLabel(label)
            for nodeLabel in element.connectivity:
                if not usedNodes.has_key(nodeLabel):
                    node=instance.getNodeFromLabel(nodeLabel)
                    usedNodes[nodeLabel] = node.coordinates
                    unitVec = infVecs[name][nodeLabel]
                    int1=intersect(node.coordinates, unitVec,
                                   centrePoint, radius)
                    if int1 != None:
                        pd = vectorDifference(int1, node.coordinates)
                        length = math.sqrt(pd[0]**2 + pd[1]**2 + pd[2]**2)
                        phaseDist[name][nodeLabel] = length
                        usedNodes[nodeLabel] = int1
        sortLabels = usedNodes.keys()
        sortLabels.sort()
        sortCoords = []
        for lab in sortLabels:
            sortCoords.append(usedNodes[lab])
        newPart.addNodes(labels=sortLabels, coordinates=sortCoords)
        perInst=[]
        perInst.append(name)
        nodeKeys=usedNodes.keys()
        nodeKeys.sort()
        perInst.append(nodeKeys)
        for key in newElements[name].keys():
            labels=[]
            conn=[]
            for lc in newElements[name][key]:
                labels.append(lc[0])
                conn.append(lc[1])
            if len(labels) > 0:
                newPart.addElements(labels=labels,
                                    connectivity=conn,type=elTypes[key])
        newOdb.rootAssembly.Instance(name=newPartName, object=newPart)
    newOdb.save()
    perInst = (perInst,)
    subsetName='subset'
    counter = 1
    while subsetName in odb.rootAssembly.nodeSets.keys():
        subsetName = 'subsetName%s' % counter
        counter = counter + 1
    subSet=odb.rootAssembly.NodeSetFromNodeLabels(name=subsetName,
                                                  nodeLabels=perInst)
    s=subSet.nodes[0]
    return phaseDist, subSet
    
def createVisualizationODB(userInputs, newInstances, doName):
    odb, newOdb, newOdbName, doName = getOdbs(doName)
    elementSetName=userInputs[0]
    radius = userInputs[1]
    centrePoint = userInputs[2]
    steps = userInputs[3]
    frequencyMinMax = userInputs[4]
    newName = userInputs[6]
    sos = userInputs[7]
    rho = userInputs[8]
    writeToOrigOdb = userInputs[9]
    if writeToOrigOdb:
        newOdbName = odb.name
        newOdb = openOdb(newOdbName)
    stepKeys=odb.steps.keys()
    if len(steps) == 0:
        steps=range(1,len(stepKeys)+1)
    reqStepKeys=[]
    for i in steps:
        try:
            reqStepKeys.append(stepKeys[i-1])
        except IndexError:
            pass
    stepFrames={}
    for key in reqStepKeys:
        step=odb.steps[key]
        stepFrames[key]=[]
        procedure = step.procedure
        domain = step.domain
        if procedure == '*STEADY STATE DYNAMICS' and domain is FREQUENCY:
            count = 0
            for frame in step.frames:
                freq = frame.frequency
                for minMax in frequencyMinMax:
                    if frame.frequency >= minMax[0] and \
                       frame.frequency <= minMax[1]:
                        stepFrames[key].append(count)
                count = count + 1
    if os.path.isfile(newOdbName):
        if odb.name != newOdbName:
            newOdb = openOdb(path=newOdbName)
            if newOdb.isReadOnly:
                newOdb.close()
                newOdb = openOdb(path=newOdbName)
            checkSteps(odb, newOdb)
            checkInstanceNames(odb, newOdb, newName)
        newElements=createInfiniteElements(odb, newOdb,
                                           newInstances, elementSetName)
    else:
        newOdb = Odb(name=newOdbName,
                  analysisTitle='derived data',
                  description='test problem',
                  path=newOdbName)
        createSteps(odb, newOdb)
        newElements=createInfiniteElements(odb, newOdb,
                                           newInstances, elementSetName)
    newOdb.save()
    return newElements, stepFrames

def displayVisualizationOdb(doName, userInputs):
    odb, newOdb, newOdbName, doName = getOdbs(doName)
    writeToOrigOdb = userInputs[9]
    if writeToOrigOdb:
        newOdbName = odb.name
        newOdb = openOdb(newOdbName)

    odb = session.odbs[newOdbName]
    vp=session.viewports[session.currentViewportName]
    vp.setValues(displayedObject=odb)
    return newOdbName

try:
    vp=session.viewports[session.currentViewportName]
    do=vp.displayedObject
    if type(do) is not OdbType:
        doName = None
    else:
        doName=do.name
    doName = checkOdb()
    userInputs=getUserInput(doName)
    if userInputs != None:
        newInstances=checkForInfiniteElements(userInputs, doName)
        newElements, stepFrames=createVisualizationODB(userInputs,
                                                       newInstances, doName)
        phaseDist, subSet = createVisualizationElements(userInputs,
                                                        newInstances,
                                                        newElements, doName)
        createVisualizationOutput(doName, userInputs, newInstances, newElements,
                                  stepFrames, phaseDist, subSet)

        newOdbName = displayVisualizationOdb(doName, userInputs)
        print '\nThe script has finished successfully. Display elements have\n'\
              'been added to instance %s to the odb %s.\n'\
              'This odb is currently being displayed.' % (userInputs[6], newOdbName)



except AcousticVisException, msg:
    print '\a'
    excType = sys.exc_info()[0]
    if msg != None:
        print raiseMsg[excType] %  msg
    else:
        print raiseMsg[excType]
    if doName:
        odb = session.odbs[doName]
        vp=session.viewports[session.currentViewportName]
        vp.setValues(displayedObject=odb)
        

except:
    print '\a'
    print 'Unhandled exception.'
    print 'Please check the ODB for acoustic infinite element output. PINF, INFR, INFN outputs must be requested.'
    print 'If there is nothing obviously wrong with the ODB please'
    print 'contact SIMULIA Inc. for help.'
    print 'Diagnostic information available:'
    print '***    ',excTyValStr()
