from abaqusGui import *
from fastenerConst import *
import fastenerDB, os

###########################################################################
# Class definition
###########################################################################

class FastenerForm(AFXForm):
    [
        ID_OVERWRITE,
        ID_OK,
        ID_LAST
    ] = range(AFXForm.ID_LAST, AFXForm.ID_LAST+3)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):

        # Construct the base class.
        #
        AFXForm.__init__(self, owner)

        #self.db = None

        # Create the Map function to handle the overwrite request on the
        # output file.
        #
        FXMAPFUNC(self, SEL_COMMAND, self.ID_OVERWRITE, FastenerForm.onCmdOverwrite)
        
        # Create the command and keywords.
        #
        self.cmd = AFXGuiCommand(self, 'createFasteners', 'cWELD.fastenerModule')

        # File I/O related Keywords
        #
        self.weldFileNameKw = AFXStringKeyword(self.cmd, 'weldFileName', TRUE, DEFAULT_WELD_FILENAME)
        self.delimiterKw = AFXSymConstKeyword(self.cmd, 'delimiter', FALSE, DEFAULT_DELIMITER)
        self.outputFileNameKw = AFXStringKeyword(self.cmd, 'outputFileName', TRUE, DEFAULT_OUT_FILENAME)
        self.bulkDataFileKw = AFXStringKeyword(self.cmd, 'bulkDataFileName', FALSE, DEFAULT_BULKDATA_FILE)

        # Kernal data keywords
        #
        self.nodeOffsetKw = AFXIntKeyword(self.cmd, 'nodeOffset', TRUE, DEFAULT_NODE_OFFSET)
        self.radiusInfluenceKw = AFXFloatKeyword(self.cmd, 'radiusInfluence', FALSE, DEFAULT_RAD_INF)
        self.sheetThicknessKw = AFXFloatKeyword(self.cmd, 'sheetThickness', FALSE, DEFAULT_THICKNESS)
        self.elasticModulusKw = AFXFloatKeyword(self.cmd, 'elasticModulus', FALSE, DEFAULT_MODULUS)
        self.poissonRatioKw = AFXFloatKeyword(self.cmd, 'poissonRatio', FALSE, DEFAULT_POISSON)

        # Type keywords
        #
        self.compliantFastenerKw = AFXBoolKeyword(self.cmd, 'compliantFastener', AFXBoolKeyword.TRUE_FALSE, TRUE, DEFAULT_USE_COMPLIANT)
        self.radiusTypeKw = AFXSymConstKeyword(self.cmd, 'radiusType', TRUE, DEFAULT_RADIUS_TYPE.getId())
        self.thicknessTypeKw = AFXSymConstKeyword(self.cmd, 'thicknessType', FALSE, DEFAULT_THICKNESS_TYPE.getId())
        self.materialTypeKw = AFXSymConstKeyword(self.cmd, 'materialType', FALSE, DEFAULT_MATERIAL_TYPE.getId())
        self.weldTableFormatKw = AFXStringKeyword(self.cmd, 'weldTableFormat', FALSE, DEFAULT_WELD_TABLE_IDS)
       
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomTasks(self):

        # Report back to the user that output was successful
        mw = getAFXApp().getAFXMainWindow()
        mw.writeToMessageArea('The output file has been written to %s' % \
                                 self.outputFileNameKw.getValue())
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        #if not self.db:
         #   self.db = fastenerDB.FastenerDB(self)   
        return fastenerDB.FastenerDB(self) #self.db

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        # Method to check if the odbName is valid
        # 
        currentDb = self.getCurrentDialog()
        
        if self.weldFileNameKw.getValue().strip() == '':
            showAFXErrorDialog(currentDb,'Weld master file name must be specified.')
            currentDb.weldText.setFocusToTextField()
            return FALSE
        if self.bulkDataFileKw.getValue().strip() == '' and currentDb.bulkText.isEnabled():
            showAFXErrorDialog(currentDb,'BDF file name must be specified.')
            currentDb.bulkText.setFocusToTextField()
            return FALSE
        if self.outputFileNameKw.getValue().strip() == '':
            showAFXErrorDialog(currentDb,'Output file name must be specified.')
            currentDb.outputText.setFocusToTextField()
            return FALSE
        else:
            # Check to see if the output file has an extension associated with it
            #
            filename = self.outputFileNameKw.getValue()
            path, file = os.path.split(filename)
            job, extn = os.path.splitext(file)
            if extn == '' or extn == None:
                showAFXErrorDialog(currentDb,'Output file must have extension .inp.')
                currentDb.outputText.setFocusToTextField()
                return FALSE

        return TRUE

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):
 
        # Check to see if we need to prompt for overwrite the output file.
        #
        import os        
        
        # Update the required keywords.
        #
        if self.compliantFastenerKw.getValue() == TRUE:
            # Need thickness and material types to be REQUIRED
            #
            self.thicknessTypeKw.setRequired(TRUE)
            self.materialTypeKw.setRequired(TRUE)

            # Set thickness related keywrods to be true
            # if required to be sent to kernal.
            #
            if self.thicknessTypeKw.getValue()==DATA_FROM_FILE.getId():
                self.bulkDataFileKw.setRequired(TRUE)
                self.sheetThicknessKw.setRequired(FALSE)
            else:     
                self.sheetThicknessKw.setRequired(TRUE)

            # Set material related keywords to be required
            # 
            if self.materialTypeKw.getValue()==DATA_FROM_FILE.getId():
                self.bulkDataFileKw.setRequired(TRUE)
                self.poissonRatioKw.setRequired(FALSE)
                self.elasticModulusKw.setRequired(FALSE)
            else:
                self.poissonRatioKw.setRequired(TRUE)
                self.elasticModulusKw.setRequired(TRUE)
        else:
            self.materialTypeKw.setRequired(FALSE)
            self.thicknessTypeKw.setRequired(FALSE)
            self.sheetThicknessKw.setRequired(FALSE)
            self.poissonRatioKw.setRequired(FALSE)
            self.elasticModulusKw.setRequired(FALSE)
            
        # Handle radius of influence
        #
        if self.radiusTypeKw.getValue() == DATA_FROM_FILE.getId():
            self.bulkDataFileKw.setRequired(TRUE)
            self.radiusInfluenceKw.setRequired(FALSE)
        else:
            self.radiusInfluenceKw.setRequired(TRUE)

        # check to see if the output file already exists
        #
        if os.path.exists(self.outputFileNameKw.getValue()):
            db = self.getCurrentDialog()
            showAFXWarningDialog(db, 'Output file already exists.\n\nOK to overwrite?',
                AFXDialog.YES|AFXDialog.NO, self, self.ID_OVERWRITE)
            return 0        
        return 1    
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdOverwrite(self, sender, sel, ptr):
 
        # If user allowed overwrite, then continue, otherwise just return.
        #
        if sender.getPressedButtonId() == AFXDialog.ID_CLICKED_YES:
            AFXForm.issueCommands(self)
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def handleException(self, exception):
    
        exceptionType = exception[0]
        exceptionValue = exception[1]

        # Post an error dialog
        #
        db = self.getCurrentDialog()
        showAFXErrorDialog(db, str(exceptionValue))
