"""
ABAQUS/CAE Plug-in for plotting the connectors of an ODB satisfying a given
set of arguments. The arguments include step name, step time, connector
damage variable,an operator (>=, <=, ==, between) and a numerical value.
The connectors satisying the arguments will be displayed in the viewport.
Connectors can also be displayed using a pre-set color code definition
showing the various states of damage.
"""

from abaqusGui import *
from connVisConstants import *
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Class definition
#
class PlotConnectorsDB(AFXDataDialog):
    [
        ID_RESET,
        ID_STEP,
        ID_LAST
    ] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST+3)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):
       
        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, 'Plot Connectors', 
                                opts=DECOR_RESIZE|DIALOG_ACTIONS_SEPARATOR)
        getAFXAliasMap().setPrefix(self, 'Plot Connectors')
        
        # Add action buttons
        #
        self.appendActionButton('Plot\tPlot connectors in viewport.', self,
                                self.ID_CLICKED_APPLY)
        self.appendActionButton('Reset\tRefresh the viewport.', self,
                                self.ID_RESET)
        FXMAPFUNC(self,SEL_COMMAND,self.ID_RESET,PlotConnectorsDB.onCmdReset)

        self.appendActionButton(self.CANCEL) 
        
        self.currentOdbName = None
        self.previousOdbName = None 
        self.form = form
        
        # Add ODB Text Field
        #
        self.odbTF = AFXTextField(self, 10, 'ODB name:', form.odbTgt,
                                  0, LAYOUT_FILL_X)
        self.odbTF.setReadOnlyState(TRUE)  

        hf1 = FXHorizontalFrame(self,LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        self.stepNameCB = AFXComboBox(hf1, 0, 4, 'Step name:',
                                      form.stepNameKw, 0)
        self.stepNameCB.setMaxVisible(10)
        form.stepNameKw.setTarget(self)
        form.stepNameKw.setSelector(self.ID_STEP)
        self.stepTimeCB = AFXComboBox(hf1, 0, 4, 'Step time:',
                                      form.stepTimeKw, 0)
        self.stepTimeCB.setMaxVisible(10)
        
        gb1 = FXGroupBox(self, 'Plot connectors based on',
                        FRAME_GROOVE|LAYOUT_FILL_X)        
            
        self.radio1 = FXRadioButton(gb1, 'Query',
                                    form.selectionKw, FIRST_BUTTON.getId())

        hf2 = FXHorizontalFrame(gb1,LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        self.damageCB = AFXComboBox(hf2, 0, 10, 'Connector damage variable:',
                                    form.damageVariableKw, 0)
        self.damageCB.setMaxVisible(10)

        hf3 = FXHorizontalFrame(gb1,LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        self.conditionCB = AFXComboBox(hf3, 0, 4, 'Plot connectors:',
                                       form.damageFlagKw, 0)

        self.tf1 = AFXTextField(hf3, 10,'' , form.firstValueKw, 0,
                                AFXTEXTFIELD_FLOAT)
        self.tf2 = AFXTextField(hf3, 10,'and' , form.secondValueKw, 0,
                                AFXTEXTFIELD_FLOAT)
        
        self.addTransition(self.form.damageFlagKw, AFXTransition.EQ,
                           WITHIN_RANGE.getId(), self.tf2,
                           MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND), None)
        self.addTransition(self.form.damageFlagKw, AFXTransition.NE,
                           WITHIN_RANGE.getId(), self.tf2,
                           MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND), None)

        self.radio2 = FXRadioButton(gb1, 'Color Code', form.selectionKw,
                                    SECOND_BUTTON.getId())
        hf4 = FXHorizontalFrame(gb1,LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        self.damageCB1 = AFXComboBox(hf4, 0, 10, 'Damage evolution variable:',
                                    form.damEvolVarKw, 0)
        self.damageCB1.setMaxVisible(10)
        hf5 = FXHorizontalFrame(gb1,LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        self.damageCB2 = AFXComboBox(hf5, 0, 10, 'Damage initiation variable:',
                                    form.damInitVarKw, 0)
        self.damageCB2.setMaxVisible(10)
        
        self.switcher = FXSwitcher(self)
        self.gb2 = FXGroupBox(self.switcher, 'Color Code Explanations',
                              FRAME_GROOVE|LAYOUT_FILL_X)        
        vf1 = FXVerticalFrame(self.gb2)
        self.ccf1 = FXLabel(vf1,"Red: Elements with 'cdmg<n>' >= 1")        
        self.ccf2 = FXLabel(vf1,"Orange: Elements with 0  <  'cdmg<n>'  < 1")
        self.ccf3 = FXLabel(vf1,"Yellow: Elements with  0.8 <=  'cdif<n>' (or 'cdim<n>' or 'cdip<n>')  < 1")
        self.ccf4 = FXLabel(vf1,"Green: Elements with 'cdif<n>' (or 'cdim<n>' or 'cdip<n>')  < 0.8")

        self.gb3 = FXGroupBox(self.switcher, 'Color Code Explanations',
                              FRAME_GROOVE|LAYOUT_FILL_X)        
        vf2 = FXVerticalFrame(self.gb3)
        self.ccf5 = FXLabel(vf2,"Red: Elements with 'cdif<n>' (or 'cdim<n>' or 'cdip<n>') >= 1")
        self.ccf6 = FXLabel(vf2,"Yellow: Elements with 0.8 <=  'cdif<n>' (or 'cdim<n>' or 'cdip<n>')  < 1")
        self.ccf7 = FXLabel(vf2,"Green: Elements with 'cdif<n>' (or 'cdim<n>' or 'cdip<n>')  < 0.8")

        self.switcher.setCurrent(1)
        
        FXMAPFUNC(self,SEL_COMMAND,self.ID_STEP,PlotConnectorsDB.onStepChange)
        
        from kernelAccess import session
        vp = session.viewports[session.currentViewportName]
        odbName = vp.odbDisplay.name
        self.currentOdbName = odbName
        self.previousOdbName = odbName
        self.setInitialValues()
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdReset(self,sender,sel,ptr):
        """
        Method to issue the reset command.
        """
        sendCommand('connVis.reset()')
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onStepChange(self,sender,sel,ptr):
        """
        Method to update step time combo box when step name changes.
        """
        self.damageCB.clearItems()
        self.stepTimeCB.clearItems()         
        self.conditionCB.clearItems()
        self.damageCB1.clearItems()
        self.damageCB2.clearItems()
        from kernelAccess import session
        vp = session.viewports[session.currentViewportName]
        odbName = vp.odbDisplay.name
        odb = session.odbs[odbName]
        stepName = self.form.stepNameKw.getValue()
        getAFXApp().beginWaitCursor()
        getAFXApp().forceRefresh()
        validHOList = self.form.validVariableList(stepName)
        getAFXApp().endWaitCursor()

        firstEFlag = 0
        firstIFlag = 0
        for variable in validHOList:
            self.damageCB.appendItem(variable)
            if variable.startswith('CDMG'):
                self.damageCB1.appendItem(variable)
                if not firstEFlag:
                    damEvolVar = variable
                    firstEFlag = 1
            elif variable.startswith('CDIM') or \
                 variable.startswith('CDIP') or \
                 variable.startswith('CDIF'):
                self.damageCB2.appendItem(variable)
                if not firstIFlag:
                    damInitVar = variable
                    firstIFlag = 1    
                
        if validHOList:
            self.form.damageVariableKw.setValue(validHOList[0])
        else:
            errMsg = 'There are no connector damage variables in the odb '\
                     'for the selected step %s.' %(stepName)
            showAFXErrorDialog(getAFXApp().getAFXMainWindow(), errMsg)
            return None

        if firstEFlag:
            self.form.damEvolVarKw.setValue(damEvolVar)
        if firstIFlag:
            self.form.damInitVarKw.setValue(damInitVar)

        if firstIFlag and firstEFlag:
            self.switcher.setCurrent(0)
        elif firstIFlag and not firstEFlag:
            self.switcher.setCurrent(1)
        else:
            self.switcher.setCurrent(0)
            
        frames = odb.steps[stepName].frames
        stepNumberList = []
        for frame in frames:
            stepNumberList.append(frame.frameValue)

        for stepNum in stepNumberList:
            stepNumStr = str(stepNum)
            self.stepTimeCB.appendItem(stepNumStr)

        if len(stepNumberList):
            self.form.stepTimeKw.setValue(stepNumberList[-1])

        self.conditionCBFunc()         
        
        return 1
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):
        """
        """
        from kernelAccess import session
        vp = session.viewports[session.currentViewportName]
        self.prevRegVP = session.currentViewportName
        vp.odbDisplay.registerQuery(self.onUpdate)
        session.registerInclusive(self.onUpdate1)
        AFXDataDialog.show(self)
     
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):
        """
        """
        from kernelAccess import session

        if self.prevRegVP not in session.viewports.keys():
            pass
        else:
            session.viewports[self.prevRegVP].odbDisplay.unregisterQuery(self.onUpdate)
            self.prevRegVP = session.currentViewportName
            
        session.unregisterQuery(self.onUpdate1)
        AFXDataDialog.hide(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onUpdate1(self):
        """
        Method to update plug-in when session changes.
        """
        
        from kernelAccess import session                
        
        if self.prevRegVP not in session.viewports.keys():
            pass
        else:
            session.viewports[self.prevRegVP].odbDisplay.unregisterQuery(self.onUpdate)
            self.prevRegVP = session.currentViewportName
        
        vp = session.viewports[session.currentViewportName]       
        vp.odbDisplay.registerQuery(self.onUpdate)
        self.onUpdate()
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
    #
    def onUpdate(self):
        """
        Method to update plug-in when viewport display changes.        
        """

        from kernelAccess import session
        vp = session.viewports[session.currentViewportName]
        # See if the odb has changed
        #        
        odbName = None
        try:
            odbName = vp.odbDisplay.name            
        except:
            pass

        if odbName == None or not odbName:
            # ODB name is None
            # So remove all the lists
            #        
            self.setDefaults()
            self.currentOdbName = None
            self.previousOdbName = None  
            self.form.odbTgt.setValue('')
            
            return 1        

        # There is some other change
        #
        self.currentOdbName = odbName        

        if self.currentOdbName != self.previousOdbName:
            # ODB has changed so update the list
            #           
            self.form.odbTgt.setValue(self.currentOdbName)
            self.setDefaults()
            self.setInitialValues()
            self.previousOdbName = self.currentOdbName
            sendCommand('connVis.reset()')
        return 1    

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def setDefaults(self):
        """
        Method to set the default values for keywords and combo boxes.
        """
        self.stepNameCB.clearItems()
        self.stepTimeCB.clearItems()
        self.damageCB.clearItems()
        self.conditionCB.clearItems()
        self.damageCB1.clearItems()
        self.damageCB2.clearItems()        
        self.form.stepNameKw.setValue('')
        self.form.stepTimeKw.setValue('')
        self.form.damageVariableKw.setValue('')
        self.form.damEvolVarKw.setValue('')
        self.form.damInitVarKw.setValue('')
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
    def setInitialValues(self):
        """
        Method to initialize the plug-in.
        """
        from kernelAccess import session
        vp = session.viewports[session.currentViewportName]
        try:            
            odb = session.odbs[vp.odbDisplay.name]
        except:        
            return
        
        stepNames = odb.steps.keys()
        self.stepNameCB.clearItems()
        for stepName in stepNames:
            self.stepNameCB.appendItem(stepName)

        if len(stepNames):                
            self.form.stepNameKw.setValue(stepNames[-1])
            self.stepTimeCB.clearItems()
            frames = odb.steps[stepNames[-1]].frames
            stepNumberList = []
            for frame in frames:
                stepNumberList.append(frame.frameValue)

            for stepNum in stepNumberList:
                stepNumStr = str(stepNum)
                self.stepTimeCB.appendItem(stepNumStr)

            if len(stepNumberList):
                self.form.stepTimeKw.setValue(stepNumberList[-1])

            getAFXApp().beginWaitCursor()
            getAFXApp().forceRefresh()
            validHOList = self.form.validVariableList(stepNames[-1])
            getAFXApp().endWaitCursor()

            firstEFlag = 0
            firstIFlag = 0
            for variable in validHOList:
                self.damageCB.appendItem(variable)
                if variable.startswith('CDMG'):
                    self.damageCB1.appendItem(variable)
                    if not firstEFlag:
                        damEvolVar = variable
                        firstEFlag = 1
                elif variable.startswith('CDIM') or \
                     variable.startswith('CDIP') or \
                     variable.startswith('CDIF'):
                    self.damageCB2.appendItem(variable)
                    if not firstIFlag:
                        damInitVar = variable
                        firstIFlag = 1
                    
            if validHOList:
                self.form.damageVariableKw.setValue(validHOList[0])
            else:        
                errMsg = 'There are no connector damage variables in the '\
                         'odb for the selected step %s.' %(stepNames[-1])
                showAFXErrorDialog(getAFXApp().getAFXMainWindow(), errMsg)
                return None

            if firstEFlag:
                self.form.damEvolVarKw.setValue(damEvolVar)
                
            if firstIFlag:
                self.form.damInitVarKw.setValue(damInitVar)            

            if firstIFlag and firstEFlag:
                self.switcher.setCurrent(0)
            elif firstIFlag and not firstEFlag:            
                self.switcher.setCurrent(1)
            else:
                self.switcher.setCurrent(0)
                
            self.conditionCBFunc()
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
    def conditionCBFunc(self):
        """
        Method to append the conditionCB and text fields
        """
        self.conditionCB.clearItems()
        self.conditionCB.appendItem('>=', GREATER_THAN.getId())
        self.conditionCB.appendItem('<=', LESSER_THAN.getId())
        self.conditionCB.appendItem('==', EQUAL_TO.getId())
        self.conditionCB.appendItem('between', WITHIN_RANGE.getId())

        self.form.damageFlagKw.setValue(GREATER_THAN.getId())
        self.form.firstValueKw.setValue('1.0')

        self.tf2.disable()
        
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Class definition
#
class PlotConnectorsForm(AFXForm):
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    def __init__(self, owner):

        # Construct the base class.
        #
        AFXForm.__init__(self, owner)        
        
        self.odbTgt = AFXStringTarget()
        # Command to ConnVis
        #             
        self.cmd = AFXGuiCommand(self,'connVis','connVis')
        self.stepNameKw = AFXStringKeyword(self.cmd, 'stepName', TRUE)
        self.stepTimeKw = AFXFloatKeyword(self.cmd, 'stepTime', TRUE)
        self.selectionKw = AFXSymConstKeyword(self.cmd, 'sel_button', TRUE,
                                              FIRST_BUTTON.getId())
        self.damageVariableKw = AFXStringKeyword(self.cmd, 'damageVariable',
                                                 FALSE)
        self.damageFlagKw = AFXSymConstKeyword(self.cmd, 'damageFlag')
        self.firstValueKw = AFXFloatKeyword(self.cmd, 'firstValue')
        self.secondValueKw = AFXFloatKeyword(self.cmd, 'secondValue',
                                             FALSE, 1.0)
        self.damEvolVarKw = AFXStringKeyword(self.cmd, 'damEvolVariable',
                                            FALSE)
        self.damInitVarKw = AFXStringKeyword(self.cmd, 'damInitVariable',
                                            FALSE)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def activate(self):
        """
        Reimplemented to check if an ODB is displayed in the viewport or not.
        """
        from kernelAccess import session
        vp = session.viewports[session.currentViewportName]

        if hasattr(vp.odbDisplay,'name'):
            if not vp.odbDisplay.name:
                errMsg = 'An output database must be displayed in the '\
                         'current viewport.'
                showAFXErrorDialog(getAFXApp().getAFXMainWindow(), errMsg)
                return None
            else:
                od = vp.odbDisplay
                odbName = od.name
                self.odbTgt.setValue(odbName)
                currVp = session.currentViewportName
                sendCommand("session.viewports['%s'].setValues\
                (displayedObject=session.odbs['%s'])" %(currVp, odbName))
        else:
            errMsg = 'An output database must be displayed in the current '\
                     'viewport.'
            showAFXErrorDialog(getAFXApp().getAFXMainWindow(), errMsg)
            return None
        
        stepNames = session.odbs[odbName].steps.keys()
        getAFXApp().beginWaitCursor()
        getAFXApp().forceRefresh()
        validFlag = 0
        for stepName in stepNames:
            validHOList = self.validVariableList(stepName)
            if validHOList:
                validFlag = 1
        getAFXApp().endWaitCursor()
        if not validFlag:
            errMsg = 'There are no connector damage variables in the odb.'
            showAFXErrorDialog(getAFXApp().getAFXMainWindow(), errMsg)
            return None
        
        return AFXForm.activate(self)  

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def deactivate(self):
        """
        Re-implemented deactivate to reset the plugin.
        """
        sendCommand('connVis.resetTrans()')
        return AFXForm.deactivate(self)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        # Show the dialog
        #
        self.firstDb = PlotConnectorsDB(self)        
        return self.firstDb

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):        
        """
        Method to check various input.
        """
        db = self.getCurrentDialog()        

        if not self.stepNameKw.getValue():
            errMsg = 'The step name must be provided.'
            showAFXErrorDialog(db, errMsg)
            return FALSE

        if self.stepTimeKw.getValue() == '':
            errMsg = 'The step time must be provided.'
            showAFXErrorDialog(db, errMsg)
            return FALSE
        
        selectionKw = self.selectionKw.getValue()
        if selectionKw == FIRST_BUTTON.getId():
            self.damageVariableKw.setRequired(TRUE)
            self.damageFlagKw.setRequired(TRUE)
            self.firstValueKw.setRequired(TRUE)
            self.secondValueKw.setRequired(TRUE)
            damageVariableKw = self.damageVariableKw.getValue()
            if not self.damageVariableKw.getValue():
                errMsg = 'A connector variable must be selected.'
                showAFXErrorDialog(db, errMsg)
                return FALSE
            conditionValue = self.damageFlagKw.getValue()
            firstValue = self.firstValueKw.getValue()
            secondValue = self.secondValueKw.getValue()
            if damageVariableKw.startswith('CFAILST') or \
               damageVariableKw.startswith('CDMG'):
                if firstValue > 1.0 or firstValue < 0.0:
                    errMsg = "Enter values between 0.0 and 1.0 in the text "\
                             "fields."
                    showAFXErrorDialog(db, errMsg)
                    db.tf1.setFocusToTextField()
                    return FALSE
                if conditionValue == LESSER_THAN.getId() and firstValue < 0.0:
                    errMsg = "Enter values greater than or equal to zero in "\
                             "the text field when using '<=' operator."
                    showAFXErrorDialog(db, errMsg)
                    db.tf1.setFocusToTextField()
                    return FALSE
                if conditionValue == GREATER_THAN.getId() and firstValue > 1.0:
                    errMsg = "Enter values lesser than or equal to '1' in "\
                             "the text field when using '>=' operator."
                    showAFXErrorDialog(db, errMsg)
                    db.tf1.setFocusToTextField()
                    return FALSE
                if conditionValue == WITHIN_RANGE.getId():
                    if secondValue > 1.0 or secondValue < 0.0:
                        errMsg = 'Enter values between 0.0 and 1.0 in the '\
                                 'text fields.'
                        showAFXErrorDialog(db, errMsg)
                        db.tf2.setFocusToTextField()
                        return FALSE
            self.damEvolVarKw.setRequired(FALSE)
            self.damInitVarKw.setRequired(FALSE)
        elif selectionKw == SECOND_BUTTON.getId():
            self.damEvolVarKw.setRequired(TRUE)
            self.damInitVarKw.setRequired(TRUE)
            if not self.damInitVarKw.getValue():
                errMsg = "A connector damage initiation variable must be "\
                         "selected."
                showAFXErrorDialog(db, errMsg)
                return FALSE
            self.damageVariableKw.setRequired(FALSE)
            self.damageFlagKw.setRequired(FALSE)
            self.firstValueKw.setRequired(FALSE)
            self.secondValueKw.setRequired(FALSE)
        return AFXForm.verifyKeywordValues(self)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
    def validVariableList(self, stepName):
        """
        Method to find out valid connector variables.
        """
        
        vp = session.viewports[session.currentViewportName]
        odbName = vp.odbDisplay.name
        odb = session.odbs[odbName]
        stepObj = odb.steps[stepName]
        validHOList = []
        #Updating Damage Variable list from selected step
        for historyRegion in stepObj.historyRegions.keys():
            if historyRegion.startswith('Element'):
                historyRegionObj = stepObj.historyRegions[historyRegion]
                for historyOutput in historyRegionObj.historyOutputs.keys():
                    if historyOutput.startswith('CFAILST') or \
                       historyOutput.startswith('CDMG') or \
                       historyOutput.startswith('CDIM') or \
                       historyOutput.startswith('CDIF') or \
                       historyOutput.startswith('CDIP'):
                        if historyOutput not in validHOList:
                            validHOList.append(historyOutput)
        return validHOList

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
helpUrlLocation = 'http://abaqus.custhelp.com/cgi-bin/abaqus.cfg/php/enduser/std_adp.php?p_faqid=3129'

majorNumber, minorNumber, updateNumber = getAFXApp().getBaseVersionNumbers()
supportedVersions = ['6.6','6.7', '6.8']
currentVersion = '%s.%s' %(majorNumber, minorNumber)
plotConnectorsDescription = "ABAQUS/CAE Plug-in for plotting the connectors "\
                            "of an ODB satisfying a given set of arguments. "\
                            "The arguments include step name, step time, "\
                            "connector damage variable, an operator (>=, <=, "\
                            "==, between) and a numerical value. The "\
                            "connectors satisying the arguments will be "\
                            "displayed in the viewport"

if currentVersion in supportedVersions:  
    toolset=getAFXApp().getAFXMainWindow().getPluginToolset()
    toolset.registerGuiMenuButton(
        object=PlotConnectorsForm(toolset),
        buttonText="Visualization|&Plot Failed Connectors...",
        messageId=AFXMode.ID_ACTIVATE,
        kernelInitString='import connVis\nfrom connVisConstants import *\n',
        applicableModules=["Visualization",],
        version="1.2-1",
        author="ABAQUS, Inc.",
        description=plotConnectorsDescription,
        helpUrl=helpUrlLocation)
else:
    mainWindow = getAFXApp().getAFXMainWindow()
    sv=''
    for version in supportedVersions:
        sv += 'V%s, '%version
    sv = sv[:-2]
    st = "***Warning: The Plot Connectors Plug-in will not be available for this session, it has not been certified for ABAQUS v%s.%s, it has been certified for %s ." %(majorNumber, minorNumber,sv)
    mainWindow.writeToMessageArea(st)
