from abaqusGui import *
from odbAccess import *

###########################################################################
# Class definition
###########################################################################

class StlForm(AFXForm):

    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
        AFXForm.__init__(self, owner)
        self.owner = owner

        self.cmd = AFXGuiCommand(mode=self, method='calculateStl',
                                 objectName='stlKernel')
        self.elset1Kw = AFXStringKeyword(command=self.cmd, name='elset1',
                                         isRequired=TRUE, defaultValue=None)
        self.elset2Kw = AFXStringKeyword(command=self.cmd, name='elset2',
                                         isRequired=TRUE, defaultValue=None)

    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        return StlDb(self)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        if (self.elset1Kw.getValue() == '' or self.elset2Kw.getValue() == ''):
            showAFXErrorDialog(self.getCurrentDialog(),
                               'You must select element sets for the\nincident and transmission sides.')
            return FALSE

        if self.elset1Kw.getValue() == self.elset2Kw.getValue():
            showAFXErrorDialog(self.getCurrentDialog(),
                               'You must select different element sets for the\nincident and transmission sides.')
            return FALSE
        
        return AFXForm.verifyKeywordValues(self)

###########################################################################
# Class definition
###########################################################################

class StlDb(AFXDataDialog):

    def __init__(self, form):
        self.form = form
        
        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form,
                               title='STL Calculation',
                               opts=DIALOG_ACTIONS_SEPARATOR|DATADIALOG_BAILOUT )
        getAFXAliasMap().setPrefix(self, 'STLDB')
        self.btnOk = self.appendActionButton(self.OK)
        self.btnOk.disable()
        self.btnCancel = self.appendActionButton(self.CANCEL)
        getAFXAliasMap().setName(self.btnOk, 'STLOk')
        getAFXAliasMap().setName(self.btnCancel, 'STLCancel')
        gb = FXGroupBox(p=self, text='Element Sets',
                        opts=FRAME_GROOVE|LAYOUT_FILL_X)
        va = AFXVerticalAligner(p=gb, opts=LAYOUT_FILL_X)
        self.elset1Cb = AFXComboBox(p=va, ncols=25, nvis=-1,
                                    text='Incident side element set:',
                                    tgt=form.elset1Kw, sel=0)
        getAFXAliasMap().setName(self.elset1Cb, 'ComboBox1')
        self.elset2Cb = AFXComboBox(p=va, ncols=25, nvis=-1,
                                    text='Transmitted side element set:',
                                    tgt=form.elset2Kw, sel=0)
        getAFXAliasMap().setName(self.elset2Cb, 'ComboBox2')
        self.onUpdate()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def populateCombos(self):
        """Method to populate the combo boxes"""
        
        setsToIgnore = [' ALL ELEMENTS', 'WarnElemDistorted']

        from kernelAccess import session
        vp = session.viewports[session.currentViewportName]
        odb = vp.displayedObject
        try:
            # Get assembly and instance sets
            #
            allSets = odb.rootAssembly.elementSets.keys()
            for instance in odb.rootAssembly.instances.values():
                for elsetName in instance.elementSets.keys():
                    allSets.append('%s.%s' % (instance.name, elsetName))
            allSets.sort()

            # Repopulate combos
            #
            self.elset1Cb.clearItems()
            self.elset2Cb.clearItems()
            for setName in allSets:
                if setName not in setsToIgnore:
                    self.elset1Cb.appendItem(text=setName)
                    self.elset2Cb.appendItem(text=setName)
            if self.form.elset1Kw.getValue() not in allSets:
                self.form.elset1Kw.setValue(None)
            if self.form.elset2Kw.getValue() not in allSets:
                self.form.elset2Kw.setValue(None)

            # Pre-select elsets if they exist (and there are no current values set)
            #
            if self.form.elset1Kw.getValue() == '':
                if 'INCIDENT_SIDE' in allSets:
                    self.form.elset1Kw.setValue('INCIDENT_SIDE')
            if self.form.elset2Kw.getValue() == '':
                if 'TRANSMITTED_SIDE' in allSets:
                    self.form.elset2Kw.setValue('TRANSMITTED_SIDE')
                
        except:
            self.form.elset1Kw.setValue(None)
            self.form.elset2Kw.setValue(None)
            self.elset1Cb.clearItems()
            self.elset2Cb.clearItems()
           
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):

        # Register query on odbDisplay, populate combo boxes and call show()
        #
        registerCurrentContext(self.onUpdate)
        self.populateCombos()
        AFXDataDialog.show(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):

        # Unregister query on odbDisplay
        #
        unregisterCurrentContext(self.onUpdate)
        AFXDataDialog.hide(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onUpdate(self):

        self.elset1Cb.clearItems()
        self.elset2Cb.clearItems()
        self.form.elset1Kw.setValue(None)
        self.form.elset2Kw.setValue(None)
        self.btnOk.disable()
        from kernelAccess import session
        if hasattr(session.viewports[session.currentViewportName].odbDisplay, 'name'):
            self.populateCombos()
            self.btnOk.enable()